#include <DoQuBook/PluginLoader.hpp>
#include <DoQuBook/Book.hpp>
#include <DoQuBook/DoQuBookApp.hpp>
#include <DoQuBook/RectoVersoRegistration.hpp>
#include <QPainter>
#include <DoQuBook/AttachmentServices.hpp>
#include <DoQuBook/Repository.hpp>
#include <DoQuBook/ApplicationContext.hpp>
#include <DoQuBook/Binarizer.hpp>

#include <QtGui>

using namespace DoQuBook;

QString orgName ("labri");

/// Must be the name of the target in CMake
QString appName ("Binarize");


class Binarize : public DoQuBook::DoQuBookApp
{
public :

    Binarize(QCoreApplication &app)
        : DoQuBook::DoQuBookApp(app, orgName, appName)
    {}

protected :

    virtual QList<ProgramParameter> paramsAndDescriptions() {
        QList<ProgramParameter> params;
        params << ProgramParameter ("input", false, "Input Image. Example : input.png");
        params << ProgramParameter ("binarizer", false, "Binarier ID. Example : labri.BinarizationOtsu2T.v1.0");
        params << ProgramParameter ("output", false, "Output Image FileName. Example : output.png");

        params << ProgramParameter ("list", false, "Set it to ALL, Lists all Binarization methods and exits");
        return params;
    }

    virtual void runApp(QMap<QString, QString> args)
    {
        ApplicationContext::instance()->setUserName("test@digidoc.com");
        PluginLoader* _loader = PluginLoader::instance();
        _loader->loadPlugins();

        if(args.contains("list")) {
            QStringList binarizers = _loader->listPlugins<Binarizer>();
            foreach(QString binName, binarizers) {
                qDebug() << binName;
            }
        } else {
            if(!args.contains("input")
                    || !args.contains("output")
                    || !args.contains("binarizer")) {

                QString errorMsg ("Please provide the folowing parameters : input, binarizer and output (use -help for more details)");
                qCritical() << errorMsg;

            } else {

                BinarizerPtr bin = _loader->getPlugin<Binarizer>(args["binarizer"]);
                QImage in (args["input"]);
                QImage out = bin->binarize(in);
//                out.invertPixels();
                out.save(args["output"]);
            }
        }
    }

};


int main(int argc, char *argv[])
{
    QCoreApplication app(argc, argv);

    Binarize test (app);

    return test.exec();
}
