#ifndef BINARIZERPLUGINVIEW_HPP
#define BINARIZERPLUGINVIEW_HPP

#include <QObject>
#include <QtWebKit/QtWebKit>
#include <QWebView>
#include <QWebFrame>

//Have a look at
// http://daniel-albuschat.blogspot.fr/2008/12/embedding-qt-widgets-into-qtwebkit.html


#include <QImageReader>
class ByteArrayReply : public QNetworkReply {

    Q_OBJECT

public:
    ByteArrayReply(const QByteArray& ba, const char *mimeType, QObject *parent = NULL);

    ~ByteArrayReply();

    //reimplemented
    void abort();
    void close();
    qint64 readData(char *data, qint64 len);
    void setReadBufferSize(qint64 size);
    qint64 bytesAvailable() const;
    bool open(OpenMode mode);
    qint64 size() const;


private slots:
    void dataReady();

private:
    /** Icon data */
    QBuffer dataBuffer_;
};

class BinarizerPluginController;

class NAM : public QNetworkAccessManager {

    Q_OBJECT

public :
    explicit NAM(BinarizerPluginController *view)
        : _view(view)
    {}

protected:
    virtual QNetworkReply * createRequest(Operation op,
                                          const QNetworkRequest & req,
                                          QIODevice * outgoingData = 0) {
        if (req.url().scheme() == "convert") {
            QByteArray bytes;
            loadIcon(bytes, req.url().path(), "png");
            return new ByteArrayReply(bytes, "image/xpm");
        }

        return QNetworkAccessManager::createRequest(op, req, outgoingData);
    }

    void loadIcon(QByteArray& bytes, const QString & path, const char* format) ;
private :
    BinarizerPluginController *_view;
};



/////////


class BinarizerPluginController : public QWebView
{
    Q_OBJECT
public:
    explicit BinarizerPluginController(QWidget *parent);

    QImage getImage();
    QImage getResult();

signals:
    void hasFinished();
    void formValidated();
    
public slots:
    // Html call back
    void openFile();
    void run();
    void checkForm();
    void setPlugin(QString pluginName);

private slots:
    void populateJavaScriptWindowObject();

protected :
    QString getAttr(const QString &id, const QString &attrName);
    void setAttr(const QString &id, const QString &attrName, const QString &val);
    void setInner(const QString &id, const QString &val);
    void appendInner(const QString &id, const QString &val);

protected:
    NAM *_accessManager;
    QString _pluginName;
    int _elapseMmilliSecond;
    QImage _result;
    
};

#endif // BINARIZERPLUGINVIEW_HPP
