#include "ClassBuilder.hpp"

#include <QDebug>

ClassBuilder::ClassBuilder()
{
}


void ClassBuilder::setClassName(const QString &name)
{
    _name = name;
}

void ClassBuilder::addBasicField(const Field &basicField)
{
    _fields.append(basicField);
}

void ClassBuilder::toFile()
{
    writeDeclarationFile();

    if(hasDeclarationAndImplementationFiles()) {
        writeImplementationFile();
    }
}

void ClassBuilder::addListField(const Field &listField)
{
    _listFields.append(listField);
}


QString ClassBuilder::getTemplate()
{
    QString templ;
    QFile file (classTemplate());
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qCritical() << "Could not open file " << classTemplate();
        return templ;
    }

    QTextStream in(&file);
    templ.append(in.readAll());
    return templ.trimmed();
}

QString ClassBuilder::getAnnotationsTemplate()
{
    QString templ;
    QFile file (":/java/entity/annotations");
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qCritical() << "Could not open file :/entity/annotations";
        return templ;
    }

    QTextStream in(&file);
    templ.append(in.readAll());
    return templ.trimmed();
}

QString ClassBuilder::getConstructorTemplate()
{
    QString templ = openTemplate(":/"+lang()+"/constructor");
    templ.replace("${CONSTRUCTOR_DECLARATION}", getConstructorDeclarationTemplate());
    templ.replace("${CONSTRUCTOR_IMPLEMENTATION}", getConstructorImplementationTemplate());

    QString inits;
    foreach (Field f, _fields) {
        if(f.isList()) {
            QString listInitTemplate = getInitArrayListTemplate();
            listInitTemplate.replace("${NAME}", f.name());
            listInitTemplate.replace("${TYPE}", f.type());
            inits.append(listInitTemplate);
        }
    }
    templ.replace("${LIST_INIT}", inits);

    return templ;
}

QString ClassBuilder::getInitArrayListTemplate()
{
    QString templ;
    QFile file (":/java/init_arraylist");
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qCritical() << "Could not open file :/java/init_arraylist";
        return templ;
    }

    QTextStream in(&file);
    templ.append(in.readAll());
    return templ.trimmed();
}

QString ClassBuilder::getFieldDeclarationTemplate()
{
    return openTemplate(":/"+lang()+"/basicfield_declaration");
}

QString ClassBuilder::getBasicFieldAnnotation()
{
    return openTemplate(":/"+lang()+"/basicfield_annotation").trimmed();
}

QString ClassBuilder::getIdFieldAnnotation()
{
    return openTemplate(":/"+lang()+"/idfield_annotation").trimmed();
}

QString ClassBuilder::getIntIdFieldAnnotation()
{
    return openTemplate(":/"+lang()+"/intidfield_annotation").trimmed();
}

QString ClassBuilder::getListFieldAnnotations()
{
    return openTemplate(":/"+lang()+"/listfield_annotations").trimmed();
}

QString ClassBuilder::getGetterTemplate()
{
    return openTemplate(":/"+lang()+"/getter");
}

QString ClassBuilder::getSetterTemplate()
{
    return openTemplate(":/"+lang()+"/setter");
}

QString ClassBuilder::getGetterDeclarationTemplate()
{
    return openTemplate(":/"+lang()+"/getter_declaration").trimmed();
}

QString ClassBuilder::getGetterImplementationTemplate()
{
    return openTemplate(":/"+lang()+"/getter_implementation");
}

QString ClassBuilder::getSetterDeclarationTemplate()
{
    return openTemplate(":/"+lang()+"/setter_declaration").trimmed();
}


QString ClassBuilder::getSetterImplementationTemplate()
{
    return openTemplate(":/"+lang()+"/setter_implementation");
}


QString ClassBuilder::openTemplate(const QString &templateName)
{
    QString templ;
    QFile file (templateName);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qCritical() << "Could not open file "+templateName;
        return templ;
    }

    QTextStream in(&file);
    templ.append(in.readAll());
    return templ;
}

QString ClassBuilder::getConstructorImplementationTemplate()
{
    return openTemplate(":/"+lang()+"/constructor_implementation");
}

QString ClassBuilder::getConstructorDeclarationTemplate()
{
    return openTemplate(":/"+lang()+"/constructor_declaration").trimmed();
}

QString ClassBuilder::getImplementationFileTemplate()
{
    return openTemplate(":/"+lang()+"/entity/implementation_file").trimmed();
}

QString ClassBuilder::classNameUpper()
{
    return _name.toUpper();
}

QString ClassBuilder::getFieldType(const Field &f)
{
    if(f.isList())
        return "List<"+f.type()+">";
    else
        return f.type();
}

QString ClassBuilder::getFieldAnnotations(const Field &f)
{
    if(f.isList()) {
        return getListFieldAnnotations();
    } else {
        if(f.isId()) {
            if(f.type() == "int")
                return getIntIdFieldAnnotation();
            else
                return getIdFieldAnnotation();
        } else {
            return getBasicFieldAnnotation();
        }
    }
}

void ClassBuilder::moreTemplatesOnDeclarationFile(QString & /*templ*/)
{
}

void ClassBuilder::moreTemplatesOnImplementationFile(QString & /*templ*/)
{
}

QString ClassBuilder::fieldDeclaration(const Field &f)
{
    QString type = getFieldType(f);
    QString fieldDeclarationTempl = getFieldDeclarationTemplate();
    QString fieldAnnotations = getFieldAnnotations(f);

    fieldDeclarationTempl.replace("${FIELD_ANNOTATIONS}", fieldAnnotations);
    fieldDeclarationTempl.replace("${TYPE}", type);
    fieldDeclarationTempl.replace("${NAME}", f.name());
    QString nameUp = f.name().at(0).toUpper()+f.name().mid(1);
    fieldDeclarationTempl.replace("${NAME_FIRST_LETTER_UP}", nameUp);
    return fieldDeclarationTempl;
}

QString ClassBuilder::fieldDeclarationGettersAndSetters(const Field &f)
{
    QString getAndSet;
    QString getterTempl = getGetterTemplate();
    getterTempl.replace("${GETTER_DECLARATION}", getGetterDeclarationTemplate());
    getterTempl.replace("${GETTER_IMPLEMENTATION}", getGetterImplementationTemplate());

    getterTempl.replace("${TYPE}", getFieldType(f));

    getterTempl.replace("${NAME}", f.name());

    QString setterTempl = getSetterTemplate();
    setterTempl.replace("${SETTER_DECLARATION}", getSetterDeclarationTemplate());
    setterTempl.replace("${SETTER_IMPLEMENTATION}", getSetterImplementationTemplate());

    setterTempl.replace("${TYPE}", getFieldType(f));

    setterTempl.replace("${NAME}", f.name());
    getAndSet.append(getterTempl);
    getAndSet.append(setterTempl);
    QString nameUp = f.name().at(0).toUpper()+f.name().mid(1);
    getAndSet.replace("${NAME_FIRST_LETTER_UP}", nameUp);
    return getAndSet;
}

QString ClassBuilder::fieldImplementationGettersAndSetters(const Field &f)
{
    QString getAndSet;
    QString getterImplTempl = getGetterImplementationTemplate();
    getterImplTempl.replace("${TYPE}", getFieldType(f));
    getterImplTempl.replace("${NAME}", f.name());
    QString nameUp = f.name().at(0).toUpper()+f.name().mid(1);
    getterImplTempl.replace("${NAME_FIRST_LETTER_UP}", nameUp);

    QString setterImplTempl = getSetterImplementationTemplate();
    setterImplTempl.replace("${TYPE}", getFieldType(f));
    setterImplTempl.replace("${NAME}", f.name());
    setterImplTempl.replace("${NAME_FIRST_LETTER_UP}", nameUp);

    getAndSet.append(getterImplTempl);
    getAndSet.append(setterImplTempl);
    return getAndSet;
}

void ClassBuilder::writeImplementationFile()
{
    QFile * impl = implementationFile();
    if (!impl->open(QIODevice::WriteOnly | QIODevice::Text))
        return;

    QTextStream out2(impl);
    QString implTemplate = getImplementationFileTemplate();
    QString gettersAndsettersImpl;
    foreach (Field f, _fields) {
        gettersAndsettersImpl.append(fieldImplementationGettersAndSetters(f));
    }
    gettersAndsettersImpl.replace("${CLASS_NAME}", _name);
    implTemplate.replace("${GETTERS_AND_SETTERS}", gettersAndsettersImpl);

    QString constructorImpl = getConstructorImplementationTemplate();
    implTemplate.replace("${CONSTRUCTOR}", constructorImpl);
    implTemplate.replace("${CLASS_NAME}", _name);

    moreTemplatesOnImplementationFile(implTemplate);
    out2 << implTemplate;
}

void ClassBuilder::writeDeclarationFile()
{
    QFile * file = declarationFile();
    if (!file->open(QIODevice::WriteOnly | QIODevice::Text))
        return;

    QTextStream out(file);
    QString templ = getTemplate();
    templ.replace("${IMPORTS}", imports());
    templ.replace("${ANNOTATIONS}", getAnnotationsTemplate());
    templ.replace("${CONSTRUCTOR}", getConstructorTemplate());

    templ.replace("${CLASS_NAME}", _name);
    templ.replace("${CLASS_NAME_UPPER}", classNameUpper());
    templ.replace("${LOWER_CLASS_NAME}",_name.toLower());

    QString fields;
    foreach (Field f, _fields) {
        fields.append(fieldDeclaration(f));
    }
    templ.replace("${FIELDS}", fields);

    QString getsAndSets;
    foreach (Field f, _fields) {
           getsAndSets.append(fieldDeclarationGettersAndSetters(f));
    }
    templ.replace("${GETTERS_AND_SETTERS}", getsAndSets);

    moreTemplatesOnDeclarationFile(templ);
    out << templ;
}
