#include "CppClassBuilder.hpp"

CppClassBuilder::CppClassBuilder()
{
}

QString CppClassBuilder::classTemplate()
{
    return ":/cpp/entity/class";
}

QString CppClassBuilder::imports()
{
    QString t;
    t.append("#include <DoQuBook/DoQuBook_global.hpp> \n");
    t.append("#include <DoQuBook/SimpleMarshallable.hpp> \n");
    t.append("#include <DoQuBook/Exceptions.hpp> \n");
    t.append("#include <QString> \n");
    t.append("#include <QList> \n");
    t.append("#include <QObject> \n");
    t.append("#include <QSharedPointer> \n");

    foreach(Field f, _fields) {
        if (f.isMarshallable())
        {
            QString marshallableImport = "#include <DoQuBook/${TYPE}.hpp> \n";
            t.append(marshallableImport.replace("${TYPE}", f.type()));
        }
    }

    return t;
}

bool CppClassBuilder::hasDeclarationAndImplementationFiles()
{
    return true;
}

QFile * CppClassBuilder::declarationFile()
{
    QFile * f = new QFile(_name+".hpp");
    return f;
}

QFile * CppClassBuilder::implementationFile()
{
    QFile * f = new QFile (_name+".cpp");
    return f;
}

QString CppClassBuilder::getterTemplate()
{
    return ":/cpp/getter";
}

QString CppClassBuilder::getFieldType(const Field &f)
{
    if(f.isList() && f.isMarshallable())
        return "QList<"+f.type()+"Ptr>";
    else if(f.isList())
        return "QList<"+f.type()+">";
    else
        return f.type();
}

QString CppClassBuilder::getFieldAnnotations(const Field &f)
{
    QString s = ClassBuilder::getFieldAnnotations(f);
    if(f.isMarshallable())
        s.replace("${LIST_MACRO}", "MLIST");
    else
        s.replace("${LIST_MACRO}", "QTLIST");

    return s;
}

void CppClassBuilder::moreTemplatesOnDeclarationFile(QString &templ)
{
    QString activateList = "MARSHALLABLE_ACTIV_${N}LIST(";
    int nb = 0;
    bool firstList = true;
    for(int i = 0; i < _fields.size(); i++) {
        if(_fields[i].isList()) {
            if(firstList) {
                activateList.append(_fields[i].name()+", _"+_fields[i].name());
                firstList = false;
            } else
                activateList.append(","+_fields[i].name()+", _"+_fields[i].name());
            nb++;
        }
    }

    activateList.replace("${N}", QString::number(nb));
    activateList.append(");");
    templ.replace("${ACTIVATE_LIST}", activateList);
}

void CppClassBuilder::moreTemplatesOnImplementationFile(QString &templ)
{
}
