#include "JavaInterfaceServiceBuilder.hpp"


CRUDMethods::CRUDMethods(bool create, bool read, bool update, bool del)
    : _create (create)
    , _read (read)
    , _update (update)
    , _del (del)
{
}

bool CRUDMethods::create()
{
    return _create;
}

bool CRUDMethods::read()
{
    return _read;
}

bool CRUDMethods::update()
{
    return _update;
}

bool CRUDMethods::del()
{
    return _del;
}

Method::Method(const QString &name)
    : _name (name)
{
}

void Method::addParam(const Field &f)
{
    _params.append(f);
}

QList<Field> Method::getParams()
{
    return _params;
}

JavaInterfaceServiceBuilder::JavaInterfaceServiceBuilder()
    : _crudmethods (false, false, false, false)
{
}

void JavaInterfaceServiceBuilder::toFile()
{
    QFile &file (_name+".java");
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
        return;

    QTextStream out(&file);

    QString interface;
    interface += "package doqubook.services; \n"; //package
    interface += "import java.util.List; \n"; //

    interface += "import doqubook.entities."+_entityName+"; \n" ; // entity

    interface += "public interface "+_name+" { \n";

    if(_crudmethods.create())
        interface += "\t public int add ("+_entityName+" b); \n" ; // add method
    if(_crudmethods.read()) {
        interface += "\t public "+_entityName+" get (Integer id); \n" ;
        interface += "\t public List<"+_entityName+"> getAll (); \n";
    }
    if(_crudmethods.update())
        interface += "\t public "+_entityName+" update ("+_entityName+" b) \n";
    if(_crudmethods.del())
        interface += "\t public boolean delete (Integer id); \n";

    interface += "} \n" ;

    out << interface;
}

void JavaInterfaceServiceBuilder::setName(const QString &name)
{
    _name = name;
}

void JavaInterfaceServiceBuilder::setCrudMethods(const CRUDMethods &crudmethods)
{
    _crudmethods = crudmethods;
}

void JavaInterfaceServiceBuilder::setEntity(const QString &name)
{
    _entityName = name;
}


