#include <DoQuBook/PluginLoader.hpp>
#include <DoQuBook/Book.hpp>
#include <DoQuBook/DoQuBookApp.hpp>
#include <DoQuBook/RectoVersoRegistration.hpp>
#include <QPainter>
#include <DoQuBook/AttachmentServices.hpp>
#include <DoQuBook/ApplicationContext.hpp>
#include <DoQuBook/Binarizer.hpp>
#include <QDebug>

#include <QtGui>
#include <QtXml/QtXml>
#include "JavaClassBuilder.hpp"
#include "CppClassBuilder.hpp"
#include "JavaInterfaceServiceBuilder.hpp"

using namespace DoQuBook;

QString orgName ("labri");

/// Must be the name of the target in CMake
QString appName ("DIGIDOCTest");


class DIGIDOCTest : public DoQuBook::DoQuBookApp
{
public :
    static const int HEIGHT = 600;
    static const int WIDTH = 800;
    static const int NB_IMAGE = 3;

    DIGIDOCTest(QCoreApplication &app)
        : DoQuBook::DoQuBookApp(app, orgName, appName)

    {}



protected :

    virtual QMap<QString, QString> paramsAndDescriptions()
    {
        QMap<QString, QString> optionsAndDesk;
        optionsAndDesk["xml"] = "XML file (ex : Personne.xml";
        optionsAndDesk["out"] = "OUT file (ex : Personne.java)";
        optionsAndDesk["imports"] = "IMPORTS file (ex : imports.txt)";

        return optionsAndDesk;
    }

    virtual void run(QMap<QString, QString> args)
    {
        QString xmlfile = args["xml"];
        QDomDocument doc("mydocument");
        QFile file(xmlfile);
        if (!file.open(QIODevice::ReadOnly)) {
            qCritical() << "could not openFile";
            return;
        }

        if (!doc.setContent(&file))
        {
            file.close();
            qCritical() << "Error on XML";
            return;
        }
        file.close();

        JavaClassBuilder builder (args["imports"]);
        CppClassBuilder cppbuilder;
        QDomElement docElem = doc.documentElement();
        QDomElement entity = docElem.elementsByTagName("entity").at(0).toElement();
        QString entityName = entity.attributes().namedItem("name").nodeValue();
        builder.setClassName(entityName);
        cppbuilder.setClassName(entityName);

        QDomNodeList fields = entity.elementsByTagName("field");
        for(int i = 0; i < fields.count(); i++) {
            QString type = fields.at(i).attributes().namedItem("type").nodeValue();
            QString name = fields.at(i).attributes().namedItem("name").nodeValue();
            bool isId = fields.at(i).attributes().namedItem("id").nodeValue() == "true";

            Field f (type, name);
            f.setId(isId);
            builder.addBasicField(f);
            cppbuilder.addBasicField(f);
        }


        QDomNodeList listfields = entity.elementsByTagName("list");
        for(int i = 0; i < listfields.count(); i++) {
            QString type = listfields.at(i).attributes().namedItem("type").nodeValue();
            QString name = listfields.at(i).attributes().namedItem("name").nodeValue();
            bool isMarshallable = listfields.at(i).attributes().namedItem("marshallable").nodeValue() == "true";
            Field f (type, name);
            f.setIsList(true);
            f.setIsMarshallable(isMarshallable);
            builder.addBasicField(f);
            cppbuilder.addBasicField(f);
        }

        qDebug() << "TOTO";
        JavaInterfaceServiceBuilder builderService;
        QDomElement service = docElem.elementsByTagName("service").at(0).toElement();
        QString name = service.attributes().namedItem("name").nodeValue();
        builderService.setName(name);
        QString entityname = service.attributes().namedItem("entityname").nodeValue();
        builderService.setEntity(entityname);

        QDomElement crudmethods = service.elementsByTagName("crudmethods").at(0).toElement();
        bool create = crudmethods.attributes().namedItem("create").nodeValue() == "true";
        bool read = crudmethods.attributes().namedItem("read").nodeValue() == "true";
        bool update = crudmethods.attributes().namedItem("update").nodeValue() == "true";
        bool del = crudmethods.attributes().namedItem("delete").nodeValue() == "true";
        CRUDMethods crudmeths (create, read, update, del);
        builderService.setCrudMethods(crudmeths);
        qDebug() << "TOTO" << create;


        builder.toFile();

        cppbuilder.toFile();


        QFile outf("Service.java");
        builderService.toFile(outf);

    }
};


int main(int argc, char *argv[])
{
    QCoreApplication app(argc, argv);

    DIGIDOCTest test (app);

    return test.exec();
}
