#include <DoQuBook/PluginLoader.hpp>
#include <DoQuBook/Book.hpp>
#include <DoQuBook/DoQuBookApp.hpp>
#include <DoQuBook/RectoVersoRegistration.hpp>
#include <QPainter>
#include <DoQuBook/AttachmentServices.hpp>
#include <DoQuBook/Repository.hpp>
#include <DoQuBook/ApplicationContext.hpp>
#include <DoQuBook/Binarizer.hpp>

#include <QtGui>

using namespace DoQuBook;

QString orgName ("labri");

/// Must be the name of the target in CMake
QString appName ("Measure");


class Measure : public DoQuBook::DoQuBookApp
{
public :

    Measure(QCoreApplication &app)
        : DoQuBook::DoQuBookApp(app, orgName, appName)
    {}

protected :

    virtual QList<ProgramParameter> paramsAndDescriptions() {
        QList<ProgramParameter> params;
        params << ProgramParameter ("input", false, "Input Image. Example : input.png");
        params << ProgramParameter ("measurer", false, "Binarier ID. Example : labri.SampleMeasurer.v1.0");

        params << ProgramParameter ("list", false, "Set it to ALL, Lists all Measurer plugins and exits");
        return params;
    }

    virtual void runApp(QMap<QString, QString> args)
    {
        ApplicationContext::instance()->setUserName("test@digidoc.com");
        PluginLoader* _loader = PluginLoader::instance();
        _loader->loadPlugins();

        if(args.contains("list")) {
            QStringList measurers = _loader->listPlugins<Measurer>();
            foreach(QString binName, measurers) {
                qDebug() << binName;
            }
        } else {
            if(!args.contains("input")
                    || !args.contains("measurer")) {

                QString errorMsg ("Please provide the folowing parameters : input, binarizer and output (use -help for more details)");
                qCritical() << errorMsg;

            } else {

                MeasurerPtr measurer = _loader->getPlugin<Measurer>(args["measurer"]);
                QImage in (args["input"]);
                QList<QImage> images;
                images << in;
                QMap<QString, double> measures = measurer->measure(images);
                QMapIterator<QString, double> it(measures);
                while(it.hasNext()) {
                    it.next();
                    qDebug() << it.key() << " : " << it.value();
                }
            }
        }
    }

};


int main(int argc, char *argv[])
{
    QCoreApplication app(argc, argv);

    Measure test (app);

    return test.exec();
}
