#include "MainWindow.hpp"

#include <DoQuBook/Binarizer.hpp>
#include <DoQuBook/DegradationModel.hpp>
#include "PluginParameterConfigurator.hpp"


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent)
  , _layout (NULL)
{
    _main = new QWidget;
    _mainVBoxLayout = new QVBoxLayout;


    QPushButton *open = new QPushButton ("Open");
    connect(open, SIGNAL(clicked()),
            this, SLOT(openImages()));
    _mainVBoxLayout->addWidget(open);


    _rightLabel = new QLabel;
    _leftLabel = new QLabel;
    _rightLabel->setText("Left");
    _leftLabel->setText("Right");

    _images = new QWidget;


    _scrollArea = new QScrollArea ();
    _scrollArea->setWidgetResizable(true);
    _scrollArea->setWidget(_images);

    _scrollArea->setBackgroundRole(QPalette::Dark);
    _mainVBoxLayout->addWidget(_scrollArea);
    _main->setLayout(_mainVBoxLayout);

    box = new QComboBox;
    box->addItem("...");
    connect(box, SIGNAL(currentIndexChanged(QString)), this, SLOT(pluginNameSelected(QString)));

    _mainVBoxLayout->addWidget(box);
    QPushButton *runButton = new QPushButton("Run !");
    connect(runButton, SIGNAL(clicked()),
            this, SLOT(run()));

    _mainVBoxLayout->addWidget(runButton);

    _images->show();
    setCentralWidget(_main);
}

void MainWindow::setPluginLoader (DoQuBook::PluginLoader *loader)
{
    _loader = loader;
    foreach(QString s , _loader->listPlugins<Binarizer>()) {
        box->addItem(s);
    }

    foreach(QString s , _loader->listPlugins<DegradationModel>()) {
        box->addItem(s);
    }
}

void MainWindow::pluginNameSelected(const QString &s)
{
    if(s!="...") {
        std::tr1::shared_ptr<DoQuBook::AlgorithmOnTask> plugin = _loader->getPlugin< DoQuBook::AlgorithmOnTask >(s);
        qDebug() << plugin->algorithmName();

        PluginParameterConfigurator *pluginConfigurator = new PluginParameterConfigurator(this);

        connect(pluginConfigurator, SIGNAL(pluginConfigured(std::tr1::shared_ptr<DoQuBook::AlgorithmOnTask>,QVariantMap)),
                _runner, SLOT(pluginConfigured(std::tr1::shared_ptr<DoQuBook::AlgorithmOnTask>,QVariantMap)));

        pluginConfigurator->setWindowModality (Qt::WindowModal);
        pluginConfigurator->pluginSelected(plugin);
        pluginConfigurator->show();
    }

}

void MainWindow::setPluginRunner(PluginRunner *runner)
{
    _runner = runner;
}


void MainWindow::run()
{
    if(_layout) {
        foreach(QObject * o, _layout->children()) {
            QWidget *w = (QWidget*)o;
            w->hide();
            _layout->removeWidget(w);
            delete w;
        }
        delete _layout;
    }

    _layout = new QGridLayout ();
    _layout->addWidget(_leftLabel, 0, 0);
    _layout->addWidget(_rightLabel, 0, 1);

    _images->setLayout(_layout);



    foreach (QString f, _fileNames) {
        QImage i (f);
        QImage res = _runner->run(i);
        QLabel *l = new QLabel;
        l->setPixmap (QPixmap::fromImage(i.scaled(800,800,Qt::KeepAspectRatio)));
        QLabel *r = new QLabel;
        r->setPixmap (QPixmap::fromImage(res.scaled(800,800,Qt::KeepAspectRatio)));

        _layout->addWidget(l);
        _layout->addWidget(r);
    }
}

void MainWindow::openImages()
{
    QFileDialog dialog(this);
    dialog.setFileMode(QFileDialog::ExistingFiles);
    dialog.setNameFilter(tr("Images (*.png)"));
    //dialog.setViewMode(QFileDialog::Detail);

    if (dialog.exec())
        _fileNames = dialog.selectedFiles();


}
