#include "PluginParameterConfigurator.hpp"



ImageParameterWidget::ImageParameterWidget(QWidget *parent)
    : QWidget (parent)
    , _browse ("Browse")
{
    _layout.addWidget(&_path);
    _layout.addWidget(&_browse);
    connect(&_browse, SIGNAL(clicked()),
            this, SLOT(browse()));
    setLayout(&_layout);
}

void ImageParameterWidget::browse()
{
    QFileDialog dialog(this);
    dialog.setFileMode(QFileDialog::AnyFile);
    dialog.setNameFilter(tr("Images (*.png)"));
    //dialog.setViewMode(QFileDialog::Detail);

    QStringList fileNames ;
    if (dialog.exec())
        fileNames = dialog.selectedFiles();
    _realPath = fileNames.at(0);

    _path.setText(_realPath);
}

QImage ImageParameterWidget::selectedImage()
{
    return QImage (_path.text());
}

PluginParameterConfigurator::PluginParameterConfigurator(QWidget *parent) :
    QDialog(parent)
  , _layout (NULL)
  , _okButton (NULL)
  , _paramsWidget (NULL)
  , _paramsLayout (NULL)
  , title (NULL)
{
    hide();
    _layout = new QVBoxLayout;
    setLayout(_layout);

    title = new QLabel("Parameters for ");
    _layout->addWidget(title);
    _paramsLayout = new QVBoxLayout;
    _paramsWidget = new QWidget;
    _paramsWidget->setLayout(_paramsLayout);

    _layout->addWidget(_paramsWidget);

    _okButton = new QPushButton("Ok");
    connect(_okButton, SIGNAL(clicked()), this, SLOT(configured()));
    _layout->addWidget(_okButton);

}

void PluginParameterConfigurator::pluginSelected(std::tr1::shared_ptr<DoQuBook::AlgorithmOnTask> plugin)
{
    _plugin = plugin;
    setWindowTitle(plugin->algorithmName());
    title->setText(QString("Parameters for ")+plugin->algorithmName()+" :");

    foreach(DoQuBook::ExecutorParameterPtr param, plugin->parameters()) {
        QHBoxLayout *layout = new QHBoxLayout(this);
        QLabel *paramName = new QLabel (param->getName()+" ["+param->getParameterType()+"] :", this);
        QWidget *paramValue = NULL;
        if(param->getParameterType()=="String"
                || param->getParameterType()=="Double"
                || param->getParameterType()=="Integer" ) {
            paramValue = new QLineEdit (this);
        } else if(param->getParameterType()=="Boolean") {
            paramValue = new QComboBox (this);
            ((QComboBox*)paramValue)->addItem("true");
            ((QComboBox*)paramValue)->addItem("false");
        } else if (param->getParameterType() == "Image") {
            paramValue = new ImageParameterWidget (this);
        }

        if(paramValue != NULL) {
            paramsName[paramValue] = param->getName();
            // connect(paramValue, SIGNAL(textChanged(QString)),
            //        this, SLOT(someParamChanged(QString)));

            layout->addWidget(paramName);
            layout->addWidget(paramValue);

            //        if(param->getRequired())
            //            layout->addWidget(new QLabel("*"));

            qDebug() << param->getName();

            QWidget *dummy = new QWidget (this);
            dummy->setLayout(layout);
            _paramsLayout->addWidget(dummy);
        } else
            qCritical() << param->getParameterType() << "not supported";
    }

    show();
}

void PluginParameterConfigurator::configured()
{
    QVariantMap params;
    foreach(QWidget *line, paramsName.keys()) {
        QLineEdit *l = dynamic_cast<QLineEdit*>(line);
        if(l) {
            qDebug() << "Line" << l;
            QString paramName = paramsName[line];
            qDebug() << paramName;
            params [paramName] = l->text();
        } else {
            QComboBox *c = dynamic_cast<QComboBox*>(line);
            if(c)
                params [paramsName[line]] = c->currentText();
            else {
                ImageParameterWidget *i = dynamic_cast<ImageParameterWidget*>(line);
                params [paramsName[line]] = i->selectedImage();
            }
        }
    }

    emit pluginConfigured(_plugin, params);
    close();
}

