#include <DoQuBook/PluginLoader.hpp>
#include <DoQuBook/Book.hpp>
#include <DoQuBook/DoQuBookApp.hpp>
#include <DoQuBook/RectoVersoRegistration.hpp>
#include <QPainter>
#include <DoQuBook/AttachmentServices.hpp>
#include <DoQuBook/Repository.hpp>
#include <DoQuBook/ApplicationContext.hpp>
#include <DoQuBook/Binarizer.hpp>

#include <QtGui>

using namespace DoQuBook;

QString orgName ("labri");

/// Must be the name of the target in CMake
QString appName ("SampleDoQuBookApp");


class SampleDoQuBookApp : public DoQuBook::DoQuBookApp
{
public :

    SampleDoQuBookApp(QCoreApplication &app)
        : DoQuBook::DoQuBookApp(app, orgName, appName)
        , mainWin(NULL)
        , mainLayout(NULL)
        , scrollarea(NULL)
        , vLayout (NULL)
        , scrollArea (NULL)
        , widget (NULL)
        , i(0), j(0)
    {
        createGui();
        mainWin->raise();
    }

protected :
    virtual void runApp(QMap<QString, QString> args)
    {
				ApplicationContext::instance()->setUserName("test@digidoc.com");
        PluginLoader* _loader = PluginLoader::instance();
        _loader->loadPlugins();

        QString repoName = "LocalhostDev";
        DoQuBook::RepositoryPtr _repo = DoQuBook::ApplicationContext::instance()->getRepository(repoName);
        DoQuBook::PageCollectionServicePtr _collectionService = _repo->getPageCollectionService();
        BinarizerPtr bin = _loader->getPlugin<Binarizer>("labri.BinarizationOtsu.v1.0");
		
        QString firstCollectionName = _collectionService->getAll().at(0);
        QList<PagePtr> pages = _collectionService->getPages(firstCollectionName);
        foreach(PagePtr p, pages) {
            addImage(p->getImage(), p->getName());
			if(bin)
				addImage(bin->binarize(p->getImage()), bin->algorithmName());
        }
    }


    void createGui()
    {

        mainWin = new QMainWindow;
        vLayout = new QVBoxLayout (mainWin);
        scrollArea = new QScrollArea(mainWin->centralWidget());
        scrollArea->setWidgetResizable(true);
        widget = new QWidget;
        mainLayout = new QVBoxLayout(widget);
        scrollArea->setWidget(widget);
        scrollArea->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
        scrollArea->setVerticalScrollBarPolicy( Qt::ScrollBarAsNeeded);
        scrollArea->setFixedWidth(550);

        vLayout->addWidget(scrollArea);
        mainWin->setCentralWidget(scrollArea);
        mainWin->setFixedWidth(550);

        mainWin->show();
    }

    void addImage(const QImage &image,const QString &name)
    {
        QLabel *label = new QLabel;
        QImage scaled = image.scaled(500,500, Qt::KeepAspectRatio);
        label->setPixmap(QPixmap::fromImage(scaled));

        mainLayout->addWidget(new QLabel(name));
        mainLayout->addWidget(label);
    }

private :
    QMainWindow *mainWin;
    QVBoxLayout * mainLayout;
    QScrollArea *scrollarea;
    QVBoxLayout *vLayout;
    QScrollArea *scrollArea;
    QWidget *widget;

    int i;
    int j;
};


int main(int argc, char *argv[])
{
    QApplication app(argc, argv);

    SampleDoQuBookApp test (app);

    return test.exec();
}
