#include "DBWebAnnotationService.hpp"

#include <DoQuBook/Exceptions.hpp>
#include <DoQuBook/ApplicationContext.hpp>
#include <DoQuBook/Config.hpp>
#include <MarshallableFormaterFactory.hpp>

using namespace DoQuBook;

DBWebAnnotationService::DBWebAnnotationService (const QString & serverAdress)
    : DBWebCRUDServiceIntId<Annotation> (serverAdress, "annotation", "id")
    , m_annotationGlobalManager (WebClient::JSON, m_marshaller)
    , m_annotationLocalManager (WebClient::JSON, m_marshaller)
    , m_getAllByChunksManager (WebClient::JSON, m_marshaller)
    , m_namesManager (WebClient::JSON, m_marshaller)
    , m_getPagesWithAnnNameByChunksManager (WebClient::JSON, m_marshaller)
    , m_getAnnotationGlobalByPageAndNameManager (WebClient::JSON, m_marshaller)
    , m_getAnnotationLocalByPageAndNameManager (WebClient::JSON, m_marshaller)
{
    QString addGlobalAnnUrl (serverAdress+ "/" + m_rest_service_url + "/annotation/addGlobalAnnotationToPage/(pageId)");
    m_annotationGlobalManager.addRessourcePath (WebClient::POST, addGlobalAnnUrl);

    QString addLocalAnnUrl (serverAdress+ "/" + m_rest_service_url + "/annotation/addLocalAnnotationToPage/(pageId)");
    m_annotationLocalManager.addRessourcePath (WebClient::POST, addLocalAnnUrl);

    QString getAllByChunks ( m_server_adress + "/" + m_rest_service_url + "/annotation/ofPage/(pageId)");
    m_getAllByChunksManager.addRessourcePath (WebClient::GET, getAllByChunks);

    QString getAllNamesUrl (m_server_adress + "/" + m_rest_service_url + "/annotation/names");
    m_namesManager.addRessourcePath(WebClient::GET, getAllNamesUrl);

    QString getAllPagesWithAnnotationNamesUrl (m_server_adress + "/" + m_rest_service_url + "/annotation/pagesWithAnnotationName/(name)");
    m_getPagesWithAnnNameByChunksManager.addRessourcePath(WebClient::GET, getAllPagesWithAnnotationNamesUrl);

    QString getAnnotationByPageAndNameManagerUrl (m_server_adress + "/" + m_rest_service_url + "/annotation/annotationByPageAndName/(name)/(pageId)");
    m_getAnnotationGlobalByPageAndNameManager.addRessourcePath(WebClient::GET, getAnnotationByPageAndNameManagerUrl);
    m_getAnnotationLocalByPageAndNameManager.addRessourcePath(WebClient::GET, getAnnotationByPageAndNameManagerUrl);
}

QStringList DBWebAnnotationService::getAllAnnotationNames()
{
  QMap<QString, QString> params;
  return m_namesManager.get(params)->getStrings();
}

PageIteratorPtr DBWebAnnotationService::getPagesWithAnnotationName(const QString& name)
{
  QMap<QString, QString> params;
  params["name"] = name;
  PageIteratorPtr it = m_getPagesWithAnnNameByChunksManager.get (params);
  return it;
}


AnnotationLocalPtr DBWebAnnotationService::getAnnotationLocal (const int & id)
{
    AnnotationPtr ann = get (id);
    return ann.dynamicCast<AnnotationLocal>();
}

AnnotationLocalPtr DBWebAnnotationService::getAnnotationLocalOfPageByName(const PagePtr& p, const QString& name)
{
  QMap<QString, QString> params;
  params["pageId"] = QString::number(p->getId());
  params["name"] = name;
  return m_getAnnotationLocalByPageAndNameManager.get(params);
}
AnnotationGlobalPtr DBWebAnnotationService::getAnnotationGlobal (const int & id)
{
    AnnotationPtr ann = get (id);
    return ann.dynamicCast<AnnotationGlobal>();
}

AnnotationGlobalPtr DBWebAnnotationService::getAnnotationGlobalOfPageByName(const PagePtr& p, const QString& name)
{
  QMap<QString, QString> params;
  params["pageId"] = QString::number(p->getId());
  params["name"] = name;
  return m_getAnnotationGlobalByPageAndNameManager.get(params);
}

AnnotationIteratorPtr DBWebAnnotationService::getAnnotationOfPage(const PagePtr& p)
{
  QMap<QString, QString> params;
  params["pageId"] = QString::number(p->getId());
  AnnotationIteratorPtr it = m_getAllByChunksManager.get (params);
  return it;
}

AnnotationGlobalPtr  DBWebAnnotationService::addGlobalAnnotationToPage(PagePtr& p, AnnotationGlobalPtr &ann)
{
  AnnotationPtr a = ann;
  preAdd(a);
  QMap<QString, QString> params;
  params["pageId"] = QString::number (p->getId() );
  AnnotationGlobalPtr onServerAnn = m_annotationGlobalManager.post(ann, params);
  return onServerAnn;
}

AnnotationGlobalPtr DBWebAnnotationService::addGlobalAnnotationToPage(PagePtr& p, const QString& name, const QVariant& value)
{
    AnnotationGlobalPtr ann = AnnotationGlobal::create();
    ann->setName(name);
    ann->setValue(value);

    return addGlobalAnnotationToPage(p, ann);
}

AnnotationLocalPtr DBWebAnnotationService::addLocalAnnotationToPage(PagePtr& p, AnnotationLocalPtr& ann)
{
  AnnotationPtr a = ann;
  preAdd(a);
  QMap<QString, QString> params;
  params["pageId"] = QString::number (p->getId() );
  AnnotationLocalPtr onServerAnn = m_annotationLocalManager.post(ann, params);
  return onServerAnn;
}

