#ifndef DBWEBANNOTATIONSERVICE_HPP
#define DBWEBANNOTATIONSERVICE_HPP

#include <DoQuBook/DoQuBook_global.hpp>

#include <DoQuBook/AnnotationService.hpp>
#include <DoQuBook/RESTObjectManager.hpp>


#include <DoQuBook/StringList.hpp>

#include <DBWebCRUDService.hpp>
#include <DBWebPageService.hpp>
#include <DBWebIterator.hpp>

namespace DoQuBook
{

  class AnnotationRestIterator;
  typedef QSharedPointer<AnnotationRestIterator> AnnotationRestIteratorPtr;


  class DoQuBookSHARED_EXPORT AnnotationRestIterator : public SimpleMarshallable, public Iterator<AnnotationPtr>
  {
      Q_OBJECT
    public :
      Q_PROPERTY (QList<AnnotationPtr> elements READ getElements WRITE setElements)
      Q_PROPERTY (QUrl nextUrl READ getNextUrl WRITE setNextUrl)

      REST_ITERATOR (Annotation)

      public :
        void setServerAdress(const QString &serverAdress) {_serverAdress =serverAdress;}

    protected:
      virtual void configure (Iterator<AnnotationPtr> *nextIt) {
        AnnotationRestIterator *pageIt = dynamic_cast<AnnotationRestIterator*>(nextIt);
        pageIt->setServerAdress(_serverAdress);
      }

    private :
      QString _serverAdress;
      MarshallableFormaterPtr marshaller;
  };

  class DoQuBookSHARED_EXPORT DBWebAnnotationService : public AnnotationService,  public DBWebCRUDServiceIntId<Annotation>
  {
    public:
      DBWebAnnotationService (const QString & serverAdress);

      QStringList getAllAnnotationNames();

      virtual PageIteratorPtr getPagesWithAnnotationName(const QString& name);

      virtual AnnotationLocalPtr getAnnotationLocal (const int & id);
      virtual AnnotationLocalPtr getAnnotationLocalOfPageByName(const PagePtr &p, const QString &name);

      virtual AnnotationGlobalPtr getAnnotationGlobal (const int & id);
      virtual AnnotationGlobalPtr getAnnotationGlobalOfPageByName(const PagePtr &p, const QString &name);

      virtual AnnotationIteratorPtr getAnnotationOfPage(const PagePtr &p);

      virtual AnnotationGlobalPtr addGlobalAnnotationToPage(PagePtr &p, AnnotationGlobalPtr &ann);
      virtual AnnotationGlobalPtr addGlobalAnnotationToPage(PagePtr &p, const QString &name, const QVariant &value) ;
      virtual AnnotationLocalPtr addLocalAnnotationToPage(PagePtr &p, AnnotationLocalPtr &ann);

    private :
      RESTObjectManager<AnnotationGlobal> m_annotationGlobalManager;
      RESTObjectManager<AnnotationLocal> m_annotationLocalManager;
      RESTObjectManager<AnnotationRestIterator> m_getAllByChunksManager;
      RESTObjectManager<StringList> m_namesManager;
      RESTObjectManager<PageRestIterator> m_getPagesWithAnnNameByChunksManager;

      RESTObjectManager<AnnotationGlobal> m_getAnnotationGlobalByPageAndNameManager;
      RESTObjectManager<AnnotationLocal> m_getAnnotationLocalByPageAndNameManager;
  };

}
#endif // DBWEBANNOTATIONSERVICE_HPP
