#include "DBWebBinarizerRemoteProxy.hpp"

#include <QMap>

using namespace DoQuBook;

DBWebBinarizerRemoteProxy::DBWebBinarizerRemoteProxy (DoQuBook::RepositoryPtr repo, DoQuBook::ExecutorPtr executor)
    : _repo (repo)
    , _executor (executor)
{
}

QString DBWebBinarizerRemoteProxy::organisationName()
{
    QStringList infos = _executor->getId().split (".");
    return infos.at (0);
}

QString DBWebBinarizerRemoteProxy::description()
{
    return "Remote Binarizer Proxy on " + _executor->getId();
}

QString DBWebBinarizerRemoteProxy::algorithmName()
{
    QStringList infos = _executor->getId().split (".");
    return infos.at (1);
}

QString DBWebBinarizerRemoteProxy::version()
{
    QStringList infos = _executor->getId().split (".");
    return infos.at (2);
}

QString DBWebBinarizerRemoteProxy::pluginType()
{
    return _executor->getPluginType();
}

QImage DBWebBinarizerRemoteProxy::binarize (const QImage & image, const QVariantMap & params)
{
    qDebug() << "TRYING REMOTE BINARIZER";
    qDebug() << _executor.isNull();
    TaskPtr t = Task::create (_executor->getId() );
    t->addParam ("image", image);

    QMapIterator<QString, QVariant> it (params);
    while (it.hasNext() ) {
        it.next();
        t->addParam (it.key(), it.value() );
    }
    TaskServicePtr taskService = _repo->getTaskService();
    taskService->add (t);

    TaskPtr onServerUpdatedTask = taskService->getTaskWhenDone (t->getId() );

    // The task is done : we delete it
    taskService->del (t);

    /// If task has failed
    if (onServerUpdatedTask->getStatus() == Task::Fail) {
        qCritical() << onServerUpdatedTask->getErrorMessage();
        throw MessageException (onServerUpdatedTask->getErrorMessage() );
    }

    /// If it succeded, we get the image back to a QImage.
    QVariant result = onServerUpdatedTask->getPureResult();
    QImage resultingImage = result.value<QImage>();
    return resultingImage;
}


QObject * DBWebBinarizerRemoteProxy::BinarizerRemotePluginFactory::build()
{
    DBWebBinarizerRemoteProxy * proxy = new DBWebBinarizerRemoteProxy (m_repo, m_exec);
    return proxy;
}

void DBWebBinarizerRemoteProxy::BinarizerRemotePluginFactory::unLoad()
{
    // TODO: DELETE THE PROXY !!!
    qCritical() << "DELETE THE PROXY !!!";
}

RemotePluginFactory * DBWebBinarizerRemoteProxy::BinarizerRemotePluginFactory::clone()
{
    return new BinarizerRemotePluginFactory;
}
