#ifndef DBWEBCRUDSERVICE_HPP
#define DBWEBCRUDSERVICE_HPP

#include <QString>
#include <DoQuBook/RESTObjectManager.hpp>
#include <DoQuBook/DoQuBook_global.hpp>
#include <DoQuBook/CRUDService.hpp>

#include <MarshallableFormaterFactory.hpp>

namespace DoQuBook
{

    template <class Resource, class IdType>
    class DoQuBookSHARED_EXPORT DBWebCRUDService : virtual public CRUDService<Resource, IdType>
    {
        public :
            DBWebCRUDService (const QString & serverAdress,
                              const QString & resourceName,
                              const QString & idPropertyName)
                : m_marshaller (DoQuBook::DBWebRepo::MarshallableFormaterFactory::instance()->create())
                , m_server_adress (serverAdress)
                , m_resourceManager (WebClient::JSON, m_marshaller)
                , m_resourceName (resourceName)
                , m_idPropertyName (idPropertyName) {
                m_rest_service_url = "DoQuBookWeb/rest";
                m_restServiceURL = m_server_adress + "/" + m_rest_service_url + "/" + m_resourceName;

                QString getUrl (m_restServiceURL + "/(" + m_idPropertyName + ")");
                m_resourceManager.addRessourcePath (WebClient::GET, getUrl);

                QString postUrl (m_restServiceURL);
                m_resourceManager.addRessourcePath (WebClient::POST, postUrl);

                QString putUrl (m_restServiceURL);
                m_resourceManager.addRessourcePath (WebClient::PUT, putUrl);

                QString delUrl (m_restServiceURL + "/(" + m_idPropertyName + ")");
                m_resourceManager.addRessourcePath (WebClient::DEL, delUrl);

            }

            virtual ~DBWebCRUDService() {}

            QSharedPointer<Resource> get (const IdType & id) {
                QMap<QString, QString> params;
                params[m_idPropertyName] = convert (id);

                try {
                    QSharedPointer<Resource> obj = m_resourceManager.get (params);
                    if (!obj.isNull() ) {
                        postGet (obj);
                    }
                    return obj;
                }
                catch (CouldNotParseJSONException e) {
#ifdef DEBUG_LOG
                    qCritical()  << "DBWebCRUDService::get->CouldNotParseJSONException";
#endif
                    return QSharedPointer<Resource>();
                }
            }

            virtual void add (QSharedPointer<Resource>  &obj) {
                preAdd (obj);
                obj = m_resourceManager.post (obj);
                if (!obj.isNull() ) {
                    postAdd (obj);
                }
            }

            virtual void update (QSharedPointer<Resource>  &obj) {
                preUpdate (obj);

                QVariantMap mapObj = m_resourceManager.put (obj)->serialize();
                obj->from (mapObj);

                if (!obj.isNull() ) {
                    postUpdate (obj);
                }
            }

            virtual bool del (const QSharedPointer<Resource> &obj) {
                preDel (obj);
                m_resourceManager.del (obj);
                // Parse the BoooleanResponse
                return true;
            }

            virtual bool del (const IdType & id) {
                preDel (id);
                QMap<QString, QString> params;
                params[m_idPropertyName] = convert (id);

                m_resourceManager.del (params);
                // Parse the BoooleanResponse
                return true;
            }

        protected :
            virtual void preAdd (QSharedPointer<Resource>  &/*obj*/) {}
            virtual void preUpdate (QSharedPointer<Resource>  &/*obj*/) {}
            virtual void preDel (const QSharedPointer<Resource>  &/*obj*/) {}
            virtual void preDel (const IdType &) {}

            virtual void postGet (QSharedPointer<Resource>  &/*obj*/) {}
            virtual void postAdd (QSharedPointer<Resource>  &/*obj*/) {}
            virtual void postUpdate (QSharedPointer<Resource>  &/*obj*/) {}

            virtual QString convert (const IdType & id) = 0;

        protected :
            MarshallableFormaterPtr m_marshaller;
            QString m_server_adress;
            RESTObjectManager<Resource> m_resourceManager;
            QString m_resourceName;
            QString m_rest_service_url;
            QString m_idPropertyName;
            QString m_restServiceURL;
    };


    template <class Resource>
    class DoQuBookSHARED_EXPORT DBWebCRUDServiceIntId : public DBWebCRUDService<Resource, int>
    {
        public :

            DBWebCRUDServiceIntId (const QString & serverAdress,
                                   const QString & resourceName,
                                   const QString & idPropertyName)
                : DBWebCRUDService<Resource, int> (serverAdress, resourceName, idPropertyName)
            {}

        protected:

            virtual QString convert (const int & id) {
                return QString :: number (id);
            }
    };


}

#endif // DBWEBCRUDSERVICE_HPP
