#include "DBWebExecutorService.hpp"


#include <MarshallableFormaterFactory.hpp>

using namespace DoQuBook;

DBWebExecutorService::DBWebExecutorService (const QString & serverAdress)
    : m_marshaller(DBWebRepo::MarshallableFormaterFactory::instance()->create())
    , m_serverAdress (serverAdress)
    , m_executorManager (WebClient::JSON, m_marshaller)
    , m_idsManager (WebClient::JSON, m_marshaller)
{
    m_restServicesUrl = "DoQuBookWeb/rest/executor";


    QString postUrl (m_serverAdress + "/" + m_restServicesUrl + "/");
    m_executorManager.addRessourcePath (WebClient::POST, postUrl);

    QString getUrl (m_serverAdress + "/" + m_restServicesUrl + "/(id)");
    m_executorManager.addRessourcePath (WebClient::GET, getUrl);

    QString getAll (m_serverAdress + "/" + m_restServicesUrl);
    m_idsManager.addRessourcePath (WebClient::GET, getAll);

    QString putUrl (m_serverAdress + "/" + m_restServicesUrl + "/");
    m_executorManager.addRessourcePath (WebClient::PUT, putUrl);

    QString delUrl (m_serverAdress + "/" + m_restServicesUrl + "/(id)");
    m_executorManager.addRessourcePath (WebClient::DEL, delUrl);

}

void DBWebExecutorService::add (DoQuBook::ExecutorPtr & executor)
{
    WebClient c;

    if (!c.isAvailable (m_serverAdress) ) {
        throw InternetConnectionException (m_serverAdress);
    }

    executor = m_executorManager.post (executor);
}

ExecutorPtr DBWebExecutorService::get (const QString & id)
{
    WebClient c;

    if (!c.isAvailable (m_serverAdress) ) {
        throw InternetConnectionException (m_serverAdress);
    }

    QMap<QString, QString> params;
    params["id"] = id;

    try {
        return m_executorManager.get (params);
    }
    catch (CouldNotParseJSONException e) {
        qCritical()  << "DBWebAnnotationService::get->CouldNotParseJSONException";
        return ExecutorPtr();
    }
}

QList<QString> DBWebExecutorService::getAll()
{
    WebClient c;

    if (!c.isAvailable (m_serverAdress) ) {
        throw InternetConnectionException (m_serverAdress);
    }

    QList<QString> ids;

    try {
        QMap<QString, QString> params;
        StringListPtr stringList = m_idsManager.get (params);
        if(!stringList.isNull())
          ids = stringList->getStrings();
        return ids;
    }
    catch (CouldNotParseJSONException e) {
        qCritical()  << "DBWebAnnotationService::get->CouldNotParseJSONException";
        return ids;
    }
}

bool DBWebExecutorService::del (ExecutorPtr & executor)
{
    WebClient c;

    if (!c.isAvailable (m_serverAdress) ) {
        throw InternetConnectionException (m_serverAdress);
    }

    QMap<QString, QString> params;
    params["id"] = executor->getId();

    m_executorManager.del (params);
    /// \todo Parse the boolean response;
    return true;
}

void DBWebExecutorService::update (ExecutorPtr & executor)
{
    executor = m_executorManager.put (executor);
}
