#include "DBWebFeatureService.hpp"

#include <MarshallableFormaterFactory.hpp>

using namespace DoQuBook;

DBWebFeatureService::DBWebFeatureService (const QString & serverAdress)
    : m_marshaller (DBWebRepo::MarshallableFormaterFactory::instance()->create())
    , m_restServiceURL (serverAdress + "/DoQuBookWeb/rest/globalnamedfeature")
    , m_globalNamedFeatureManager (WebClient::JSON, m_marshaller)
    , m_getAllManager (WebClient::JSON, m_marshaller)
{
    QString postImageUrl (m_restServiceURL + "/(pageId)/(featureName)/(featureValue)");
    m_globalNamedFeatureManager.addRessourcePath (WebClient::POST, postImageUrl);

    QString getAll (m_restServiceURL + "/iterator/(pageId)");
    m_getAllManager.addRessourcePath (WebClient::GET, getAll);

}

GlobalNamedFeaturePtr DBWebFeatureService::addGlobalNamedFeatureToPage (PagePtr page, QString featureName, double featureValue)
{
    QMap<QString, QString> params;
    params["pageId"] = QString::number (page->getId() );
    params["featureName"] = featureName;
    params["featureValue"] = QString::number (featureValue);

    GlobalNamedFeaturePtr feature = GlobalNamedFeature::create();
    feature->setFeatureName (featureName);
    feature->setFeatureValue (featureValue);

    GlobalNamedFeaturePtr addedFeature = m_globalNamedFeatureManager.post (feature, params);
    return addedFeature;
}

meta_iterator<GlobalNamedFeaturePtr>::IteratorPtr DBWebFeatureService::getGlobalNamedFeaturesOfPage (PagePtr page)
{
    QMap<QString, QString> params;
    params["pageId"] = QString::number (page->getId() );
    GlobalNamedFeatureRestIteratorPtr it = m_getAllManager.get (params);
    return it;
}
