#include "DBWebGroundTruthService.hpp"

#include <MarshallableFormaterFactory.hpp>
using namespace DoQuBook;

DBWebGroundTruthService::DBWebGroundTruthService (const QString & serverAdress)
    : m_marshaller (DBWebRepo::MarshallableFormaterFactory::instance()->create())
    , m_server_adress (serverAdress)
    , m_binImageManager (RESTImageManager::JPEG)
    , m_getAllByChunksManager (WebClient::JSON, m_marshaller)
{
    QString getImageUrl (m_server_adress + "/DoQuBookWeb/rest/" + "groundtruth/binarisation/" + "ofPage/(pageId)");
    m_binImageManager.addRessourcePath (WebClient::GET, getImageUrl);

    QString postImageUrl (m_server_adress + "/DoQuBookWeb/rest/" + "groundtruth/binarisation/" + "addToPage/(pageId)");
    m_binImageManager.addRessourcePath (WebClient::POST, postImageUrl);

    QString getAllByChunks ( m_server_adress + "/DoQuBookWeb/rest/" + "groundtruth/binarisation/pages");
    m_getAllByChunksManager.addRessourcePath (WebClient::GET, getAllByChunks);
}

DBWebGroundTruthService::~DBWebGroundTruthService()
{
}

meta_iterator<PagePtr>::IteratorPtr DBWebGroundTruthService::getAllBinarisationGTedPages()
{
    QMap<QString, QString> params;
    PageRestIteratorPtr it = m_getAllByChunksManager.get (params);
    return it;
}

void DBWebGroundTruthService::addBinarisationGroundTruth (const PagePtr page, const QFileInfo & file)
{
    QMap<QString, QString> params;
    params["pageId"] = QString::number (page->getId () );

    QImage image (file.absoluteFilePath() );

    if (image.isNull() ) {
        throw MessageException ("Image to upload is null");
    }

    QString suffix = file.completeSuffix().toLower();

    RESTImageManager::ImageType type;
    if (suffix == "tiff" || suffix == "tif" ) {
        type = RESTImageManager::TIFF;
    }
    else if (suffix == "jpeg" || suffix == "jpg") {
        type = RESTImageManager::JPEG;
    }
    else if (suffix == "png") {
        type = RESTImageManager::PNG;
    }
    else  {
        qWarning() << "Image with extention " << suffix << " not yet supported ... Converting to JPEG";
        type = RESTImageManager::JPEG;
    }


    m_binImageManager.post (image, "bin." + suffix, params, type);

}

QImage DBWebGroundTruthService::getBinarisationGroundTruth (const PagePtr page)
{
    QMap<QString, QString> params;
    params["pageId"] = QString::number (page->getId () );

    return m_binImageManager.get (params);
}

void DBWebGroundTruthService::onBinarizationGroundTruthAdded (Functor<Page> *functor)
{
    HttpEventHandler<Page>* f3 = new HttpEventHandler<Page> (m_server_adress + "/DoQuBookWeb/rest/groundtruth/binarisation/events/add", m_marshaller);
    f3->addObserver (functor);
    f3->listen();
}
