#include "DBWebMeasurerRemoteProxy.hpp"

using namespace DoQuBook;

DBWebMeasurerRemoteProxy::DBWebMeasurerRemoteProxy (DoQuBook::RepositoryPtr repo, DoQuBook::ExecutorPtr executor)
    : _repo (repo)
    , _executor (executor)
{
}

QString DBWebMeasurerRemoteProxy::organisationName()
{
    QStringList infos = _executor->getId().split (".");
    return infos.at (0);
}

QString DBWebMeasurerRemoteProxy::description()
{
    return "Remote Measurer Proxy on " + _executor->getId();
}

QString DBWebMeasurerRemoteProxy::algorithmName()
{
    QStringList infos = _executor->getId().split (".");
    return infos.at (1);

}

QString DBWebMeasurerRemoteProxy::version()
{
    QStringList infos = _executor->getId().split (".");
    return infos.at (2);
}

QString DBWebMeasurerRemoteProxy::pluginType()
{
    return _executor->getPluginType();
}

QMap<QString, double> DBWebMeasurerRemoteProxy::measure (QList<QImage> &images, const QVariantMap & params)
{
    TaskPtr t = Task::create (_executor->getId() );
    QList<QVariant> variants;
    convert (images, variants);
    t->addParam ("images", variants);

    QMapIterator<QString, QVariant> it (params);
    while (it.hasNext() ) {
        it.next();
        t->addParam (it.key(), it.value() );
    }
    TaskServicePtr taskService = _repo->getTaskService();
    taskService->add (t);

    TaskPtr onServerUpdatedTask = taskService->getTaskWhenDone (t->getId() );

    // The task is done : we delete it
    taskService->del (t);

    /// If task has failed
    if (onServerUpdatedTask->getStatus() == Task::Fail) {
        qCritical() << onServerUpdatedTask->getErrorMessage();
        throw MessageException (onServerUpdatedTask->getErrorMessage() );
    }

    QVariant result = onServerUpdatedTask->getPureResult();
    QMap<QString, double> measures;
    convert (result, measures);
    return measures;
}

void DBWebMeasurerRemoteProxy::convert (const QList<QImage> &images, QList<QVariant> &variants)
{
    foreach (QImage img, images) {
        variants << img;
    }
}

void DBWebMeasurerRemoteProxy::convert (const QVariant & variant, QMap<QString, double> &measures)
{
    QVariantMap map = variant.toMap();
    QMapIterator<QString, QVariant> it (map);
    while (it.hasNext() ) {
        it.next();
        double value = it.value().toDouble();
        QString k = it.key();
        measures[k] = value;
    }
}


QObject * DBWebMeasurerRemoteProxy::MeasurerRemotePluginFactory::build()
{
    DBWebMeasurerRemoteProxy * proxy = new DBWebMeasurerRemoteProxy (m_repo, m_exec);
    return proxy;
}

void DBWebMeasurerRemoteProxy::MeasurerRemotePluginFactory::unLoad()
{
    // TODO: DELETE THE PROXY !!!
    qCritical() << "DELETE THE PROXY !!!";
}

DoQuBook::RemotePluginFactory * DBWebMeasurerRemoteProxy::MeasurerRemotePluginFactory::clone()
{
    return new MeasurerRemotePluginFactory;
}
