#include "DBWebOCRRemoteProxy.hpp"
#include <DoQuBook/Task.hpp>

using namespace DoQuBook;

DBWebOCRRemoteProxy::DBWebOCRRemoteProxy (DoQuBook::RepositoryPtr repo, DoQuBook::ExecutorPtr executor)
    : _repo (repo)
    , _executor (executor)
{
}

QString DBWebOCRRemoteProxy::organisationName()
{
    QStringList infos = _executor->getId().split (".");
    return infos.at (0);
}

QString DBWebOCRRemoteProxy::description()
{
    return "Remote Binarizer Proxy on " + _executor->getId();
}

QString DBWebOCRRemoteProxy::algorithmName()
{
    QStringList infos = _executor->getId().split (".");
    return infos.at (1);
}

QString DBWebOCRRemoteProxy::version()
{
    QStringList infos = _executor->getId().split (".");
    return infos.at (2);
}

QString DBWebOCRRemoteProxy::ocr (const QImage & image, const QVariantMap & params)
{
    TaskServicePtr taskService = _repo->getTaskService();
    TaskPtr t = Task::create (_executor->getId() );

    t->addParam ("image", image);

    QMapIterator<QString, QVariant> it (params);
    while (it.hasNext() ) {
        it.next();
        t->addParam (it.key(), it.value() );
    }

    taskService->add (t);

    TaskPtr onServerUpdatedTask = taskService->getTaskWhenDone (t->getId() );

    qDebug() << "HEY I GOT THE TASK BACK !!!!!!!";

    // The task is done : we delete it
    taskService->del (t);
    if (onServerUpdatedTask->getStatus() == Task::Fail) {
        qCritical() << onServerUpdatedTask->getErrorMessage();
        throw MessageException (onServerUpdatedTask->getErrorMessage() );
    }
    QVariant result = onServerUpdatedTask->getResult();
    if (onServerUpdatedTask->getResourceType() == "String") {
        return result.toString();
    }
    else {
        qDebug() << "ResourceType : " <<  onServerUpdatedTask->getResourceType();
        qDebug() << onServerUpdatedTask->getResult();
        qCritical() << "Unsuported resource result for task " << onServerUpdatedTask->getId();
        return "Unsuported resource result for task ";
    }
}
