#include "DBWebPageCollectionService.hpp"

using namespace DoQuBook;

using namespace DoQuBook::DBWebRepo;

DBWebPageCollectionService::DBWebPageCollectionService (const QString & serverAdress)
  : m_marshaller (MarshallableFormaterFactory::instance()->create())
  , m_serverAdress (serverAdress)
  , m_collectionPageManager (WebClient::JSON, m_marshaller)
  , m_pageListManager (WebClient::JSON, m_marshaller)
  , m_pageIdsManager (WebClient::JSON, m_marshaller)
  , m_idsManager (WebClient::JSON, m_marshaller)
  , m_pageService (new DBWebPageService (serverAdress) )
{
  QString rest_service_url = "DoQuBookWeb/rest";
  QString pageCollectionUrl = "pagecollection";

  QString addPageUrl (m_serverAdress + "/" + rest_service_url + "/" + pageCollectionUrl + "/(collectionName)/page");
  m_collectionPageManager.addRessourcePath (WebClient::POST, addPageUrl);

  QString getPagesUrl (m_serverAdress + "/" + rest_service_url + "/" + pageCollectionUrl + "/(collectionName)/pagesbychunk");
  m_pageListManager.addRessourcePath (WebClient::GET, getPagesUrl);

  QString getPagesIdsUrl (m_serverAdress + "/" + rest_service_url + "/" + pageCollectionUrl + "/(collectionName)/pages");
  m_pageIdsManager.addRessourcePath (WebClient::GET, getPagesIdsUrl);

  QString getAllUrl (m_serverAdress + "/" + rest_service_url + "/" + pageCollectionUrl);
  m_idsManager.addRessourcePath (WebClient::GET, getAllUrl);

}

void DBWebPageCollectionService::addPage (const QString & collectionName, DoQuBook::PagePtr & page)
{
  if (m_pageService->get (page->getId() ).isNull() ) {
    m_pageService->add (page);
  }

  QMap<QString, QString> params;
  params["collectionName"] = collectionName;

  m_collectionPageManager.post (page, params);
}

QList<int> DBWebPageCollectionService::getPagesIds (const QString & collectionName)
{
  QMap<QString, QString> params;
  params["collectionName"] = collectionName;

  IntlistPtr intList = m_pageIdsManager.get (params);
  return intList->getIds();
}

QList<QString> DBWebPageCollectionService::getAll()
{
  QMap<QString, QString> params;

  StringListPtr list = m_idsManager.get (params);

  return list->getStrings();
}

meta_iterator<PagePtr>::IteratorPtr  DBWebPageCollectionService::getPages (const QString & collectionName)
{
  QMap<QString, QString> params;
  params["collectionName"] = collectionName;
  PageRestIteratorPtr it =  m_pageListManager.get (params);
  it->setServerAdress(m_serverAdress);
  return it;
}
