#include "DBWebPerformanceMetricService.hpp"

using namespace DoQuBook;



DBWebPerformanceMetricService::~DBWebPerformanceMetricService () {}



DBWebPerformanceMetricService::DBWebPerformanceMetricService (const QString & serverAdress)
    : DBWebCRUDService<PerformanceMetric, int> (serverAdress, "performancemetric", "id")
    , m_pagesByMetricNameManager (WebClient::JSON, m_marshaller)
    , m_getByPageAndNames (WebClient::JSON, m_marshaller)
{
    QString postUrl (m_restServiceURL + "/toPage/(pageId)");
    m_resourceManager.addRessourcePath (WebClient::POST, postUrl);

    QString getPagesByMetricName ( m_restServiceURL + "/(metricName)/pages");
    m_pagesByMetricNameManager.addRessourcePath (WebClient::GET, getPagesByMetricName);

    QString getByPageAndNames ( m_restServiceURL + "/(pageId)/(metricName)/(algorithmName)");
    m_getByPageAndNames.addRessourcePath (WebClient::GET, getByPageAndNames);

}

void DBWebPerformanceMetricService::addToPage (QSharedPointer<PerformanceMetric> &obj, PagePtr page)
{
    QMap<QString, QString> params;
    params["pageId"] = QString::number (page->getId() );

    preAdd (obj);
    obj = m_resourceManager.post (obj, params);
    if (!obj.isNull() ) {
        postAdd (obj);
    }
}

meta_iterator<PagePtr>::IteratorPtr  DBWebPerformanceMetricService::getPagesByMetricName (const QString & metricName)
{
    QMap<QString, QString> params;
    params["metricName"] = metricName;
    PageRestIteratorPtr it = m_pagesByMetricNameManager.get (params);
    return it;
}

PerformanceMetricPtr DBWebPerformanceMetricService::getByPageMetricNameAndAlgorithmName (PagePtr page, QString metricName, QString algorithmName)
{
    QMap<QString, QString> params;
    params["pageId"] = QString::number (page->getId() );
    params["metricName"] = metricName;
    params["algorithmName"] = algorithmName;

    return m_getByPageAndNames.get (params);
}
