#ifndef DBWEBTASKSERVICE_HPP
#define DBWEBTASKSERVICE_HPP

#include <DoQuBook/DoQuBook_global.hpp>

#include <DoQuBook/TaskService.hpp>
#include <DoQuBook/RESTObjectManager.hpp>
#include <DoQuBook/Intlist.hpp>

#include <DBWebTempImageService.hpp>
#include <DBWebTaskImageService.hpp>
#include <DBWebPageService.hpp>
#include <DBWebAnnotationService.hpp>

#include <QImage>
#include <RemoteQVariantConverter.hpp>


namespace DoQuBook
{

    class DoQuBookSHARED_EXPORT DBWebTaskService : public TaskService
    {
        public:
            DBWebTaskService (const QString & serverAdress);
            virtual ~DBWebTaskService() {}

            virtual TaskPtr get (int id);
            virtual TaskPtr getTaskWhenDone (int id);

            virtual void add (TaskPtr & task);
            virtual void notifyAccepted (TaskPtr & task);
            virtual void notifyHasFailed (TaskPtr & task);
            virtual void setFinalResult (TaskPtr & task);
            virtual void refreshForm (TaskPtr & task);
            virtual bool del (TaskPtr & task);


            virtual QList<int> getAll ();
            virtual QList<int> getNewTasks ();
            virtual QList<int> getNewTasksForExecutors (QStringList & executors);


            // TODO: This should be moved to DBWebXXXXXRemoteProxy.
            virtual void runExecutor (const QString & name,
                                      PagePtr & page,
                                      const QVariantMap & paramsMap = QVariantMap()
                                     );

        public :
            virtual void convertResultToQVariant (TaskPtr t);
            virtual void convertResultQVariantToString (TaskPtr & t);

            virtual void convertTaskParametersToQVariant (TaskPtr task);
            virtual void convertTaskParametersToString (TaskPtr & t);

            virtual QVariant convertTaskParameterToQVariant (TaskParameterPtr taskParam, TaskPtr task);

            void replaceClassByURLs (TaskPtr & t);

        private :
            MarshallableFormaterPtr m_marshaller;
            QString m_serverAdress;
            RESTObjectManager<Task> m_taskManager;
            RESTObjectManager<Task> m_asyncTaskManager;

            RESTObjectManager<Intlist> m_taskIdsManager;
            RESTObjectManager<Intlist> m_newTaskIdsManager;

            QString m_restServicesUrl;

            DBWebTempImageService m_tempImageService;
            DBWebTaskImageService m_taskImageService;
            DBWebPageService m_pageService;
            DBWebAnnotationService m_annotationService;

            QList<RemoteQVariantConverterPtr> m_factos;
    };

}

#endif // DBWEBTASKSERVICE_HPP
