#include "DoQuBookWebRepository.hpp"

#include <DBWebBookService.hpp>
#include <DBWebImageService.hpp>
#include <DBWebTaskService.hpp>
#include <DBWebTaskImageService.hpp>
#include <DBWebExecutorService.hpp>
#include <DBWebPageService.hpp>
#include <DBWebPageCollectionService.hpp>
#include <DBWebDaemonService.hpp>
#include <DBWebGroundTruthService.hpp>
#include <DBWebPerformanceMetricService.hpp>
#include <DBWebFeatureService.hpp>
#include <DBWebPredictionModelResultService.hpp>


#include <DBWebBinarizerRemoteProxy.hpp>
#include <DBWebMeasurerRemoteProxy.hpp>
#include <DBWebOCRRemoteProxy.hpp>


using namespace DoQuBook;
using namespace DoQuBook::DBWebRepo;

DoQuBookWebRepository::DoQuBookWebRepository()
{
}

RepositoryPtr DoQuBookWebRepository::clone()
{
  RepositoryPtr cloned (new DoQuBookWebRepository);
  cloned->setName ("digidoc");
  QVariant url = m_serverURl;
  cloned->setParam (url);
  return cloned;
}

PageServicePtr DoQuBookWebRepository::getPageService()
{
    DBWebPageServicePtr ptr (new DBWebPageService (m_serverURl) );
    return ptr;
}

void DoQuBook::DoQuBookWebRepository::setCredentials (const QString login, const QString & password)
{
    m_login = login;
    m_password = password;
}

DoQuBook::Repository::RepositoryParamWidgetType DoQuBook::DoQuBookWebRepository::widgetType()
{
    return LineEdit;
}

DoQuBook::BookServicePtr DoQuBook::DoQuBookWebRepository::getBookService()
{
    DBWebBookServicePtr ptr (new DBWebBookService (m_serverURl) );
    return ptr;
}

void DoQuBook::DoQuBookWebRepository::setParam (QVariant v)
{
    m_serverURl = QString(v.toMap()["url"].toString().toUtf8());
}

QVariant DoQuBook::DoQuBookWebRepository::getParam()
{
    return m_serverURl;
}

DoQuBook::ImageServicePtr DoQuBook::DoQuBookWebRepository::getImamgeService()
{
    DBWebImageServicePtr ptr (new DBWebImageService (m_serverURl) );
    return ptr;
}

DoQuBook::PageCollectionServicePtr DoQuBook::DoQuBookWebRepository::getPageCollectionService()
{
    PageCollectionServicePtr ptr (new DBWebPageCollectionService (m_serverURl) );
    return ptr;

}

TaskServicePtr DoQuBookWebRepository::getTaskService()
{
    TaskServicePtr taskService (new DBWebTaskService (m_serverURl) );
    return taskService;
}

TaskImageServicePtr DoQuBookWebRepository::getTaskImageService()
{
    TaskImageServicePtr taskImageService (new DBWebTaskImageService (m_serverURl) );
    return taskImageService;
}

TempImageServicePtr DoQuBookWebRepository::getTempImageService()
{
    TempImageServicePtr tempImageService (new DBWebTempImageService (m_serverURl) );
    return tempImageService;
}

ExecutorServicePtr DoQuBookWebRepository::getExecutorService()
{
    ExecutorServicePtr executorService (new DBWebExecutorService (m_serverURl) );
    return executorService;
}

DaemonServicePtr DoQuBookWebRepository::getDaemonService()
{
    DaemonServicePtr daemonService (new DBWebDaemonService (m_serverURl) );
    return daemonService;
}

GroundTruthServicePtr DoQuBookWebRepository::getGroundTruthService()
{
    GroundTruthServicePtr gtService (new DBWebGroundTruthService (m_serverURl) );
    return gtService;
}

PerformanceMetricServicePtr DoQuBookWebRepository::getPerformanceMetricService()
{
    PerformanceMetricServicePtr perfService (new DBWebPerformanceMetricService (m_serverURl) );
    return perfService;
}

FeatureServicePtr DoQuBookWebRepository::getFeatureService()
{
    FeatureServicePtr  featSrv (new DBWebFeatureService (m_serverURl) );
    return featSrv;
}

PredictionModelResultServicePtr DoQuBookWebRepository::getPredictionModelResultService()
{
    PredictionModelResultServicePtr srv (new DBWebPredictionModelResultService (m_serverURl) );
    return srv;
}

QList<DoQuBook::RemotePluginFactory*> DoQuBook::DoQuBookWebRepository::getRemotePluginFactories()
{
  QList<DoQuBook::RemotePluginFactory*> factos;
  factos << new DBWebBinarizerRemoteProxy::BinarizerRemotePluginFactory;
  factos << new DBWebMeasurerRemoteProxy::MeasurerRemotePluginFactory;
  return factos;
}


DQB_EXPORT_PLUGIN(DoQuBookWebRepository)
