#ifndef GENERICTESTFORENTITYFIELDS_HPP
#define GENERICTESTFORENTITYFIELDS_HPP

#include <DoQuBook/SimpleMarshallable.hpp>
#include <DoQuBook/RESTObjectManager.hpp>

#include <MarshallableFormaterFactory.hpp>

#include <QtTest>

namespace DoQuBook {
  namespace DBWebRepo {
    namespace TestsUtils {

      class RESTEntityDescription;
      typedef QSharedPointer<RESTEntityDescription> RESTEntityDescriptionPtr;

      class RESTEntityDescription : public SimpleMarshallable {
          Q_OBJECT

          Q_PROPERTY(QList<QString> attributes READ getAttributes WRITE setAttributes)
          QList<QString> m_attributes;

        public :
          MARSHALLABLE_CREATE_FONCTIONS (RESTEntityDescription)
          MARSHALLABLE_QTLIST (QList, QString, attributes)
          MARSHALLABLE_ACTIV_1LIST (attributes, m_attributes)

          RESTEntityDescription(){}

          QList<QString> getAttributes() const
          {
            return m_attributes;
          }
        public slots:
          void setAttributes(QList<QString> arg)
          {
            m_attributes = arg;
          }
      };


      template <class T,class IdType>
      class GenericTestForEntityFields {
        public :
          void testFields() {
            RESTObjectManager<RESTEntityDescription> manager (WebClient::JSON, DBWebRepo::MarshallableFormaterFactory::instance()->create());

            T a;
            const QMetaObject * o = a.metaObject();

            QString realClassName  = QString (o->className() ).toLower();
            QStringList namespaces = realClassName.split ("::");
            QString className = namespaces.at (namespaces.length()-1);

            QString getUrl ("http://localhost:8080/DoQuBookWeb/rest/"+className+"/description/entity");
            manager.addRessourcePath(WebClient::GET, getUrl);

            QMap<QString, QString> params;
            QSharedPointer<RESTEntityDescription> entityDescription = manager.get(params);
            qDebug() << entityDescription->getAttributes();


            QList<QString> attrs = entityDescription->getAttributes();
            foreach(QString attr, attrs) {
              bool found = false;
              for(int i = 0; i < o->propertyCount() && !found; i++) {
                found = attr == o->property(i).name();
              }
              QVERIFY(found);
            }
            for(int i = 0; i < o->propertyCount(); i++) {
              bool found = false;
              QString prop = o->property(i).name();
              if(prop != "objectName" && prop != "type") {
                for(int j = 0; j < attrs.size() && !found; j++) {
                  found = attrs[j] == prop;
                }
                if(!found) qCritical() << prop << " not found";
                QVERIFY(found);
              }
            }
          }
      };
    }
  }
}

#endif // GENERICTESTFORENTITYFIELDS_HPP
