# Find DoQuBookOpenCV - 
#
# This module defines
#  DoQuBookOpenCV_FOUND - whether the DoQuBookOpenCV library was found
#  DoQuBookOpenCV_LIBRARIES - the DoQuBookOpenCV library
#  DoQuBookOpenCV_INCLUDE_DIRS - the include path of the DoQuBookOpenCV library
#  DoQuBookOpenCV_DEFINITIONS

if (DoQuBookOpenCV_INCLUDE_DIRS AND DoQuBookOpenCV_LIBRARIES)
  # Already in cache
  set (DoQuBookOpenCV_FOUND TRUE)

else (DoQuBookOpenCV_INCLUDE_DIRS AND DoQuBookOpenCV_LIBRARIES)

  if (NOT WIN32)
            # use pkg-config to get the values of DoQuBookOpenCV_INCLUDE_DIRS
            # and DoQuBookOpenCV_LIBRARY_DIRS to add as hints to the find commands.
            include (FindPkgConfig)
            pkg_check_modules (PC_DoQuBookOpenCV DoQuBookOpenCV>=1.0)
  endif (NOT WIN32)
  
  if(WIN32)
    	# the hints
    	SET(DoQuBookOpenCV_POSSIBLE_ROOT_DIRS
        	"$ENV{DoQuBookOpenCV_ROOT_DIR}"
        	"$ENV{SystemDrive}/DoQuBookOpenCV"                    
        	"$ENV{SystemDrive}/Program Files/DoQuBookOpenCV"       
        	"$ENV{SystemDrive}/Program Files (x86)/DoQuBookOpenCV"
       )
			# find the right and only path
			FIND_PATH(DoQuBookOpenCV_ROOT_DIR
          NAMES include/DoQuBookOpenCV/ExtractFeature.h
          PATHS ${DoQuBookOpenCV_POSSIBLE_ROOT_DIRS}
      )
			# libs and includes
			SET(DoQuBookOpenCV_LIB_SEARCH_PATH 
					"${DoQuBookOpenCV_ROOT_DIR}/bin"
					"${DoQuBookOpenCV_ROOT_DIR}/lib"
			)
			SET(DoQuBookOpenCV_INCLUDE_SEARCH_PATH "${DoQuBookOpenCV_ROOT_DIR}/include")
  endif(WIN32)

  find_library (DoQuBookOpenCV_LIBRARIES
    NAMES
    DoQuBookOpenCV
    PATHS
    ${PC_DoQuBookOpenCV_LIBRARY_DIRS}
    ${DoQuBookOpenCV_FRAMEWORKS}
    ${LIB_INSTALL_DIR}  
    ${DoQuBookOpenCV_LIB_SEARCH_PATH}
  )


  find_path (DoQuBookOpenCV_INCLUDE_DIR
    NAMES
    DoQuBookOpenCV/DoQuBookOpenCVConverter.hpp
    PATHS
    ${PC_DoQuBookOpenCV_INCLUDE_DIRS}
    ${INCLUDE_INSTALL_DIR}
    ${DoQuBookOpenCV_FRAMEWORKS}
    ${DoQuBookOpenCV_INCLUDE_SEARCH_PATH}
  )

  include(FindPackageHandleStandardArgs)
  find_package_handle_standard_args(DoQuBookOpenCV DEFAULT_MSG DoQuBookOpenCV_LIBRARIES DoQuBookOpenCV_INCLUDE_DIR)

  if (DoQuBookOpenCV_INCLUDE_DIR AND DoQuBookOpenCV_LIBRARIES)
    # Already in cache
    set (DoQuBookOpenCV_FOUND TRUE)
  endif (DoQuBookOpenCV_INCLUDE_DIR AND DoQuBookOpenCV_LIBRARIES)
    

endif (DoQuBookOpenCV_INCLUDE_DIRS AND DoQuBookOpenCV_LIBRARIES)

if(${DoQuBookOpenCV_FOUND})

#@DoQuBookOpenCV_INCLUDES@
set(DoQuBookOpenCV_INCLUDE_DIRS ${DoQuBookOpenCV_INCLUDE_DIRS} ${DoQuBookOpenCV_INCLUDE_DIR} )


# OpenCV (optional)
FIND_PACKAGE(OpenCV REQUIRED)
set(DoQuBookOpenCV_INCLUDE_DIRS ${OpenCV_INCLUDE_DIRS} ${DoQuBookOpenCV_INCLUDE_DIRS})
set(DoQuBookOpenCV_LIBRARIES ${OpenCV_LIBS} ${DoQuBookOpenCV_LIBRARIES})

endif(${DoQuBookOpenCV_FOUND})

MARK_AS_ADVANCED(DoQuBookOpenCV_INCLUDE_DIRS DoQuBookOpenCV_LIBRARIES)