#include "QJSONMarshallableFormater.hpp"

#include <qjson/parser.h>
#include <qjson/serializer.h>

#include <DoQuBook/Exceptions.hpp>
#include <assert.h>

bool QJSONMarshallableFormater::accept(const QString&)
{return true;}

QString QJSONMarshallableFormater::contentType()
{return "application/json";}

void QJSONMarshallableFormater::addIgnoredProp(const QString& prop) {
  ignoredProps.append (prop);
}

void QJSONMarshallableFormater::unMarshall(const QByteArray& file, QSharedPointer<Marshallable> dest) {
  QJson::Parser parser;
  QSharedPointer<Marshallable> superMarshall = dest;

  bool ok;
  QVariantMap map = parser.parse (file, &ok).toMap();

  if (!ok || map.isEmpty() ) {
    throw DoQuBook::CouldNotParseJSONException ("Failed JSON Parsing", file);
  }

  QList<MarshallablePtr> seeAlso = superMarshall->seeAlso();

  if (!seeAlso.empty() ) {
    foreach (MarshallablePtr marshal, seeAlso) {

      QString s  = QString (marshal->metaObject()->className() ).toLower();
      QStringList names = s.split ("::");

      if (map.contains (names.at (names.length() - 1) ) ) {
        superMarshall = marshal;
      }
    }
  }

  QString entityName = superMarshall->entityName();
  QVariantMap bookMap = map[entityName].toMap();

  dest->from (bookMap);
}

void QJSONMarshallableFormater::marshall(QSharedPointer<Marshallable> m, QString* dest) {
  assert (dest);
  QJson::Serializer serializer;
  //serializer.setIndentMode(QJson::IndentFull);

  QVariantMap map;
  QString className  = QString (m->metaObject()->className() ).toLower();

  QStringList namespaces = className.split ("::");
  m->addIgnoredProperty ("@property.type");
  m->addIgnoredProperty ("objectName");
  foreach (QString s, ignoredProps) {
    m->addIgnoredProperty (s);
  }
  map[namespaces.at (namespaces.length() - 1)] =  m->serialize();

  QByteArray array = serializer.serialize (map);
  dest->append (array);
}


DQB_EXPORT_PLUGIN(QJSONMarshallableFormater)
