#include "Qt5JSONMarshallableFormater.hpp"

#include <QJsonDocument>
#include <DoQuBook/Exceptions.hpp>
#include <assert.h>

bool Qt5JSONMarshallableFormater::accept(const QString&)
{return true;}

QString Qt5JSONMarshallableFormater::contentType()
{return "application/json";}

void Qt5JSONMarshallableFormater::addIgnoredProp(const QString& prop) {
  ignoredProps.append (prop);
}

void Qt5JSONMarshallableFormater::unMarshall(const QByteArray& file, QSharedPointer<Marshallable> dest) {
  QSharedPointer<Marshallable> superMarshall = dest;

  QJsonDocument doc = QJsonDocument::fromJson(file);
  QVariantMap map = doc.toVariant().toMap();

  if (map.isEmpty()) {
    throw DoQuBook::CouldNotParseJSONException ("Failed JSON Parsing", file);
  }

  QList<MarshallablePtr> seeAlso = superMarshall->seeAlso();

  if (!seeAlso.empty() ) {
    foreach (MarshallablePtr marshal, seeAlso) {

      QString s  = QString (marshal->metaObject()->className() ).toLower();
      QStringList names = s.split ("::");

      if (map.contains (names.at (names.length() - 1) ) ) {
        superMarshall = marshal;
      }
    }
  }

  QString entityName = superMarshall->entityName();
  QVariantMap bookMap = map[entityName].toMap();

  dest->from (bookMap);
}

void Qt5JSONMarshallableFormater::marshall(QSharedPointer<Marshallable> m, QString* dest) {
  assert (dest);

  QVariantMap map;
  QString className  = QString (m->metaObject()->className() ).toLower();

  QStringList namespaces = className.split ("::");
  m->addIgnoredProperty ("@property.type");
  m->addIgnoredProperty ("objectName");
  foreach (QString s, ignoredProps) {
    m->addIgnoredProperty (s);
  }
  map[namespaces.at (namespaces.length() - 1)] =  m->serialize();


  QJsonDocument doc = QJsonDocument::fromVariant(map);
  QByteArray array = doc.toJson();
  dest->append (array);
}


DQB_EXPORT_PLUGIN(QJSONMarshallableFormater)
