#include "AlgorithmOnTask.hpp"

using namespace DoQuBook;

AlgorithmOnTask::AlgorithmOnTask (QObject * /*parent*/)
  : PluginBase (0)
{}

AlgorithmOnTask::AlgorithmOnTask()
  : PluginBase (0)
{}

AlgorithmOnTask::~AlgorithmOnTask()
{
}

bool AlgorithmOnTask::accept (TaskPtr t)
{
  return t->getExecutorID() == organisationName() + "." + algorithmName() + "." + version();
}


void AlgorithmOnTask::registerExecutor (RepositoryPtr & repo, Daemon daemonToken)
{
  ExecutorPtr executor = Executor::create();
  executor->setId (organisationName() + "." + algorithmName() + "." + version() );
  executor->setDescription (description() );
  executor->setReturnType (returnType() );
  executor->setPluginType (pluginType() );

  QList<ExecutorParameterPtr> methodParamsName = methodParameters();
  foreach (ExecutorParameterPtr p, methodParamsName) {
    executor->addParam (p);
  }
  // Les autre param :
  QList<ExecutorParameterPtr> paramsName = parameters();
  foreach (ExecutorParameterPtr p, paramsName) {
    executor->addParam (p);
  }
  DaemonServicePtr daemonService = repo->getDaemonService();
  daemonService->registerExecutor (executor, daemonToken);
}


void AlgorithmOnTask::runTask (TaskPtr t, RepositoryPtr & repo)
{
  m_repository = repo;
  try {
    // method parameters
    QVariantMap pluginMethodParams = retreiveParameters (methodParameters(), t);

    // Les autres params :
    QVariantMap pluginParams = retreiveParameters (parameters(), t);

    QMapIterator<QString, QVariant> i(pluginParams);
    QVariantMap changedParams;
    while (i.hasNext()) {
      i.next();
      qDebug() << i.key() << ": " << i.value();
      QVariantMap changedParam = onParameterChanged(i.key(), i.value());
      QMapIterator<QString, QVariant> c(changedParam);
      while (c.hasNext()) {
        c.next();
        changedParams[c.key()] = c.value();
      }
    }
    if(changedParams.keys().size()>0) {
      QMapIterator<QString, QVariant> i2(pluginParams);
      while (i2.hasNext()) {
        i2.next();
        QString paramName = i2.key();
        if(pluginParams[paramName] != changedParams[paramName]) {
          TaskPtr t2 = t;
          t2->setParam(paramName, changedParams[paramName]);

          DoQuBook::TaskServicePtr taskService = m_repository->getTaskService();
          qDebug() << "REFRESH the form !!";;

          t2->setResult ("");
          taskService->refreshForm(t2);

          return;
        }
      }
    }


    DoQuBook::TaskServicePtr taskService = m_repository->getTaskService();
    t->setStatus (Task::Accepted);
    t->setResult ("");
    taskService->notifyAccepted (t);

    QVariant bin = compute (pluginMethodParams, pluginParams);
    updateTaskResult (bin, t);

    taskService->setFinalResult (t);
  }
  catch (MessageException e) {
    DoQuBook::TaskServicePtr taskService = m_repository->getTaskService();
    t->setResult ("");
    t->setStatus (Task::Fail);
    t->setErrorMessage (e.message() );
    taskService->notifyHasFailed (t);
    return ;
  }
}

QVariantMap AlgorithmOnTask::retreiveParameters (const QList<ExecutorParameterPtr> &params, const TaskPtr & task)
{
  QVariantMap variantParams;

  foreach (ExecutorParameterPtr p, params) {
    buildQVariantMapsParameters (p, task, variantParams);
  }
  return variantParams;
}


void DoQuBook::AlgorithmOnTask::buildQVariantMapsParameters (const ExecutorParameterPtr & executorParameter, const TaskPtr & task, QVariantMap & params)
{
  if (executorParameter->getRequired() ) {
    if (!task->hasParam (executorParameter->getName() ) ) {
      throw MissingMarameterException (executorParameter->getName() );
    }
    else {
      TaskParameterPtr taskParameter = task->getParam (executorParameter->getName() );

      if (taskParameter->getParameterType() != executorParameter->getParameterType() ) {
        throw MessageException (taskParameter->getName() + "  is of wrong type [" + taskParameter->getParameterType() + "]... " + executorParameter->getParameterType() + " waited");
      }


      params[executorParameter->getName()] = taskParameter->getValue();
    }
  }
  else {
    if (task->hasParam (executorParameter->getName() ) ) {

      TaskParameterPtr taskParameter = task->getParam (executorParameter->getName() );
      if (taskParameter->getParameterType() != executorParameter->getParameterType() ) {
        throw MessageException (taskParameter->getName() + "  is of wrong type [" + taskParameter->getParameterType() + "]... " + executorParameter->getParameterType() + " waited");
      }

      params[executorParameter->getName()] = taskParameter->getValue();
    }
  }
}

void AlgorithmOnTask::updateTaskResult (const QVariant & result, TaskPtr & t)
{
  t->setResult (result);
  t->setStatus (Task::Done);
}

