#ifndef ALGORITHMONTASK_HPP
#define ALGORITHMONTASK_HPP

#include <DoQuBook/DoQuBook_global.hpp>

#include <QString>

#include <DoQuBook/Task.hpp>
#include <QDebug>
#include <QSharedPointer>
#include <QDebug>

#include <DoQuBook/Repository.hpp>

#include <DoQuBook/Executor.hpp>
#include <DoQuBook/PluginBase.hpp>


namespace DoQuBook
{

  /**
 * @brief The AlgorithmOnTask class is an abstract class from which all remote-able plugins inherits.
 *It provides utility function for converting a Task (parameters and results to QVariant objects) and calling the appropriate plugin method.
 */
  class DoQuBookSHARED_EXPORT AlgorithmOnTask : public PluginBase
  {
      Q_OBJECT
    public:
      AlgorithmOnTask();
      AlgorithmOnTask (QObject * parent);
      virtual ~AlgorithmOnTask();

      virtual QList<ExecutorParameterPtr> parameters () = 0;
      virtual QList<ExecutorParameterPtr> methodParameters () = 0;

      virtual bool accept (DoQuBook::TaskPtr t);

      virtual void runTask (DoQuBook::TaskPtr t, DoQuBook::RepositoryPtr & repo);

      /// Please implement this function to create your own task
      virtual QVariant compute (const QVariantMap & pluginMethodParams, const QVariantMap & params = QVariantMap() ) = 0;

      virtual void registerExecutor (DoQuBook::RepositoryPtr & repo, Daemon daemonToken) ;

    public slots :
      // Returns the new calculated parameters.
      virtual QVariantMap onParameterChanged(const QString &/*name*/, const QVariant &/*var*/)
      {QVariantMap m; return m;}
    signals:
      void autoParametersCalculated(QVariantMap &m);


    private :
      QVariantMap retreiveParameters (const QList<ExecutorParameterPtr> &params, const TaskPtr & task);
      void buildQVariantMapsParameters (const ExecutorParameterPtr & executorParameter, const TaskPtr & task, QVariantMap & params);
      void updateTaskResult (const QVariant & result, TaskPtr & t);

    protected :
      RepositoryPtr m_repository;
  };

}


#endif // ALGORITHMONTASK_HPP
