/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#include <DoQuBook/AnnotatedPolygon.hpp>


using namespace DoQuBook;


DoQuBook::AnnotatedPolygon::AnnotatedPolygon()
    : m_id (-1)
{
}

bool AnnotatedPolygon::equals (const AnnotatedPolygon & autre)
{

    qDebug() << "points1 size = " << _points.size() << "autre =" << autre._points.size();
    if (_points.size() != autre._points.size() ) {
        return false;
    }

    QListIterator<PointPtr> polyIter (_points);
    bool equal = true;
    while (equal && polyIter.hasNext() ) {
        PointPtr p = polyIter.next();
        bool found = false;

        QListIterator<PointPtr> polyIterAutre (autre._points);
        while (!found && polyIterAutre.hasNext() ) {
            PointPtr autrePoint = polyIterAutre.next();
            found = p->equals (*autrePoint);
        }
        equal = found;
        if (!found) {
            return false;
        }
    }

    return (equal && (_value == autre._value) );
}

bool AnnotatedPolygon::pointsMatches (const AnnotatedPolygon & autre)
{

    if (_points.size() != autre._points.size() ) {
        return false;
    }

    QListIterator<PointPtr> polyIter (_points);
    while (polyIter.hasNext() ) {
        PointPtr p = polyIter.next();
        bool found = false;

        QListIterator<PointPtr> polyIterAutre (autre._points);
        while (!found && polyIterAutre.hasNext() ) {
            PointPtr autrePoint = polyIterAutre.next();
            found = p->equals (*autrePoint);
        }

        if (!found) {
            return false;
        }
    }

    return true;
}

QPolygonF AnnotatedPolygon::toQPolygonF() const
{
    QPolygonF p;
    foreach (PointPtr point, _points) {
        p.append (point->toQPointF() );
    }

    return p;
}

/*bool DoQuBook::AnnotatedPolygon::merge (MarshallablePtr marsh)
{
    AnnotatedPolygonPtr poly = marsh.dynamicCast<AnnotatedPolygon>();
    if(!poly.isNull()) {
        if(getId() == poly->getId()) {
            /// Modify annotated Polygon attributes
            setValue (poly->getValue ());

            /// For sub marshalables
            foreach(PointPtr point, poly->getPoints()) {
                /// Look if a point has been added
                PointPtr samePoint = findPoint (point);
                if(samePoint.isNull ())
                {
                    /// if not add it
                    addPoint (point);
                }
                else {
                    /// merge the modifications
                    samePoint->merge (point);
                }
            }
        }
    }
    return true;
}
*/
DoQuBook::PointPtr DoQuBook::AnnotatedPolygon::findPoint (DoQuBook::PointPtr p)
{
    bool found = false;

    QListIterator<PointPtr> it (_points);

    PointPtr foundPoint;
    while (!found && it.hasNext () ) {
        foundPoint = it.next ();
        found = foundPoint->equals (*p);
    }
    if (found) {
        return foundPoint;
    }
    else {
        return PointPtr();
    }
}

QList<DoQuBook::PointPtr> DoQuBook::AnnotatedPolygon::getPoints() const
{
    return _points;
}

QVariant DoQuBook::AnnotatedPolygon::getValue() const
{
    return _value;
}

QString DoQuBook::AnnotatedPolygon::valueQString() const
{
    return value <QString> ();
}

int DoQuBook::AnnotatedPolygon::valueInt() const
{
    return value <int> ();
}

void DoQuBook::AnnotatedPolygon::setValue (QVariant value)
{
    _value = value;
    emitModified();
}

DoQuBook::PointPtr DoQuBook::AnnotatedPolygon::getPoint (int i) const
{
    return _points[i];
}

void DoQuBook::AnnotatedPolygon::setPoints (QList<DoQuBook::PointPtr> p)
{
    _points = p;
    foreach (PointPtr point, p) {
        connect (point.data(), SIGNAL (modified() ),
                 this, SLOT (emitModified() ) );
    }
    emitModified();
}

void DoQuBook::AnnotatedPolygon::addPoint (DoQuBook::PointPtr p)
{
    _points.push_back (p);
    connect (p.data(), SIGNAL (modified() ),
             this, SLOT (emitModified() ) );
    emitModified();
}

bool DoQuBook::AnnotatedPolygon::operator == (const DoQuBook::AnnotatedPolygon & b)
{
    return equals (b);
}

int DoQuBook::AnnotatedPolygon::getId() const
{
    return m_id;
}

void DoQuBook::AnnotatedPolygon::setId (int arg)
{
    if (m_id != arg) {
        m_id = arg;
        emit idChanged (arg);
    }
}

void DoQuBook::AnnotatedPolygon::merge (MarshallablePtr other)
{
    AnnotatedPolygonPtr poly = other.dynamicCast<AnnotatedPolygon>();
    if (getId() == -1) {
        setId (poly->getId() );
    }

    if (!poly.isNull() && poly->getId() == getId() ) {
        setValue (poly->getValue() );
        foreach (PointPtr p , poly->getPoints() ) {
            PointPtr existing = findPoint (p);
            if (existing.isNull() ) {
                qDebug() << "Could not find point when merging";
                addPoint (p);
            }
            else {
                existing->merge (p);
            }
        }
    }
}
