/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#include "AnnotationGlobal.hpp"

using namespace DoQuBook;


AnnotationGlobal::AnnotationGlobal()
  : m_type ("annotationGlobal")
{
}

AnnotationGlobal::AnnotationGlobal (const QString & name)
  : Annotation (name)
{
  _name = name;
  setType ("annotationGlobal");
}

QList<QVariant> AnnotationGlobal::getValueAsList()
{
  QList<QVariant> list;
  QVariant v = getValue();
  QStringList sList = v.toString().split (";");
  foreach (QString s, sList)
    list << s;
  return list;
}

QMap<QString, QVariant> AnnotationGlobal::getValueAsMap()
{
  QMap<QString, QVariant> list;
  QVariant v = getValue();
  QStringList sList = v.toString().split ("|");
  foreach (QString s, sList) {
    QStringList listValues = s.split (";");
    list[listValues.at (0)] = listValues.at (1);
  }

  return list;
}

void AnnotationGlobal::accept (AnnotationVisitor & visitor)
{
  visitor.visit (this);
}


bool AnnotationGlobal::equals (const DoQuBook::Annotation & autre)
{
  const AnnotationGlobal * global = dynamic_cast<const AnnotationGlobal *> (&autre);
  if (!global) {
    return false;
  }

  return _value == global->_value && Annotation::equals (autre);
}

void DoQuBook::AnnotationGlobal::merge (MarshallablePtr ann)
{
  Annotation::merge (ann);
  AnnotationGlobalPtr global = ann.dynamicCast<AnnotationGlobal>();
  if (!global.isNull () ) {
    setValue (global->getValue () );
  }
}

void DoQuBook::AnnotationGlobal::setValue (QVariant value)
{
  _value = value;
  emitModified();
}

void AnnotationGlobal::setValue (const QList<QVariant> &values)
{
  QString s;
  for (int i = 0; i < values.size() - 1; i++) {
    if (values[i].canConvert (QVariant::String) ) {
      s.append (values[i].toString() );
      s.append (";");
    }
  }
  if (values[values.size() - 1].canConvert (QVariant::String) ) {
    s.append (values[values.size() - 1].toString() );
  }
  setValue (s);
}

void AnnotationGlobal::setValue (const QMap<QString, QVariant> &values)
{
  QString s;
  QMapIterator<QString, QVariant> it (values);
  int i = 0;
  while (it.hasNext() && i < values.size() - 1) {
    it.next();
    QVariant value = it.value();
    if (value.canConvert (QVariant::String) ) {
      QString k = it.key();
      s.append (k);
      QString val = value.toString();
      s.append (";");
      s.append (val);
      s.append ("|");
    }
    i++;
  }
  it.next();
  QVariant value = it.value();
  if (value.canConvert (QVariant::String) ) {
    QString k = it.key();
    s.append (k);
    QString val = value.toString();
    s.append (";");
    s.append (val);
  }
  setValue (s);
}




