/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#ifndef ANNOTATIONGLOBAL_HPP
#define ANNOTATIONGLOBAL_HPP

#include <DoQuBook/Annotation.hpp>
#include <QDebug>

namespace DoQuBook
{



  MARSHALLABLE_DECLARE_POINTER (AnnotationGlobal)


  /*! \class AnnotationGlobal
         * \brief An annotation with a value
         *
         * The class provides access to the value of a global annotation
         */
  class DoQuBookSHARED_EXPORT AnnotationGlobal : public Annotation
  {
                                                          Q_OBJECT

                                                          Q_PROPERTY (QVariant value READ getValue WRITE setValue)

                                                          /// This property is duplicated in all Annotation SubClass (Introspection issues)
                                                          /// if property type is in Annotation : type == Annotation ... and we want the real type
                                                          Q_PROPERTY (QString type READ getType WRITE setType)

                                                          public:

                                                          MARSHALLABLE_EXTENDS (Annotation)

                                                          MARSHALLABLE_CREATE_FONCTIONS (AnnotationGlobal)

                                                          AnnotationGlobal ();
  AnnotationGlobal (const QString & name);



  QVariant getValue() const {
    return _value;
  }

  /**
              * Value must be formated as : value1;value2;value3
              */
  QList<QVariant> getValueAsList();
  /**
              * Value must be formated as : key1;value1|key2|value2|key3;value3
              */
  QMap<QString, QVariant> getValueAsMap();

  void setValue (QVariant value);

  /**
              * Value will me formated as : value1;value2;value3
              * \warning only variant that canConvert(String) will be added
              * see http://developer.qt.nokia.com/doc/qt-4.8/qvariant.html#id-7faf8213-8d97-4ca6-b16e-ffbf4ca55316
              */
  void setValue (const QList<QVariant> &values);

  /**
              * Value will me formated as : key1;value1|key2|value2|key3;value3
              * \warning only variant that canConvert(String) will be added
              * see http://developer.qt.nokia.com/doc/qt-4.8/qvariant.html#id-7faf8213-8d97-4ca6-b16e-ffbf4ca55316
              */
  void setValue (const QMap<QString, QVariant> &values);

  virtual void accept (AnnotationVisitor & visitor);
  virtual QStringList listProperties() const {
    return QStringList();
  }

  virtual void merge (MarshallablePtr ann);

  bool equals (const Annotation & autre);

  QString getType() const {
    return m_type;
  }

  public slots:
  void setType (QString arg) {
    m_type = arg;
  }

  private:
  QVariant _value;
  QString m_type;
};


} // namespace

Q_DECLARE_METATYPE ( DoQuBook::AnnotationGlobalPtr )

#endif // ANNOTATIONGLOBAL_HPP
