/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#ifndef ANNOTATIONVISITOR_HPP
#define ANNOTATIONVISITOR_HPP


namespace DoQuBook
{
  class AnnotationGlobal;
  class AnnotationLocal;

  /*! \brief An abstract class to process annotation without the need to cast them.
     *  See visitor pattern for more infos.
     *
     * Example usage :
     \code
class AnnotationShowVisitor : AnnotationVisitor {
   public :
        virtual void visit (AnnotationGlobal* annotationGlobal) {
            qDebug() <<  annotationGlobal->getName() << annotationGlobal->getValue();
        }
        virtual void visit (AnnotationLocal* annotationLocal) {
            qDebug() <<  annotationLocal->getName();
            foreach (AnnotatedPolygonPtr annPoly, annotationLocal->getAnnotatedPolygons()) {
                qDebug() << annPoly->getId() << annPoly->getValue();
            }
        }

};
/// Usage of AnnotationShowVisitor with AnnotationService
AnnotationShowVisitor v;
AnnotationIteratorPtr it = service->getAnnotationOfPage(page);
while(it->hasNext()) {
  AnnotationPtr ann = it->next();
  ann->accept(v);
}
     \endcode
     */
  class AnnotationVisitor
  {
    public :
      virtual ~AnnotationVisitor() {}
      virtual void visit (AnnotationGlobal* annotationGlobal) = 0;
      virtual void visit (AnnotationLocal* annotationLocal) = 0;
  };

} // namespace

#endif // ANNOTATIONVISITOR_HPP
