#include "AttachmentServices.hpp"

#include <QtNetwork>
#include <QNetworkReply>
#include <QNetworkRequest>
#include <QNetworkAccessManager>
#include <QBuffer>
#include <DoQuBook/WebClient.hpp>

using namespace DoQuBook;

AttachmentServices::AttachmentServices (QString serverAdress)
    : server_adress (serverAdress)
    , m_attachmentImageManager (RESTImageManager::JPEG)
{
    rest_service_url = "DoQuBookWeb/rest";
    attachments_ressource = "attachment";

    QString getUrl (server_adress + "/" + rest_service_url + "/" + attachments_ressource + "/(id)/(imageName)");
    m_attachmentImageManager.addRessourcePath (WebClient::GET, getUrl);

    QString url (server_adress + "/" + rest_service_url + "/" + attachments_ressource + "/(id)");
    m_attachmentImageManager.addRessourcePath (WebClient::POST, url);

}

QUrl DoQuBook::AttachmentServices::addImage (QImage image, DoQuBook::PagePtr page, QString imageName)
{
    QMap<QString, QString> params;
    params["id"] = QString::number (page->getId () );

    m_attachmentImageManager.post (image, imageName, params);

    return getImageUrl (page, imageName);
}

QImage DoQuBook::AttachmentServices::getImage (DoQuBook::PagePtr page, QString imageName)
{
    QMap<QString, QString> params;
    params["id"] = QString::number (page->getId () );
    params["imageName"] = imageName;

    QImage img = m_attachmentImageManager.get (params);

    return img;
}

QUrl DoQuBook::AttachmentServices::getImageUrl (PagePtr page, QString imageName)
{
    return QUrl (server_adress + "/" + rest_service_url + "/" + attachments_ressource + "/" + QString::number (page->getId() ) + "/" + imageName);
}

