/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */#include "BookImporterRegExp.hpp"
#include <QRegExp>

using namespace DoQuBook;

BookImporterRegExp::BookImporterRegExp (const QString & dirPath, const QString & pRegExp, const QString & extention)
    : bookDir (dirPath)
    , ext (extention)
    , regExp (pRegExp)
{
    regExpTranslationMap["bookname"] = "(.*)";
    regExpTranslationMap["pagenumber"] = "(\\d+)";
    regExpTranslationMap["ext"] = "\\.";

    foreach (QString key, regExpTranslationMap.keys() ) {
        regExp = regExp.replace (key, regExpTranslationMap[key]);
    }
    regExp = regExp + "." + ext;

}

DoQuBook::BookPtr DoQuBook::BookImporterRegExp::import()
{
    BookPtr book = Book::create();
    book->setName (bookDir.dirName() );

    foreach (QFileInfo file, bookDir.entryInfoList (QDir::NoDotAndDotDot | QDir::Files) ) {

        if (file.completeSuffix() == ext) {
            PagePtr page = Page::create();
            int pos = 0;
            QRegExp rx (regExp);
            while ( (pos = rx.indexIn (file.fileName(), pos) ) != -1) {
                page->setName (rx.cap (1) + rx.cap (2) );
                if (book->getName() != rx.cap (1) ) {
                    book->setName (rx.cap (1) );
                }

                pos += rx.matchedLength();
            }
            page->setImage (file.absoluteFilePath () );
            book->addPage (page);
        }
    }
    return book;
}
