#include "DBWebDaemonService.hpp"
#include <DoQuBook/WebClient.hpp>
#include <QDebug>
#include <DoQuBook/Exceptions.hpp>


using namespace DoQuBook;

DBWebDaemonService::DBWebDaemonService (const QString & serverAdress)
    : m_serverUrl (serverAdress)
    , m_rest_service_url ("DoQuBookWeb/rest/daemon")
    , m_downloadTokenUrl (m_serverUrl + "/DoQuBookWeb/rest/daemon/register")
    , m_executorManager (WebClient::JSON)
    , m_taskIdsManager (WebClient::JSON)
{

    QString postUrl (m_serverUrl + "/" + m_rest_service_url + "/(daemonToken)");
    m_executorManager.addRessourcePath (WebClient::POST, postUrl);

    QString getUrl (m_serverUrl + "/" + m_rest_service_url + "/(daemonToken)/(executorId)");
    m_executorManager.addRessourcePath (WebClient::GET, getUrl);

    //    QString getAll (m_serverUrl+"/"+m_restServicesUrl);
    //    m_idsManager.addRessourcePath(WebClient::GET, getAll);

    QString putUrl (m_serverUrl + "/" + m_rest_service_url + "/(daemonToken)");
    m_executorManager.addRessourcePath (WebClient::PUT, putUrl);


    QString getNewUrl (m_serverUrl + "/" + m_rest_service_url + "/(daemonToken)/new");
    m_taskIdsManager.addRessourcePath (WebClient::GET, getNewUrl);

    QString delUrl (m_serverUrl + "/" + m_rest_service_url + "/(daemonToken)");
    m_executorManager.addRessourcePath (WebClient::DEL, delUrl);

}

DBWebDaemonService::~DBWebDaemonService()
{
}

Daemon DBWebDaemonService::downloadToken()
{
    WebClient c;
    c.setContentType (WebClient::JSON);
    QByteArray response = c.sendSynchronous (m_downloadTokenUrl, WebClient::GET, "");
    return response;
}

void DBWebDaemonService::registerExecutor (ExecutorPtr & exec, const Daemon & d)
{
    QMap<QString, QString> params;
    params["daemonToken"] = d;

    if (!isRegisteredExecutor (exec, d) ) {
        exec = m_executorManager.post (exec, params);
    }
}

bool DBWebDaemonService::unRegister (const Daemon & d)
{
    QMap<QString, QString> params;
    params["daemonToken"] = d;

    m_executorManager.del (params);
    return true;
}

bool DBWebDaemonService::isRegisteredExecutor (ExecutorPtr exec, const Daemon & d)
{
    QMap<QString, QString> params;
    params["executorId"] = exec->getId();
    params["daemonToken"] = d;

    try {
        return !m_executorManager.get (params).isNull();
    }
    catch (CouldNotParseJSONException e) {
        qCritical()  << e.getFailedJSONString();
        return false;
    }
}

meta_iterator<TaskPtr>::IteratorPtr DBWebDaemonService::getNewTasks (const Daemon & d)
{
    QMap<QString, QString> params;
    params["daemonToken"] = d;

    TaskRestIteratorPtr it = m_taskIdsManager.get (params);
    return it;
}
