#include "DBWebImageService.hpp"

#include <QDir>
#include <DoQuBook/Logger.hpp>
#include <DoQuBook/ApplicationContext.hpp>

using namespace DoQuBook;

DBWebImageService::DBWebImageService (const QString & server_url)
    : m_serverAdress (server_url)
    , m_pageImageManager (RESTImageManager::JPEG)
{

    rest_service_url = "DoQuBookWeb/rest";
    book_ressource = "book";
    image_ressource = "image";


    QString getImageUrl (m_serverAdress + "/" + rest_service_url + "/" + image_ressource + "/(id)" + "?width=(width)");
    m_pageImageManager.addRessourcePath (WebClient::GET, getImageUrl);

    QString postImageUrl (m_serverAdress + "/" + rest_service_url + "/" + image_ressource + "/(id)");
    m_pageImageManager.addRessourcePath (WebClient::POST, postImageUrl);
}


QImage DoQuBook::DBWebImageService::get (const PagePtr & page, int w)
{
    QImage i = getImageOnRemoteServer (page, w);
    return i;

}

QImage DoQuBook::DBWebImageService::getImageOnRemoteServer (const PagePtr & page, int w)
{
    QMap<QString, QString> params;
    params["id"] = QString::number (page->getId() );
    params["width"] = QString::number (w);

    return m_pageImageManager.get (params);
}

void DoQuBook::DBWebImageService::upload (const QImage & image, const DoQuBook::PagePtr & page)
{
    QMap<QString, QString> params;
    params["id"] = QString::number (page->getId () );

    m_pageImageManager.post (image, "original.jpeg", params);

}
