#include "DBWebPageService.hpp"


#include <QFileInfo>

using namespace DoQuBook;

DBWebPageService::DBWebPageService (const QString & serverAdress)
    : DBWebCRUDService<Page, int> (serverAdress, "page", "id")
    , m_pageImageManager (RESTImageManager::JPEG)
    , m_pageByNameManager (WebClient::JSON)
    , m_getAllByChunksManager (WebClient::JSON)
    , m_listRestAttachmentsManager (WebClient::JSON)
    , m_pageByTagManager (WebClient::JSON)
    , m_getAllTagsManager (WebClient::JSON)
{
    QString getImageUrl (m_server_adress + "/" + m_rest_service_url + "/image/(id)?width=(width)");
    m_pageImageManager.addRessourcePath (WebClient::GET, getImageUrl);

    QString postImageUrl (m_server_adress + "/" + m_rest_service_url + "/image/(id)");
    m_pageImageManager.addRessourcePath (WebClient::POST, postImageUrl);

    QString getIdsByName (m_server_adress + "/" + m_rest_service_url + "/page/byName/(name)");
    m_pageByNameManager.addRessourcePath (WebClient::GET, getIdsByName);

    QString getByTag (m_server_adress + "/" + m_rest_service_url + "/page/withTag/(tag)");
    m_pageByTagManager.addRessourcePath(WebClient::GET, getByTag);

    QString getAllByChunks ( m_server_adress + "/" + m_rest_service_url + "/page/pagebychunk");
    m_getAllByChunksManager.addRessourcePath (WebClient::GET, getAllByChunks);

    QString getAttachments ( m_server_adress + "/" + m_rest_service_url + "/attachment/ofPage/(pageId)");
    m_listRestAttachmentsManager.addRessourcePath (WebClient::GET, getAttachments);

    QString gettags ( m_server_adress + "/" + m_rest_service_url + "/page/tags");
    m_getAllTagsManager.addRessourcePath(WebClient::GET, gettags);
}

DBWebPageService::~DBWebPageService()
{
}

meta_iterator<PagePtr>::IteratorPtr DBWebPageService::getAll()
{
    QMap<QString, QString> params;
    PageRestIteratorPtr it = m_getAllByChunksManager.get (params);
    return it;
}

ListRestFileAttachmentPtr DBWebPageService::getAllAttachments (const PagePtr p)
{
    QMap<QString, QString> params;
    params["pageId"] = QString::number (p->getId() );
    ListRestFileAttachmentPtr it = m_listRestAttachmentsManager.get (params);
    return it;
}

void DBWebPageService::addAttachment (PagePtr page, const QFileInfo & fileInfo)
{
    WebClient wc;
    if (fileInfo.suffix() == "xml") {
        wc.setContentType (WebClient::XML_MULTIPART);
        QUrl url = m_server_adress + "/DoQuBookWeb/rest/attachment/addToPage/" + QString::number (page->getId() );
        QFile file (fileInfo.absoluteFilePath() );
        if (file.open (QFile::ReadOnly) ) {
#if DEBUG_LOG
            qDebug() << "POST at url " << url;
#endif
            QByteArray content = file.readAll();
#if DEBUG_LOG
            qDebug() << "content of " << fileInfo.absoluteFilePath();
#endif
            QByteArray response = wc.sendSynchronous (url, WebClient::POST, content , fileInfo.fileName() );
#if DEBUG_LOG
            qDebug() << "Reponse " << response;

#endif
        }
    }
    else if (fileInfo.suffix() == "zip") {
        wc.setContentType (WebClient::ZIP_MULTIPART);
        QUrl url = m_server_adress + "/DoQuBookWeb/rest/attachment/addToPage/" + QString::number (page->getId() );
        QFile file (fileInfo.absoluteFilePath() );
        if (file.open (QFile::ReadOnly) ) {
#if DEBUG_LOG
            qDebug() << "POST at url " << url;
#endif
            QByteArray content = file.readAll();
#if DEBUG_LOG
            qDebug() << "content of " << fileInfo.absoluteFilePath();
#endif
            QByteArray response = wc.sendSynchronous (url, WebClient::POST, content , fileInfo.fileName() );
#if DEBUG_LOG
            qDebug() << "Reponse " << response;

#endif
        }
    }
    else {
        qCritical() << "Unsuported file type : " + fileInfo.suffix();
        throw MessageException ("Unsuported file type : " + fileInfo.suffix() );
    }
}

PageIteratorPtr DBWebPageService::getByName(const QString & name)
{
    QMap<QString, QString> params;
    params["name"] = name;

    PageRestIteratorPtr it = m_pageByNameManager.get (params);
    return it;
}

PagePtr DBWebPageService::findByName(const QString& name)
{
    meta_iterator<PagePtr>::IteratorPtr it = getByName(name);
    PagePtr found;
    while (it->hasNext() && found.isNull()) {
      PagePtr current = it->next();
      if(current->getName() == name) {
        found = current;
      }
    }
    return found;
}

bool DBWebPageService::existPageByName (const QString & name)
{
    QString existPageByNameUrl (m_server_adress + "/" + m_rest_service_url + "/page/existsByName/" + name);
    try {
        WebClient c;
        c.sendSynchronous (existPageByNameUrl, WebClient::GET, "");
        return true;
    }
    catch (ContentNotFoundException e) {
        return false;
    }
}

QStringList DBWebPageService::existingTags()
{
    QMap<QString, QString> params;
    StringListPtr list = m_getAllTagsManager.get(params);
    return list->getStrings();
}

meta_iterator<PagePtr>::IteratorPtr DBWebPageService::findByTag(const QString &tag)
{
    QMap<QString, QString> params;
    params["tag"] = tag;

    PageRestIteratorPtr it = m_pageByTagManager.get (params);
    return it;
}

void DBWebPageService::addTag(const PagePtr &p, const QString &tag)
{
    WebClient c;
    c.setContentType(WebClient::JSON);
    QString url =  m_server_adress + "/" + m_rest_service_url + "/page/addTag/"+tag+"/to/"+QString::number(p->getId());
    c.sendSynchronous(url,WebClient::GET,"");
}

void DBWebPageService::removeTag(const PagePtr &p, const QString &tag)
{
    WebClient c;
    c.setContentType(WebClient::JSON);
    QString url =  m_server_adress + "/" + m_rest_service_url + "/page/removeTag/"+tag+"/from/"+QString::number(p->getId());
    c.sendSynchronous(url,WebClient::GET,"");
}

PageEvent* DBWebPageService::onPageTaggedEvent(const QString &tag, QList< Functor<Page>* > functors)
{
    QString subscriptionUrl = m_server_adress + "/" + m_rest_service_url + "/page/events/tagged/"+tag;
    DBWebPageEvent * e = new DBWebPageEvent(m_server_adress, subscriptionUrl, NULL);
    foreach(Functor<Page>* f, functors) {
        e->addFunctor(f);
    }
    return e;
}

#ifdef HAVE_CPP11
void DBWebPageService::onPageTaggedEvent(const QString& tag, std::function<void (PagePtr)> func)
{
  PageEvent *e = onPageTaggedEvent(tag);
  QObject::connect(e, &PageEvent::processSignal, func);
}
#endif

Future<Page> *DBWebPageService::onPageAdded (Functor<Page> *functor)
{
    HttpEventHandler<Page>* f3 = new HttpEventHandler<Page> ("http://localhost:8080/DoQuBookWeb/rest/page/events/add");
    f3->addObserver (functor);
    f3->listen();

    return NULL;
}

void DBWebPageService::preAdd (PagePtr & page)
{
    if (page->getImage().isNull() ) {
        throw new MessageException ("Cannot add page : image is null");
    }
    m_lastPageImagePath = page->getImageFilePath();

}

void DBWebPageService::postGet (PagePtr & page)
{
    page->installImageService (new DBWebImageService (m_server_adress) );
}

void DBWebPageService::postAdd (PagePtr & page)
{
    page->setImage (m_lastPageImagePath);

    uploadImage (page);

    page->installImageService (new DBWebImageService (m_server_adress) );
}

void DBWebPageService::postUpdate (PagePtr & page)
{
    page->installImageService (new DBWebImageService (m_server_adress) );
}

bool DBWebPageService::uploadImage (const PagePtr & page) throw (MessageException)
{
    QMap<QString, QString> params;
    params["id"] = QString::number (page->getId () );

    QImage image = page->getImage();

    if (image.isNull() ) {
        throw MessageException ("Image to upload is null");
    }

    QFileInfo file (page->getImageFilePath() );
    QString suffix = file.completeSuffix();

    RESTImageManager::ImageType type;
    if (suffix == "tiff" || suffix == "tif") {
        type = RESTImageManager::TIFF;
    }
    else if (suffix == "jpeg" || suffix == "jpg") {
        type = RESTImageManager::JPEG;
    }
    else if (suffix == "png") {
        type = RESTImageManager::PNG;
    }
    else  {
        qWarning() << "Image with extention " << suffix << " not yet supported ... Converting to JPEG";
        type = RESTImageManager::JPEG;
    }

    m_pageImageManager.post (image, "original." + suffix, params, type);

    return true;
}

QString DBWebPageService::convert (const int & id)
{
    return QString::number (id);
}

