#ifndef DBWEBPAGESERVICE_HPP
#define DBWEBPAGESERVICE_HPP

#include <DoQuBook/PageService.hpp>
#include <DoQuBook/DoQuBook_global.hpp>
#include <DoQuBook/RESTImageManager.hpp>

#include <DoQuBook/DBWebCRUDService.hpp>
#include <DoQuBook/Intlist.hpp>
#include <DoQuBook/StringList.hpp>

#include <DoQuBook/DBWebImageService.hpp>

#include <functional>


namespace  DoQuBook
{

    class ImageService;

    class DBWebPageService ;
    typedef QSharedPointer<DBWebPageService> DBWebPageServicePtr;

    class PageRestIterator;
    typedef QSharedPointer<PageRestIterator> PageRestIteratorPtr;


    class DoQuBookSHARED_EXPORT PageRestIterator : public SimpleMarshallable, public Iterator<PagePtr>
    {
            Q_OBJECT
        public :
            Q_PROPERTY (QList<PagePtr> elements READ getElements WRITE setElements)
            Q_PROPERTY (QUrl nextUrl READ getNextUrl WRITE setNextUrl)

            REST_ITERATOR (Page)

        public :
            void setServerAdress(const QString &serverAdress) {_serverAdress =serverAdress;}

        protected:
            virtual void configure (Iterator<PagePtr> *nextIt) {
                PageRestIterator *pageIt = dynamic_cast<PageRestIterator*>(nextIt);
                pageIt->setServerAdress(_serverAdress);
            }

            virtual void preNext(PagePtr &page)
            {
                page->installImageService (new DBWebImageService (_serverAdress) );
            }
        private :
            QString _serverAdress;
    };

    class DoQuBookSHARED_EXPORT DBWebPageService : public PageService, public DBWebCRUDService<Page, int>
    {
        public:
            DBWebPageService (const QString & serverAdress);
            virtual ~DBWebPageService();

            virtual meta_iterator<PagePtr>::IteratorPtr getAll();

            virtual ListRestFileAttachmentPtr getAllAttachments (const PagePtr p);
            virtual void addAttachment (const PagePtr page, const QFileInfo & fileInfo);

            virtual PageIteratorPtr getByName (const QString & name);
            virtual PagePtr findByName (const QString & name);
            virtual bool existPageByName (const QString & name);

            virtual QStringList existingTags();
            virtual PageIteratorPtr findByTag (const QString & tag);
            virtual void addTag(const PagePtr &p, const QString &tag);
            virtual void removeTag(const PagePtr &p, const QString &tag);
            virtual PageEvent* onPageTaggedEvent(const QString &tag, QList< Functor<Page>* > functors = QList< Functor<Page>* >());
#ifdef HAVE_CPP11
            virtual void onPageTaggedEvent(const QString &tag, std::function<void (PagePtr)> func);
#endif

            virtual Future<Page>* onPageAdded (Functor<Page> *functor);

        protected :
            virtual void preAdd (PagePtr & page);
            virtual void postGet (PagePtr & page);
            virtual void postAdd (PagePtr & page);
            virtual void postUpdate (PagePtr & page);

            virtual bool uploadImage (const PagePtr & page) throw (MessageException);

            virtual QString convert (const int & id);

        private :
            QString m_lastPageImagePath;
            RESTImageManager m_pageImageManager;
            RESTObjectManager<PageRestIterator> m_pageByNameManager;
            RESTObjectManager<PageRestIterator> m_getAllByChunksManager;
            RESTObjectManager<ListRestFileAttachment> m_listRestAttachmentsManager;
            RESTObjectManager<PageRestIterator> m_pageByTagManager;
            RESTObjectManager<StringList> m_getAllTagsManager;

    };

}
#endif // DBWEBPAGESERVICE_HPP
