#ifndef DEGRADATIONMODEL_HPP
#define DEGRADATIONMODEL_HPP

#include <QImage>
#include <QVariant>
#include <QSharedPointer>

#include <DoQuBook/AlgorithmOnTask.hpp>


class DoQuBookSHARED_EXPORT DegradationModel : public DoQuBook::AlgorithmOnTask
{
    public:
        DegradationModel();
        virtual ~DegradationModel();

        // --- To be implemented //
        virtual QString organisationName () = 0;
        virtual QString description () = 0;
        virtual QString algorithmName () = 0;
        virtual QString version() = 0;
        virtual QImage degradate (const QImage & image,  const QVariantMap & parameters = QVariantMap() ) = 0;
        // --- //

        virtual QList<DoQuBook::ExecutorParameterPtr> parameters () {
            QList<DoQuBook::ExecutorParameterPtr> params;
            return params;
        }

        virtual QString pluginType () {
            return "DegradationModel";
        }

        virtual QString returnType() {
            return "Image";
        }

        virtual QList<DoQuBook::ExecutorParameterPtr> methodParameters() {
            QList<DoQuBook::ExecutorParameterPtr> params;
            params << DoQuBook::ExecutorParameter::create ("image", "Image", true);
            return params;
        }

        virtual QVariant compute (const QVariantMap & pluginMethodParams, const QVariantMap & params = QVariantMap() ) {
            qDebug() << "Degradation !!! ";
            QImage image = pluginMethodParams["image"].value<QImage>();
            return degradate (image, params);
        }

};



typedef std_shared_ptr<DegradationModel>::type DegradationModelPtr;

Q_DECLARE_INTERFACE (DegradationModel, "doqubook.degradationmodel")

#endif // DEGRADATIONMODEL_HPP
