#include "DoQuBookDeamon.hpp"
#include <DoQuBook/Config.hpp>

#include <assert.h>

using namespace DoQuBook;

DoQuBookDeamon::DoQuBookDeamon (QCoreApplication & app, const QString & orgName, const QString & appName, const QString & exeName)
    : DoQuBookApp (app, orgName, appName, exeName)
    , stopped (false)
{
}

void DoQuBookDeamon::unregister()
{

    assert (repo);

    DaemonServicePtr daemonService = repo->getDaemonService();
    daemonService->unRegister (_token);
}

QList<DoQuBookApp::ProgramParameter> DoQuBook::DoQuBookDeamon::paramsAndDescriptions()
{
    QList<DoQuBookApp::ProgramParameter> paramsAndDesc;
    ProgramParameter page ("repoName", false, "Repository Name = the name of the repository (ex : Gallifrey)");
    paramsAndDesc.append (page);
    return paramsAndDesc;
}



#ifdef Q_WS_WIN
void my_sleep (int /*seconds*/)
{
    // TODO: Could not find sleep for windows with Visual Studio 9 2008
}
#else

#include <unistd.h>
void my_sleep (int seconds)
{
    sleep (seconds);
}

#endif


void DoQuBookDeamon::registerDaemon (QMap<QString, std_shared_ptr<AlgorithmOnTask>::type > &algos, DaemonServicePtr daemonService, QStringList & executors)
{
    bool connceted = false;
    while (!connceted && !stopped) {
        try {
            _token = daemonService->downloadToken();

            foreach (std_shared_ptr<AlgorithmOnTask>::type algo, algos) {
                algo->registerExecutor (repo, _token);
                executors << algo->organisationName() + "." + algo->algorithmName() + "." + algo->version();
            }

            connceted = true;
        }
        catch (InternetConnectionException e) {
            qWarning() << e.getMessage();
            connceted = false;
            my_sleep (10); // We wait 10 seconds
        }
        catch (NetworkConnectionException e) {
            qWarning() << e.message();
            connceted = false;
            my_sleep (10); // We wait 10 seconds
        }
        catch (ContentNotFoundException e) {
            qWarning() << e.message();
            connceted = false;
            my_sleep (10); // We wait 10 seconds
        }
        catch (DoQuBook::CouldNotParseJSONException e) {
            qWarning() << e.message();
            connceted = false;
            my_sleep (10); // We wait 10 seconds
        }
    }
}


/***** Should really refactor *////
void DoQuBookDeamon::runApp (QMap<QString, QString> args)
{
    if (args.contains ("repoName") ) {
        repo = ApplicationContext::instance()->getRepository (args["repoName"]);
    }
    else {
        qWarning() << "No repoName specified in command line";
        repo = ApplicationContext::instance()->getDefaultRepository();
    }

    PluginLoader * l = PluginLoader::instance();
    QStringList executors;
    QMap<QString, std_shared_ptr<AlgorithmOnTask>::type > algos = l->algorihmsOnTaskImageResource (false);

    TaskServicePtr taskService = repo->getTaskService();

    DaemonServicePtr daemonService = repo->getDaemonService();
    registerDaemon (algos, daemonService, executors);

    while (!stopped) {
        /// get all task

        meta_iterator<TaskPtr>::IteratorPtr itTasks;
        bool connceted = false;
        while (!connceted && !stopped) {
            try {
                itTasks = daemonService->getNewTasks (_token);

                connceted = true;
            }
            catch (NeedToRegister e) {
                qWarning() << "Need To register exception";
                qWarning() << "We must reregister daemon ";
                unregister();
                registerDaemon (algos, daemonService, executors);
            }

            catch (InternetConnectionException e) {
                qWarning() << "Internet connection exception";
                qWarning() << e.getMessage();
                qWarning() << "We must reregister daemon ";
                my_sleep (40); // We wait 10 seconds
                unregister();
                registerDaemon (algos, daemonService, executors);
            }
            /// AsyncResponse closed
            catch (NetworkConnectionException e) {
                qWarning() << "NetworkConnectionException e";
                qWarning() << e.message();
                qWarning() << "Retry QUICK !!!";
            }
            /// The server may have been rebooted.
            catch (ContentNotFoundException e) {
                qWarning() << "ContentNotFoundException e";
                qWarning() << e.message();
                //qWarning() << "Sleep 10 sec and retry";
                qWarning() << "We must reregister daemon ";
                my_sleep (40); // We wait 10 seconds
                unregister();
                registerDaemon (algos, daemonService, executors);
            }
            catch (DoQuBook::CouldNotParseJSONException e) {
                qWarning() << "Could not parse json .... ";
                qWarning() << e.message();
                //qWarning() << "We must reregister daemon ";
                qWarning() << "We must reregister daemon ";
                my_sleep (10); // We wait 10 seconds
                unregister();
                registerDaemon (algos, daemonService, executors);
            }
        }

        while (!itTasks.isNull() && itTasks->hasNext() ) {
            TaskPtr onServerTask;
            try {
                onServerTask = itTasks->next();
                taskService->convertTaskParametersToQVariant (onServerTask);

                if (!onServerTask.isNull() && onServerTask->getStatus() == Task::New) {
                    if (algos.contains (onServerTask->getExecutorID() ) ) {
#ifdef DEBUG_LOG
                        qDebug() << "Running task " << onServerTask->getId() << " executor = " << onServerTask->getExecutorID();
#endif
                        algos [onServerTask->getExecutorID()]->runTask (onServerTask, repo);
#ifdef DEBUG_LOG
                        qDebug() << "Task done";
#endif
                    }
                }
#ifdef DEBUG_LOG
                else {
                    qDebug() << "!!!!!!!!! No New Task   !!!!!!";
                }
#endif
            }
            catch (const std::exception & ex) {
                qWarning() << "Exception while performing task";
                qCritical() << "std::exception " << ex.what();
                // TODO: Update task
                my_sleep (40); // We wait 10 seconds
            }
            catch (const std::string & ex) {
                qWarning() << "Exception while performing task";
                qCritical() << "std::string " << QString::fromStdString (ex);
                onServerTask->setStatus (Task::Fail);
                onServerTask->setErrorMessage (QString::fromStdString (ex) );
                taskService->notifyHasFailed (onServerTask);
                my_sleep (40); // We wait 10 seconds
            }
            catch (MessageException e) {
                qWarning() << "Exception while performing task";
                qCritical () << "MessageException : " << e.message();
                try {
                    onServerTask->setStatus (Task::Fail);
                    onServerTask->setErrorMessage (e.message() );
                    taskService->notifyHasFailed (onServerTask);
                } catch (NetworkConnectionException e) {
                    qCritical () << "NetworkConnectionException : ";
                    qCritical () << "we will try to re-register";
                    my_sleep (40); // We wait 10 seconds

                    unregister();
                    registerDaemon (algos, daemonService, executors);
                }
                my_sleep (10); // We wait 10 seconds
            }
            catch (InternetConnectionException e) {
                qWarning() << "Exception Internet connection while performing task ";
                qCritical () << "InternetConnectionException : " << e.getMessage();
                my_sleep (40); // We wait 10 seconds
            }
            catch (...) {
                qCritical() << "Unkonwn exception while performing task";
                my_sleep (40); // We wait 10 seconds
            }
        }
    }
}

