#include "DoQuBookPluginParamsDialog.hpp"

#include <QPushButton>
#include <QLabel>
#include <QLineEdit>
#include <QFileDialog>

using namespace DoQuBook;
using namespace DoQuBook::gui;

DoQuBookPluginParamsDialog::DoQuBookPluginParamsDialog(std_shared_ptr<AlgorithmOnTask>::type plugin, QWidget *parent) :
    QDialog(parent)
  , _layout(NULL)
  , _plugin (plugin)
{
    QList<ExecutorParameterPtr> params = plugin->parameters();

    this->setWindowTitle("Plugin Parameters.");
    _layout = new QGridLayout(this);

    int row = 0;
    foreach(ExecutorParameterPtr p, params) {
        QLabel *label = new QLabel(p->getName(),this);
        _layout->addWidget(label,row,0);
        QWidget *edit = NULL;
        if(p->getParameterType()=="Image") {
            QPushButton *browse = new QPushButton("Browse", this);
            connect(browse, SIGNAL(clicked()), this, SLOT(setFile()));
            edit = browse;
        } else {
            QLineEdit *e = new QLineEdit("["+p->getParameterType()+"]", this);
            connect(e, SIGNAL(textChanged(QString)), SLOT(parameterFieldChanged(QString)));
            edit = e;
        }
        edit->setProperty("param_name", p->getName());
        _layout->addWidget(edit, row, 1);
        row++;
    }


    QPushButton *ok = new QPushButton("Ok", this);
    ok->setDefault(true);
    connect(ok, SIGNAL(clicked()), this, SLOT(dispose()));
    _layout->addWidget(ok,row,1);

    this->setLayout(_layout);

    connect(_plugin.get(), SIGNAL(autoParametersCalculated(QVariantMap&)),
            this, SLOT(refreshGui(QVariantMap&)));

}

void DoQuBookPluginParamsDialog::dispose()
{
    disconnect(this, SLOT(refreshGui(QVariantMap&)));
    this->close();
}

void DoQuBookPluginParamsDialog::setFile()
{
    QObject *w = sender();
    QString paramName = w->property("param_name").toString();
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open Image file"),"",tr("Files (*.*)"));
    QImage img (fileName);

    _parameters[paramName] = img;
}

void DoQuBookPluginParamsDialog::parameterFieldChanged(QString)
{
    QObject *w = sender();
    QString paramName = w->property("param_name").toString();
    QVariant value;
    QLineEdit *edit = dynamic_cast<QLineEdit*>(w);
    if(edit) {
         value = edit->text();
         if(!value.isNull()) {
            _parameters[paramName] = value;
            _plugin->onParameterChanged(paramName, value);
         }
    }

}

void DoQuBookPluginParamsDialog::refreshGui(QVariantMap &m)
{
    for(int idx = 0; idx < _layout->count(); idx++)
    {
        QLayoutItem *item = _layout->itemAt(idx);
        if(dynamic_cast<QWidgetItem *>(item))  {
            QWidget *w = item->widget();
            if(w) {
                QLineEdit * c = dynamic_cast<QLineEdit *>(w);
                if(c) {
                    QString param_name = c->property("param_name").toString();
                    QVariant value = m[param_name];
                    if(!value.isNull()) {
                        c->setText(value.toString());
                        _parameters[param_name] = value;
                    }
                }
            }
        }
    }
}


