/*!
 * \file DoQuBook_global.hpp
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef DOQUBOOK_GLOBAL_HPP
#define DOQUBOOK_GLOBAL_HPP

//#define DoQuBook_LIBRARY 1

//#if defined(DoQuBook_LIBRARY)
//#  define DoQuBookSHARED_EXPORT Q_DECL_EXPORT
//#else
//#  define DoQuBookSHARED_EXPORT Q_DECL_IMPORT
//#endif

#ifdef Q_WS_WIN
#   ifdef DoQuBook_LIBRARY
#           define DoQuBookSHARED_EXPORT Q_DECL_EXPORT
#   else
#           define DoQuBookSHARED_EXPORT Q_DECL_IMPORT
#   endif
#else
#   define DoQuBookSHARED_EXPORT
#endif

/*! \mainpage DoQuBook [DOcument QUality BOOK ;-)]
 *
 * \section intro_sec Introduction
 *
 *  \htmlonly
 *   <img src="http://sd-22392.dedibox.fr/~zoolonly/Icon_100x100.png" align=left>
 *   </img>
 *  \endhtmlonly
 * DoQuBook is a framework that aims to be an API to manipulate digitized documents
 * that may be hosted on different repositories. A repository can be created and
 *  managed by specialized organizations such as research labs.
 *
 * Each repository can be accesed with this library and any developper can implement his
 * own algorithm and test it on the collection (repository) he wants.
 *
 * For now we provide a repository implemented as a web application written in JEE called DoQuBookWeb.
 * Also, we aim (in a close future) to create a real community with specials roles and rights to any manager or developper or else ...
 *
 * Author and main contributors :
 * - Rabeux Vincent <rabeux@labri.fr>
 * - Maxime Daymard <mr.xyz@hotmail.fr>
 * - Jean-Philippe Domenger <domenger@labri.fr>
 * - Nicholas Journet <journet@labri.fr>
 * - Corentin Lallier <corentin.lallier@labri.fr>
 *
 * Icon Designer :
 * - Bastien Israel <larbrenboule@hotmail.com>
 *
 * Where to go from here ?
 *  - \ref install_sec
 *  - \ref DoQuBook
 *  - \ref pageREST
 *  - \ref pageMarshallable
 *  - \ref createAPlugin
 *  - \todo the cmake macros
 *
 * \htmlonly
 <iframe width="420" height="315" src="http://www.youtube.com/embed/7caF_hcpaDo" frameborder="0" allowfullscreen></iframe>
 * \endhtmlonly
 */

/*! \page install_sec Installation
 *
 * \section install_dep Step 0 : Installing Dependencies
 *
 *      - cmake 2.8.5
 *      - Qt4
 *      - QJson
 *      - OpenCV
 *
 * \subsection install_ubuntu Installation on Ubuntu Linux.
 *
 *  - Qt : PATH
    - Opencv : from tarball
 *  \code
 *   sudo apt-get install cmake libqt4-dev qtcreator  libpng-dev  libcv-dev libqjson-dev
 *  \endcode
 *
 * \subsection install_osx Installation on Mac os x.
 *
 *  - Install Mercurial :
        - http://mercurial.selenic.com/downloads
 *  - Install cmake :
 *      - http://www.cmake.org/files/v2.8/cmake-2.8.5-Darwin-universal.dmg
 *      - Add the folowing line in ~/.profile :
            \code
   export PATH=$PATH:/Applications/CMake\ 2.8-5.app/Contents/bin
            \endcode
 *  - Installing Qt :
 *      - Download and install packages at : http://qt.nokia.com/downloads/sdk-mac-os-cpp-offline
 *  - Install XCode from AppStore
 *  - Install HomeBrew :
 *      \code
 *      /usr/bin/ruby -e "$(curl -fsSL https://raw.github.com/gist/323731)"
 *      \endcode
 *  - Dependencies with homebrew :
 *      \code
 *       brew install qjson
 *       brew install opencv
 *      \endcode
 *
 *
 * \section install_getsources Step 1: Download latest source codes :
 *  \code
 *      hg clone  http://sd-22392.dedibox.fr/hg/doqubook
 *  \endcode
 * \section install_compile Step 2 : Compiling and installing DoQuBook
 *  \code
 *  cd doqubook/DoQuBook
 *  mkdir build
 *  cd build
 *  cmake ..
 *  make -j 5
 *  sudo make install
 *  \endcode
 *
 * \section install_compile_plugin Extra Step : Compiling and installing a DoQuBook plugin
 *  \code
 *  cd doqubook/DoQuBook/plugins/{plugin_of_your_choice}
 *  mkdir build
 *  cd build
 *  cmake ..
 *  make -j 5
 *  sudo make install
 *  \endcode
 *
 * \section install_compile_app Extra Step : Compiling and installing a DoQuBook App
 *  \code
 *  cd doqubook/DoQuBook/App/{app_of_your_choice}
 *  mkdir build
 *  cd build
 *  cmake ..
 *  make -j 5
 *  sudo make install
 *  \endcode
 *
 *
 * etc...
 */

#endif // DOQUBOOK_GLOBAL_HPP
