#ifndef FEATURESSERVICE_H
#define FEATURESSERVICE_H
#include <QSharedPointer>
#include <DoQuBook/Page.hpp>

namespace DoQuBook
{


    class GlobalNamedFeature;
    typedef QSharedPointer<GlobalNamedFeature> GlobalNamedFeaturePtr;

    class DoQuBookSHARED_EXPORT  GlobalNamedFeature : public SimpleMarshallable
    {

            Q_OBJECT

        public :

            MARSHALLABLE_CREATE_FONCTIONS (GlobalNamedFeature)

            explicit GlobalNamedFeature (QObject * parent = NULL) :
                SimpleMarshallable (parent)
                , m_id (-1)
                , m_featureValue (-1)
            {}

            virtual ~GlobalNamedFeature() {}



            int getId() const {
                return m_id;
            }

            QString getFeatureName() const {
                return m_featureName;
            }

            double getFeatureValue() const {
                return m_featureValue;
            }

        public slots:
            void setId (int arg) {
                if (m_id != arg) {
                    m_id = arg;
                    emit idChanged (arg);
                }
            }

            void setFeatureName (QString arg) {
                if (m_featureName != arg) {
                    m_featureName = arg;
                    emit featureNameChanged (arg);
                }
            }

            void setFeatureValue (double arg) {
                if (m_featureValue != arg) {
                    m_featureValue = arg;
                    emit featureValueChanged (arg);
                }
            }

        signals:
            void idChanged (int arg);

            void featureNameChanged (QString arg);

            void featureValueChanged (double arg);

        protected :
            Q_PROPERTY (int id READ getId WRITE setId NOTIFY idChanged)
            Q_PROPERTY (QString featureName READ getFeatureName WRITE setFeatureName NOTIFY featureNameChanged)
            Q_PROPERTY (double featureValue READ getFeatureValue  WRITE setFeatureValue NOTIFY featureValueChanged)


        private :
            int m_id;
            QString m_featureName;
            double m_featureValue;
    };


    class FeatureService;
    typedef QSharedPointer<FeatureService> FeatureServicePtr;


    class FeatureService
    {
        public:
            FeatureService () {}
            virtual ~FeatureService () {}

            virtual GlobalNamedFeaturePtr addGlobalNamedFeatureToPage (PagePtr page, QString featureName, double featureValue) = 0;
            virtual meta_iterator<GlobalNamedFeaturePtr>::IteratorPtr getGlobalNamedFeaturesOfPage (PagePtr page) = 0;
    };

}

#endif // FEATURESSERVICE_H
