#ifndef HISTOGRAMCOMPUTER_HPP
#define HISTOGRAMCOMPUTER_HPP

#include <DoQuBook/DoQuBook_global.hpp>

#include <DoQuBook/Config.hpp>

#include <DoQuBook/AlgorithmOnTask.hpp>
#include <DoQuBook/Histogram.hpp>


class DoQuBookSHARED_EXPORT HistogramComputer  : public DoQuBook::AlgorithmOnTask
{
    public:

        HistogramComputer() {}
        virtual ~HistogramComputer() {}

        // --- To be implemented //
        virtual QString organisationName () = 0;
        virtual QString description () = 0;
        virtual QString algorithmName () = 0;
        virtual QString version() = 0;
        virtual DoQuBook::HistogramPtr computeHistogram (const QImage & image, const QVariantMap & params = QVariantMap() ) = 0;
        // --- //

        virtual QList<DoQuBook::ExecutorParameterPtr> parameters () {
            QList<DoQuBook::ExecutorParameterPtr> params;
            return params;
        }

        virtual QString pluginType () {
            return "HistogramComputer";
        }

        virtual QString returnType() {
            return "Histogram";
        }

        virtual QList<DoQuBook::ExecutorParameterPtr> methodParameters() {
            QList<DoQuBook::ExecutorParameterPtr> params;
            params << DoQuBook::ExecutorParameter::create ("image", "Image", true);
            return params;
        }

        virtual QVariant compute (const QVariantMap & pluginMethodParams, const QVariantMap & params = QVariantMap() ) {
            qDebug() << "Computing histogram !!! ";
            QImage image = pluginMethodParams["image"].value<QImage>();
            DoQuBook::HistogramPtr histo = computeHistogram (image, params);
            return QVariant::fromValue<DoQuBook::HistogramPtr> (histo);
        }

};


typedef std::tr1::shared_ptr<HistogramComputer> HistogramComputerPtr;


Q_DECLARE_INTERFACE (HistogramComputer, "doqubook.histogramcomputer")

#endif // HISTOGRAMCOMPUTER_HPP
