#ifndef IMAGESERVICE_HPP
#define IMAGESERVICE_HPP

#include <QImage>
#include <QString>
#include <DoQuBook/Page.hpp>

namespace DoQuBook
{


    class ImageService;
    typedef QSharedPointer<ImageService> ImageServicePtr;

    /**
     * @brief The ImageService class is used by PageService to store the page image on a specific repo.
     *
     * \attention This class (or instance of it) are also injected to the Page class so that the method Page::getImage() uses it to retreive an image.
     *
     So ... there is two ways to download an image
        - First way, just use a Page [recomanded] :
        \code
PageServicePtr pageService = ...;
PagePtr thePage = pageService->get(pageId);
QImage img = thePage->getImage(); // This will use ImageService.
        \endcode
        - Second, use a ImageService [much more verbose but folows the doqubook "style"] :
PageServicePtr pageService = ...;
ImageServicePtr imageService = ...:
PagePtr thePage = pageService->get(pageId);
QImage img = imageService->get(thePage);
     *
     */
    class ImageService
    {
        public:
            ImageService();
            virtual ~ImageService() {}

            /**
              * @brief get
              * \warning page must exist on the repo
              * @param page
              * @param w
              * @return
              */
            virtual QImage get (const PagePtr & page, int w = -1) = 0;

            /**
             * @brief upload
             * \warning page must exist on the repo
             * @param image
             * @param page
             */
            virtual void upload (const QImage & image, const PagePtr & page) = 0;
    };

}

#endif // IMAGESERVICE_HPP
