/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#include "Logger.hpp"

#include <QTextStream>
#include <QTime>
#include <QDebug>
#include <QFileInfo>

#include <DoQuBook/Config.hpp>

#if QT_VERSION >= 0x050000
#include <QStandardPaths>
#else
#include <QDesktopServices>
#endif

using namespace  DoQuBook;

QMap<QString, Logger *> Logger::loggers;
static QMutex m;

Logger::Logger (const QString & name)
    : fileName (name)
{
#if QT_VERSION >= 0x050000
    QString filePath = QStandardPaths::writableLocation(QStandardPaths::DataLocation);
#else
    QString filePath = QDesktopServices::storageLocation( QDesktopServices::DataLocation );
#endif
    QDir dataDir (filePath);
    if(!dataDir.exists()) {
      dataDir.mkpath(filePath);
    }
    filePath.append("/"+name);
    logFilePath = filePath;
    QFile log_file(logFilePath);
    log_file.remove();
}

void Logger::writeToFile( const QString& s)
{
  QFile log_file(logFilePath);
  log_file.open(QIODevice::Append | QIODevice::Text);
  QTextStream logs(&log_file);
  logs << s;
  log_file.close();
}


void DoQuBook::Logger::error (const QString & /*msg*/)
{
}



DoQuBook::Logger * DoQuBook::Logger::getLogger (const QString & name)
{
    m.lock();
    if (!loggers.contains (name) ) {
        Logger * l = new Logger (name);
        loggers[name] = l;
    }
    m.unlock();
    return loggers[name];
}

DoQuBook::Logger * DoQuBook::Logger::getErrorLogger()
{
    return getLogger ("errors.txt");
}

DoQuBook::Logger * DoQuBook::Logger::getServerLogger()
{
    return getLogger ("serverLogs.txt");
}

void DoQuBook::Logger::info (const QString & file, const int & line, const QString & msg)
{
    m.lock();
    QFileInfo f (file);
    QString formatedLineNumber;
    for (int i = 0; i < 4 - QString::number (line).size(); i++) {
        formatedLineNumber.append (" ");
    }

    std::cout << COLOR_SERVER <<
              QTime::currentTime().toString ("hh:mm:ss").toUtf8().constData()
              << " [" << f.baseName().toUtf8().constData() << ":" << line << "]" << formatedLineNumber.toUtf8().constData() << ":"
              << COLOR_RESET << msg.toUtf8().constData() << "\n";



    QString log_msg = "(" + QDateTime::currentDateTime().toString ("dd-MM-yyyy") +  ") -"
                     + QTime::currentTime().toString ("hh:mm:ss")
                     + "- [" + f.baseName() + ":" + QString::number(line) + "]" + formatedLineNumber + ":" + msg + "\n";
    writeToFile(log_msg);
    m.unlock();

}


DoQuBook::Logger*DoQuBook::Logger::getInfoLogger()
{
   return getLogger ("serverLogs.txt");
}
