/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#include "Marshallable.hpp"
#include <QDebug>

QStringList Marshallable::ignoredProps;

// Constructor
Marshallable::Marshallable (QObject * parent)
    : QObject (parent)
{}

QList<MarshallablePtr> Marshallable::seeAlso()
{
    return QList<MarshallablePtr>();
}

QString Marshallable::entityName()
{
    QString className  = QString (this->metaObject()->className() ).toLower();
    QStringList namespaces = className.split ("::");
    return namespaces.at (namespaces.length() - 1);
}


//operator QVariantMap ()

QStringList Marshallable::ignoredProperties()
{
    return ignoredProps;
}

void Marshallable::addIgnoredProperty (const QString propName)
{
    ignoredProps.append (propName);
}

QVariantMap Marshallable::serialize (const QPoint & p) const
{
    QVariantMap m;
    if (!ignoredProperties().contains ("@property.type") ) {
        m["@property.type"] = "QPoint";
    }

    m[QString::fromUtf8 ("x")] = QByteArray::number (p.x() );
    m[QString::fromUtf8 ("y")] = QByteArray::number (p.y() );
    return m;
}

QPoint Marshallable::from (QVariant v)
{
    QPoint p;
    QVariantMap m = v.toMap();
    p.setX (m["x"].toInt() );
    p.setY (m["y"].toInt() );
    return p;
}
