#ifndef MEASURER_HPP
#define MEASURER_HPP

#include <DoQuBook/DoQuBook_global.hpp>

#include <DoQuBook/AlgorithmOnTask.hpp>

#include <DoQuBook/Page.hpp>
#include <DoQuBook/Region.hpp>
#include <DoQuBook/Config.hpp>

class DoQuBookSHARED_EXPORT Measurer : public DoQuBook::AlgorithmOnTask
{
    public:
        Measurer() {}
        virtual ~Measurer() {}

        // --- To be implemented //
        virtual QString organisationName () = 0;
        virtual QString description () = 0;
        virtual QString algorithmName () = 0;
        virtual QString version() = 0;
        virtual QMap<QString, double> measure (QList<QImage> &images, const QVariantMap & params = QVariantMap() ) = 0;
        // --- //


        virtual QList<DoQuBook::ExecutorParameterPtr> parameters () {
            QList<DoQuBook::ExecutorParameterPtr> params;
            return params;
        }

        virtual QString pluginType () {
            return "Measurer";
        }

        virtual QString returnType() {
            return "NumberMap";
        }

        virtual QList<DoQuBook::ExecutorParameterPtr> methodParameters() {
            QList<DoQuBook::ExecutorParameterPtr> params;
            params << DoQuBook::ExecutorParameter::create ("images", "ImageList", true);
            return params;
        }

        virtual QVariant compute (const QVariantMap & pluginMethodParams, const QVariantMap & params = QVariantMap() ) {
            QList<QImage> images;
            QVariantList list = pluginMethodParams["images"].toList();
            foreach (QVariant v, list) {
                images << v.value<QImage>();
            }
            QMap<QString, double> measures = measure (images, params);
            QMap<QString, QVariant> map;
            QMapIterator<QString, double> it (measures);
            while (it.hasNext() ) {
                it.next();
                qDebug() << it.value();
                double value = it.value();
                QString k = it.key();
                map[k] = value;
            }

            return map;
        }

};

typedef std_shared_ptr<Measurer>::type MeasurerPtr;

#define Measurer_iid "fr.labri.doqubook.Measurer"

Q_DECLARE_INTERFACE (Measurer, Measurer_iid)

#endif // BINARIZER_HPP
