#ifndef PAGECOLLECTIONSERVICE_HPP
#define PAGECOLLECTIONSERVICE_HPP

#include <DoQuBook/DoQuBook_global.hpp>
#include <DoQuBook/Page.hpp>
#include <DoQuBook/Intlist.hpp>
#include <DoQuBook/StringList.hpp>


namespace DoQuBook
{

    class PageService;

    class PageCollectionService;
    typedef QSharedPointer<PageCollectionService> PageCollectionServicePtr;

    class DoQuBookSHARED_EXPORT PageCollectionService
    {
        public:
            virtual ~PageCollectionService () {}
            virtual void addPage (const QString & collectionName, PagePtr & page) = 0;
            virtual QList<int> getPagesIds (const QString & collectionName) = 0;
            virtual QList<QString> getAll () = 0;
            virtual meta_iterator<PagePtr>::IteratorPtr getPages (const QString & collectionName) = 0;
    };


    class OnQListPageIterator;
    typedef QSharedPointer<OnQListPageIterator> OnQListPageIteratorPtr;

    class DoQuBookSHARED_EXPORT OnQListPageIterator : public QObject, public Iterator<PagePtr>
    {
            Q_OBJECT
        public :
            OnQListPageIterator (QList<PagePtr> &pages) : _pages (pages), _deleg (_pages) {}
            virtual ~OnQListPageIterator () {}
            virtual bool hasNext() {
                return _deleg.hasNext();
            }

            virtual PagePtr next() {
                return _deleg.next();
            }

        private :
            QList<PagePtr> _pages;
            QListIterator<PagePtr> _deleg;
    };


    class DoQuBookSHARED_EXPORT PageRandomSplitedSets : public QObject
    {
            Q_OBJECT
        public :
            PageRandomSplitedSets (meta_iterator<PagePtr>::IteratorPtr it, int percentInFirstSet);
            ~PageRandomSplitedSets();

            meta_iterator<PagePtr>::IteratorPtr firstSetIterator() {
                return firstSetIt;
            }

            meta_iterator<PagePtr>::IteratorPtr secondSetIterator() {
                return secondSetIt;
            }

        private :
            meta_iterator<PagePtr>::IteratorPtr firstSetIt;
            meta_iterator<PagePtr>::IteratorPtr secondSetIt;
    };

}

#endif // PAGECOLLECTIONSERVICE_HPP
