/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */#include "PageImageProvider.hpp"

#include <DoQuBook/Page.hpp>
#include <DoQuBook/Book.hpp>
#include <DoQuBook/Logger.hpp>

using namespace DoQuBook;

PageImageProvider::PageImageProvider()
    : imagesCache (CACHE_SIZE)
{}

PageImageProvider::~PageImageProvider()
{}


QImage PageImageProvider::getImage (PagePtr page, int w)
{
    QString imageName;
    if (w == -1) {
        imageName = page->getName();
    }
    else {
        imageName = page->getName() + QString::number (w);
    }

    QImage * image = NULL;
    if (imagesCache.contains (imageName) ) {
        image = imagesCache[imageName];
    }
    else {
        image = get (page, w);
        imagesCache.insert (imageName, image);
    }
    if (image->isNull() ) {
        QString typeName (getClassName() );
        Logger::getErrorLogger()->error (typeName + " : Could not find Image" + page->getName () );
        throw ImageNotFoundException (page->getId(), typeName + " : Could not find Image" + page->getFilePath() );
    }
    return *image;
}

void DoQuBook::PageImageProvider::setBook (DoQuBook::BookPtr pbook)
{
    book = pbook;
}

