/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */#include "PageImageProviderQHttp.hpp"
#include <DoQuBook/BookServiceServer.hpp>
#include <DoQuBook/Logger.hpp>
#include <DoQuBook/ApplicationContext.hpp>

using namespace DoQuBook;

PageImageProviderQHttp::PageImageProviderQHttp()
{}

QString PageImageProviderQHttp::getClassName()
{
    return "PageImageProviderQHttp";
}

QImage * PageImageProviderQHttp::get (PagePtr page, int w)
{
    QImage i ;
    if (USE_FILESYSTEM_CACHE) {
        i = getImageOnFileSystem (page);
    }
    if (!i.isNull() ) {
        return new QImage (i);
    }
    else {
        i = getImageOnRemoteServer (page, w);

        if (USE_FILESYSTEM_CACHE) {
            saveImageOnFileSystem (page, i);
        }
        return new QImage (i);
    }
}

PageImageProvider * PageImageProviderQHttp::clone()
{
    PageImageProviderQHttp * providerQhttp = new PageImageProviderQHttp;

    return providerQhttp;
}

QImage DoQuBook::PageImageProviderQHttp::getImageOnFileSystem (const DoQuBook::PagePtr & page)
{
    QImage i;
    QString repoPath = ApplicationContext::instance()->getTempRepoPath();
    QDir dir = QDir (repoPath);
    if (dir.exists() ) {
        qDebug() << dir.absolutePath();
        if (dir.cd (QString::number (page->getId() ) ) ) {
            i = QImage (dir.absoluteFilePath ("original.png") );
        }
        else {
            dir.mkdir (QString::number (page->getId() ) );
            dir.cd (QString::number (page->getId() ) );
        }
    }
    return i;
}

QImage DoQuBook::PageImageProviderQHttp::getImageOnRemoteServer (const DoQuBook::PagePtr & page, int w)
{
    DBWebBookService service (param.toString() );
    QImage img = service.getImage (page, w);
    if (img.isNull () ) {
        Logger::getErrorLogger()->error (" PageImageProviderQHttp retreiving " + page->getId () );
        img = service.getImage (page->getId () );
    }

    return img;
}

bool DoQuBook::PageImageProviderQHttp::saveImageOnFileSystem (const DoQuBook::PagePtr & page, const QImage & i)
{
    QString repoPath = ApplicationContext::instance()->getTempRepoPath();
    QDir dir = QDir (repoPath);
    if (dir.exists() ) {
        if (!dir.cd (QString::number (page->getId() ) ) ) {
            dir.mkdir (QString::number (page->getId() ) );
            dir.cd (QString::number (page->getId() ) );
        }
    }
    return i.save (dir.absoluteFilePath ("original.png") );
}

