#ifndef PLUGINFACTORY_HPP
#define PLUGINFACTORY_HPP

#include <QObject>

/**
 * @brief The PluginFactory class build or unLoad a plugin \see LocalPluginFactory and \see RemotePluginFactory
 */
class PluginFactory
{
    public :
        virtual ~PluginFactory () {}
        virtual QObject * build() = 0;
        virtual void unLoad() = 0;
};

/**
 * @brief The PluginDeleter class, is a functor given to a shared_ptr and calls the given PluginFactory to unLoad it.
 * The PluginFactory is indeed the only one to how the plugin was loaded.
 *
 * \code
 *      LocalPluginFactory facto("path/to/libPlugin.so");
 *      std::tr1::shared_ptr<QObject> p (facto.build(), PluginDeleter(facto));
 * \endcode
 */
class PluginDeleter
{
    public:
        PluginDeleter (PluginFactory * /*facto*/)
            /*: m_facto (facto)*/ {}
        virtual void operator () (QObject * /*d*/) const {
            //qCritical() << "PluginDeleter::operator()";
            //m_facto->unLoad();
        }
    private :
        //PluginFactory * m_facto;
};

#endif // PLUGINFACTORY_HPP
