#include "PluginLoader.hpp"
#include <QApplication>
#include <QDir>
#include <QPluginLoader>
#include <QDebug>
#include <DoQuBook/Page.hpp>

#include <DoQuBook/ApplicationContext.hpp>

#include <DoQuBook/RemotePluginFactory.hpp>

#include <QJsonObject>

using namespace DoQuBook;

PluginLoader::PluginLoader()
  : _pluginsLoaded (false)
{}


void PluginLoader::loadLocalPlugins()
{
  QList<QDir> pluginsDirs;
  if (_pathToPlugins.isEmpty() ) {


#if defined(Q_OS_WIN)
    pluginsDirs << QDir ("C:/Program Files/DoQuBook_plugins");
    pluginsDirs << QDir ("C:/Program Files (x86)/DoQuBook_plugins");
    pluginsDirs << QDir (qApp->applicationDirPath() + "/DoQuBook_plugins/");
#elif defined(Q_OS_MAC)
    QDir pluginsDir = QDir (qApp->applicationDirPath() );
    if (pluginsDir.dirName() == "MacOS") {
      pluginsDir.cdUp();
      if (! pluginsDir.cd ("DoQuBook_plugins") ) {
#ifdef DEBUG_LOG
        qDebug() << "!! No Plugin found in " << pluginsDir.absolutePath() << ".app Trying an other path";
#endif
        _pathToPlugins = "/Library/Frameworks/DoQuBook.framework/PlugIns/";
        pluginsDirs << QDir (_pathToPlugins);
      }
      else {
        pluginsDirs << pluginsDir;
      }
    }
    // Default path

    _pathToPlugins = "/Library/Frameworks/DoQuBook.framework/PlugIns/";
    pluginsDirs << QDir (_pathToPlugins);
#if QT_VERSION >= 0x050000
    pluginsDirs << QDir (QStandardPaths::writableLocation (QStandardPaths::HomeLocation)+"/.DoQuBook/PlugIns/");
#else
    pluginsDirs << QDir (QDesktopServices::storageLocation (QDesktopServices::HomeLocation) +"/.DoQuBook/PlugIns/");
#endif



#elif defined(Q_OS_LINUX)
    pluginsDirs << QDir ("/usr/local/bin/DoQuBook_plugins");
#endif
  }
  else {
    pluginsDirs << QDir (_pathToPlugins);
  }

  QList<QString> allreadyLoadedFilenames;
  foreach (QDir pDir, pluginsDirs) {
#ifdef DEBUG_LOG
    qDebug() << "-> Chosen plugin dir : " << pDir.absolutePath();
#endif

    foreach (QFileInfo fileInfo, pDir.entryInfoList (QDir::Files) ) {
      QString fileName = fileInfo.fileName();
#ifdef DEBUG_LOG
      qDebug() << "=== trying to load " << fileName;
#endif
      if (! allreadyLoadedFilenames.contains (fileName) ) {
        allreadyLoadedFilenames << fileName;
        QPluginLoader loader (fileInfo.filePath () );


        QVariant pId;
#if QT_VERSION >= 0x050000
        qDebug() << " ------- " << fileInfo.filePath();
        qDebug() << "MetaData" << loader.metaData();

        qDebug() << "Plugin Name " << loader.metaData().value("MetaData").toObject().value("name").toVariant();
        qDebug() << "Plugin Name " << loader.metaData().value("MetaData").toObject().value("version").toVariant();
        qDebug() << "Plugin Name " << loader.metaData().value("MetaData").toObject().value("organization").toVariant();
        qDebug() << "Loaded ? " << loader.isLoaded();
        pId = loader.metaData().value("MetaData").toObject().value("PID").toVariant();
        qDebug() << "PID " << pId;
        qDebug() << " ------- ";

        /*UNUSEDif(loader.isLoaded()) {
          pId = loader.metaData().value("MetaData").toObject().value("PID").toVariant();
        }*/
#else
#endif
        if(pId.isValid()) {
          qCritical() << "=== NOT LOADED QT5 tech: " << pId.toString();
          qDebug() << "=== Loading plugin of name : " << pId.toString();
          _plugins[pId.toString()] = new LocalPluginFactory (pDir.absoluteFilePath (fileName) );
        } else {
          QObject * plugin  = loader.instance();
          if (plugin) {
            PluginBase * ptr = qobject_cast<PluginBase *> (plugin);
            if (ptr) {
              QString name = ptr->pluginId();
              qDebug() << "=== Loading plugin of name : " << name;
              _plugins[name] = new LocalPluginFactory (pDir.absoluteFilePath (fileName) );
              if(name == "labri.CharacterDegradationGrayLevel.0.1.1") {
                qCritical() << "Unload of " << plugin->metaObject()->className();
                loader.unload();
              }
            }
            else {
              qCritical() << "Plugin does not implement the PluginBase Interface";
#ifdef DEBUG_LOG
              qDebug() << "=== Loading plugin of name : " << plugin->metaObject()->className();
#endif
              QString name (plugin->metaObject()->className());
              _plugins[name] = new LocalPluginFactory (pDir.absoluteFilePath (fileName) );
              if(name == "labri.CharacterDegradationGrayLevel.0.1.1") {
                qCritical() << "Unload of " << plugin->metaObject()->className();
                loader.unload();
              }
            }
          } else {
            qWarning() << "Plugin could not be loaded " << fileName;
            qWarning() << loader.errorString();
          }
        }
      }
    }
  }
}

void PluginLoader::loadPlugins()
{

  if (!_pluginsLoaded) {
#ifdef DEBUG_LOG
    qDebug() << "Load local plugins";
#endif
    loadLocalPlugins();
    _pluginsLoaded = true;
    if (ApplicationContext::instance()->isInternetPluginsActivated() ) {
      try {
#ifdef DEBUG_LOG
        qDebug() << "Load remote plugins";
#endif
        loadRemoteProxyPLugins();
      } catch(...) {}
    }
  }
  else {
#ifdef DEBUG_LOG
    qDebug() << "Plugins allready loaded";
#endif
  }
  _pluginsLoaded = true;
}


void PluginLoader::loadRemoteProxyPLugins()
{

  QList<RepositoryPtr> repos = ApplicationContext::instance()->getRepositories();
  // TODO: split the folowing code!!!!!!!
  foreach (RepositoryPtr repo, repos) {
    ExecutorServicePtr execService = repo->getExecutorService();
    try {
      QList<QString> executors = execService->getAll();
      foreach (QString s, executors) {
        if (!_plugins.contains (s) ) {
          ExecutorPtr exec = execService->get (s);

          bool found = false;
          QListIterator<RemotePluginFactory *> it (repo->getRemotePluginFactories());
          while (it.hasNext() && !found) {
            RemotePluginFactory * facto = it.next();
            if (facto->accepts (repo, exec) ) {

              qDebug() << "=== Loading plugin of name : " << s;
              RemotePluginFactory * selectedFacto = facto->clone();
              // TODO: delete the cloned factory;
              selectedFacto->setup (repo, exec);
              _plugins[s] = selectedFacto;
              found = true;
            }
          }
          if (!found) {
            qCritical() << "Plugin Type " << exec->getPluginType() << " are not yet supported as Remote plugins";
          }
        }
#ifdef DEBUG_LOG
        else {
          qCritical () << s << "Allready loaded";
        }
#endif
      }
    }
    catch (InternetConnectionException e) {}
    catch (CouldNotParseJSONException e) {}
  }
}

QString PluginLoader::getPluginKey(const QString& pluginNameOrAlias)
{
  return ApplicationContext::instance()->getPluginKey(pluginNameOrAlias);
}

void DoQuBook::PluginLoader::setPluginDirPath (const QString & pathToPlugins)
{
  _pathToPlugins = pathToPlugins;
  loadPlugins();
}

/*QList<BinarizerPtr> DoQuBook::PluginLoader::binarizers()
{
    QList<BinarizerPtr> bins;
    foreach(QString binName, listPlugins<Binarizer>())
    {
        bins.append(getPlugin<Binarizer>(binName));
    }
    return bins;
}*/


#include <functional>

QMap<QString, std_shared_ptr<AlgorithmOnTask>::type > DoQuBook::PluginLoader::algorihmsOnTaskImageResource (bool loadRemotePlugins)
{
  if (!_pluginsLoaded && loadRemotePlugins) {
    loadLocalPlugins();

    if (ApplicationContext::instance()->isInternetPluginsActivated() ) {
      loadRemoteProxyPLugins();
    }

    _pluginsLoaded = true;
  }
  else if (!_pluginsLoaded && !loadRemotePlugins) {
    _plugins.clear();
    loadLocalPlugins();
    _pluginsLoaded = true;
  }


  QMap<QString, std_shared_ptr<AlgorithmOnTask>::type > algos;
  QMapIterator <QString, PluginFactory * > it (_plugins);

  while (it.hasNext() ) {
    it.next();
    PluginFactory * facto = it.value();
    std_shared_ptr<QObject>::type t;
    std_shared_ptr<QObject>::type qObjectPlugin (facto->build(), PluginDeleter (facto) );
#ifdef HAVE_CPP11
    std_shared_ptr<AlgorithmOnTask>::type p = std::dynamic_pointer_cast<AlgorithmOnTask> (qObjectPlugin);
#else
    std_shared_ptr<AlgorithmOnTask>::type p = std::tr1::dynamic_pointer_cast<AlgorithmOnTask> (qObjectPlugin);
#endif

    if (p) {
      algos[p->organisationName() + "." + p->algorithmName() + "." + p->version()] = p;
    }
  }
  return algos;
}

