/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "Point.hpp"
#include <math.h>

using namespace DoQuBook;


Point::Point (float x, float y)
    : m_id (-1)
{
    _x = x;
    _y = y;
}

PointPtr Point::createFromVariantMap (float x, float y)
{
    PointPtr pointPtr = PointPtr (new Point (x, y) );
    return pointPtr;
}


PointPtr Point::createFromVariantMap (QPointF p)
{
    PointPtr pointPtr = PointPtr (new Point (p.x(), p.y() ) );
    return pointPtr;
}

float Point::getX() const
{
    return _x;
}

void Point::setX (float x)
{
    _x = x;
    emitModified();
}

float Point::getY() const
{
    return _y;
}

void Point::setY (float y)
{
    _y = y;
    emitModified();
}

QPointF Point::toQPointF() const
{
    return QPointF (_x, _y);
}

bool Point::operator == (const Point & other)
{
    return (other.getX() == getX() && other.getY() == getY() );
}

/*bool DoQuBook::Point::merge(MarshallablePtr marsh)
{
    PointPtr point = marsh.dynamicCast<Point>();
    if(!point.isNull()) {
        if(getId() == point->getId()) {
            setX (point->getX ());
            setY (point->getY ());
        }
    }
    return true;
}
*/
bool DoQuBook::Point::equals (const DoQuBook::Point & autre)
{
    /// Use of QString for a dummy hack ...
    /// the normal way did not deam to work in annotatedpolygon::equals
    /// The last point in list was not equal even if it was !!!!
    /// ......
    return (QString::number (_x) == QString::number (autre._x)
            && QString::number (_y) == QString::number (autre._y) );
}

double DoQuBook::Point::dist (QPointF point)
{
    QPointF thisP = toQPointF();
    return dist (thisP, point);
}

double DoQuBook::Point::dist (QPointF point1, QPointF point2)
{
    double xBxA = point2.x() - point1.x();
    double yByA = point2.y() - point1.y();

    return sqrt ( (xBxA * xBxA) + (yByA * yByA) );
}

void DoQuBook::Point::merge (MarshallablePtr other)
{
    PointPtr otherPoint = other.dynamicCast<Point>();

    if (getId() == -1) {
        setId (otherPoint->getId() );
    }

    if (otherPoint->getId() == getId() ) {
        setX (otherPoint->getX() );
        setY (otherPoint->getY() );
    }
}
