/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#ifndef POINT_HPP
#define POINT_HPP

#include <DoQuBook/DoQuBook_global.hpp>
#include <QObject>
#include <DoQuBook/Marshallable.hpp>
#include <DoQuBook/SimpleMarshallable.hpp>
#include <QSharedPointer>
#include <DoQuBook/Config.hpp>

namespace DoQuBook
{

    class DoQuBookSHARED_EXPORT Point;
    typedef QSharedPointer<Point> PointPtr;

    class DoQuBookSHARED_EXPORT Point : public SimpleMarshallable
    {
            Q_OBJECT
            Q_PROPERTY (int id READ getId WRITE setId NOTIFY idChanged)
            Q_PROPERTY (float x READ getX WRITE setX)
            Q_PROPERTY (float y READ getY WRITE setY)

        public :

            MARSHALLABLE_CREATE_FONCTIONS (Point)

            Point() : m_id (-1) {}
            Point (float x, float y);
            Point (const Point & p)
                : SimpleMarshallable() {
                _x = p.getX();
                _y = p.getY();
            }



            static PointPtr createFromVariantMap (float x, float y);
            static PointPtr createFromVariantMap (QPointF p);

            float getX() const;
            float getY() const;
            void setX (float x);
            void setY (float y);

            QPointF toQPointF() const;

            bool operator == (const Point & other);

            int getId() const {
                return m_id;
            }

            bool equals (const Point & autre);

            //virtual bool merge (MarshallablePtr marsh);

            double dist (QPointF point);
            static double dist (QPointF point1, QPointF point2);

            virtual void merge (MarshallablePtr other);

        public slots:
            void setId (int arg) {
                if (m_id != arg) {
                    m_id = arg;
                    emit idChanged (arg);
                }
            }

        signals:
            void idChanged (int arg);


        private :
            float _x;
            float _y;
            int m_id;
    };

}
#endif // POINT_HPP
