#include "RESTImageManager.hpp"
#include <QBuffer>
#include <DoQuBook/Config.hpp>
#include <DoQuBook/Exceptions.hpp>

using namespace DoQuBook;

RESTImageManager::RESTImageManager (ImageType type)
    : m_imageType (type)
{
}

void DoQuBook::RESTImageManager::addRessourcePath (WebClient::Method method, const QString & string)
{
    m_routPaths[method] = string;
}

QImage DoQuBook::RESTImageManager::get (const QMap<QString, QString> &params)
{
    WebClient wc;
    setContentTypeOnWebClient (wc);

    QUrl url = urlFromParams (params, WebClient::GET);
    QByteArray datas = wc.sendSynchronous (url, WebClient::GET, "");

    QImage img;
    img.loadFromData (datas);

    if (img.isNull() ) {
        throw MessageException ("Image is null");
    }

    return img;
}

QByteArray DoQuBook::RESTImageManager::post (const QImage & image, const QString & fileName, ImageType type)
{
    QMap<QString, QString> noParams;
    return post ( image, fileName, noParams, type);
}

QByteArray  DoQuBook::RESTImageManager::post (const QImage & image, const QString & fileName, const QMap<QString, QString> &params, ImageType type)
{
    QByteArray imageDatas;
    WebClient wc;
    QBuffer buff (&imageDatas);
    buff.open (QIODevice::WriteOnly);

    if (type == JPEG) {
        image.save (&buff, "jpeg");
    }
    else if (type == PNG) {
        image.save (&buff, "png");
    }
    else if (type == TIFF) {
        image.save (&buff, "TIFF");
    }
    else {   // default is jpeg
        image.save (&buff, "jpeg");
    }
    setContentTypeOnWebClient (wc);

    QUrl url = urlFromParams (params, WebClient::POST);
    QByteArray res = wc.postJpegFile (url, imageDatas, fileName);

    return res;
}

QUrl DoQuBook::RESTImageManager::urlFromParams (const QMap<QString, QString> &params, WebClient::Method method)
{
    QString url = m_routPaths [method];
    QMapIterator<QString, QString> it (params);
    while (it.hasNext() ) {
        it.next();
        url.replace ("(" + it.key() + ")", it.value() );
    }
    return QUrl (url);

}

void DoQuBook::RESTImageManager::setContentTypeOnWebClient (DoQuBook::WebClient & client)
{
    if (m_imageType == JPEG) {
        client.setContentType (WebClient::JPEG_MULTIPART);
    }
    else if (m_imageType == PNG) {
        client.setContentType (WebClient::PNG_MULTIPART);
    }
    else if (m_imageType == TIFF) {
        client.setContentType (WebClient::TIFF_MULTIPART);
    }
    else {   // default is jpeg
        client.setContentType (WebClient::JPEG_MULTIPART);
    }
}
