#include "Region.hpp"
#include <assert.h>


DoQuBook::Region::Region()
    :  _scanLines (new ScanLineList)
    , _area (-1)
    , _perimeter (-1)
{

}

DoQuBook::Region::Region (const QString & label)
    : _scanLines (new ScanLineList)
    , _label (label)
{}

DoQuBook::Region::Region(const ScanLineListPtr& scanLines)
  : _scanLines (scanLines)
  , _label ("none")
{}

void DoQuBook::Region::addScanLine(const Region::ScanLine& line)
{
  _scanLines->append(line);
}

QString DoQuBook::Region::getLabel()
{
    return _label;
}

void DoQuBook::Region::setLabel (const QString & label)
{
    _label = label;
}

QList<DoQuBook::PointPtr> DoQuBook::Region::getContour()
{
    /// This is taking to long !!!!
    /*if(_contour.isEmpty()) {
        foreach(PointPtr pt, _domain) {
            QPointF qPF = pt->toQPointF();
            if(!_contour.containsPoint(qPF, Qt::OddEvenFill)) {
                _contour.append(qPF);
            }
        }
    }*/

    QList<DoQuBook::PointPtr> contour;
    /*foreach (QPointF p, _contour) {
        PointPtr point = Point::create();
        point->setX (p.x() );
        point->setY (p.y() );
        contour.append (point);
    }*/
    return contour;
}


QRectF DoQuBook::Region::boundingBox()
{
    /*if (_bbox.isEmpty() ) {
        _bbox = _contour.boundingRect();
    }*/
    return _bbox;
}

double DoQuBook::Region::area()
{
    /*if (_area == -1) {
        _area = domain().size();
    }*/
    return _area;
}

double DoQuBook::Region::perimeter()
{
    /*if (_perimeter == -1) {
        _perimeter = _contour.size();
    }*/
    return _perimeter;
}

QImage DoQuBook::Region::thumbnail (const QImage & /*image*/)
{
    assert (false);
    return QImage();
}

void DoQuBook::Region::draw (QImage & image, const QRgb & color, DrawingMode /*drawingMode*/)
{
  QSharedPointer<RegionPainter> painter (new RegionPainter(image));
  painter->setColor(color);
  iterateOnDomain(painter);
}

void DoQuBook::Region::iterateOnDomain(const DoQuBook::RegionPointProcessorPtr& processor)
{
  processor->newRegion();
  // TODO: processor->area();
  // TODO: processor->perimeter();
  foreach (const Region::ScanLine & l, (*_scanLines)) {
    for(int x = l.start.x(); x <= l.end.x(); x++ ) {
      processor->processPoint(x, l.start.y());
    }
  }
  processor->endRegion();
}

QList<QPointF> DoQuBook::Region::domain()
{
    QList<QPointF> domain;
    DomainBuilderPtr p (new DomainBuilder (domain));
    iterateOnDomain(p);
    return domain;
}

void DoQuBook::Region::setArea (double a)
{
    _area = a;
}

void DoQuBook::Region::setPerimeter (double p)
{
    _perimeter = p;
}

void DoQuBook::Region::setBoundingBox (const DoQuBook::PointPtr & p1, const DoQuBook::PointPtr & p2)
{
    _bbox.setRect (p1->getX(), p1->getY(), p2->getX(), p2->getY() );
}


