/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#ifndef MARSHALLABLEWITHID_HPP
#define MARSHALLABLEWITHID_HPP

#include <DoQuBook/SimpleMarshallable.hpp>
#include <QObject>
#include <DoQuBook/DoQuBook_global.hpp>

namespace DoQuBook
{

    class SimpleMarshallableWithId;
    typedef QSharedPointer<SimpleMarshallableWithId> SimpleMarshallableWithIdPtr;

    /*!
      * \brief This class representes any kind of Marshallable with an id of type int.
      * \author Vincent Rabeux & Maxime Daymard
      * \todo Implement the class.
      * \todo Change inheritence of all "Beans" (ie : Book, Page, ...) from SimpleMarshalable to MarshallableWithId.
      */
    class DoQuBookSHARED_EXPORT SimpleMarshallableWithId : public SimpleMarshallable
    {
            Q_OBJECT

            Q_PROPERTY (int id READ getId WRITE setId NOTIFY idChanged)

            int m_id;

        public:
            SimpleMarshallableWithId (QObject * parent);

            int getId() const {
                return m_id;
            }

            virtual bool equals (SimpleMarshallableWithIdPtr marshallable) = 0;

        public slots:
            void setId (int arg) {
                if (m_id != arg) {
                    m_id = arg;
                    emit idChanged (arg);
                }
            }

        signals:
            void idChanged (int arg);
    };
}

#endif // MARSHALLABLEWITHID_HPP
