#ifndef STRINGCOMPARATOR_HPP
#define STRINGCOMPARATOR_HPP


#include <QVariant>
#include <DoQuBook/DoQuBook_global.hpp>
#include <DoQuBook/AlgorithmOnTask.hpp>

class DoQuBookSHARED_EXPORT StringComparator : public DoQuBook::AlgorithmOnTask
{
    public :
        StringComparator () {}
        virtual ~StringComparator () {}

        // --- To be implemented //
        virtual QString organisationName () = 0;
        virtual QString description () = 0;
        virtual QString algorithmName () = 0;
        virtual QString version() = 0;
        virtual double compare (const QString & text1, const QString & text2) = 0;
        // --- //

        virtual QString pluginType () {
            return "StringComparator";
        }
        virtual QString returnType() {
            return "Number";
        }

        virtual QList<DoQuBook::ExecutorParameterPtr> methodParameters() {
            QList<DoQuBook::ExecutorParameterPtr> params;
            params << DoQuBook::ExecutorParameter::create ("text1", "String", true);
            params << DoQuBook::ExecutorParameter::create ("text2", "String", true);
            return params;
        }

        virtual QVariant compute (const QVariantMap & pluginMethodParams, const QVariantMap & params = QVariantMap() ) {
            qDebug() << "String comparison!!! ";
            QString text1 = pluginMethodParams["text1"].value<QString>();
            QString text2 = pluginMethodParams["text2"].value<QString>();
            return compare (text1, text2);
        }

        virtual QList<DoQuBook::ExecutorParameterPtr> parameters () {
            QList<DoQuBook::ExecutorParameterPtr> params;
            return params;
        }

};

typedef std_shared_ptr<StringComparator>::type StringComparatorPtr;

#define StringComparator_iid "fr.labri.doqubook.StringComparator"

Q_DECLARE_INTERFACE (StringComparator, StringComparator_iid)


#endif // STRINGCOMPARATOR_HPP
