#include "Task.hpp"
#include <QUrl>

#include <DoQuBook/ApplicationContext.hpp>
#include <DoQuBook/Page.hpp>


using namespace DoQuBook;

Task::Task()
    : m_id (-1)
{
}

TaskParameterPtr Task::getParam (const QString & name)
{
    bool found = false;
    QListIterator <TaskParameterPtr> it (m_params);
    TaskParameterPtr param;
    while (!found && it.hasNext() ) {
        param = it.next();
        found = param->getName() == name;
    }
    return param;
}

bool Task::hasParam (const QString & name)
{
    bool found = false;
    QListIterator <TaskParameterPtr> it (m_params);
    TaskParameterPtr param;
    while (!found && it.hasNext() ) {
        param = it.next();
        found = param->getName() == name;
    }
    return found;
}

void Task::setPureResult (const QVariant & variant)
{
    m_result = variant;
}

QVariant Task::getPureResult()
{
    return m_result;
}

void Task::setResult (const QVariant & variant)
{
    m_result = variant;
}



TaskParameter::TaskParameter()
{
}

TaskParameter::TaskParameter (const QString & name, QUrl value, const QString & type)
    : m_id (-1)
    ,  m_name (name)
    , m_value (value)
    , m_parameterType (type)
{}

TaskParameter::TaskParameter (const QString & name, QImage value)
    : m_id (-1)
    , m_name (name)
    , m_value (value)
    , m_parameterType ("Image")
{
}

TaskParameter::TaskParameter (const QString & name, QVariant value)
    : m_id (-1)
    , m_name (name)
    , m_value (value)
{
    /*QStringList namespac = QString(value.typeName()).split("::");
    QString claxx = namespac[1];
    m_parameterType = claxx.replace("Ptr", "") ;*/
}

TaskParameter::TaskParameter (const QString & name, double value)
    : m_id (-1)
    , m_name (name)
    , m_value (value)
    , m_parameterType ("Double")
{
}

TaskParameter::TaskParameter (const QString & name, int value)
    : m_id (-1)
    , m_name (name)
    , m_value (value)
    , m_parameterType ("Integer")
{
}

TaskParameter::TaskParameter (const QString & name, QString value)
    : m_id (-1)
    , m_name (name)
    , m_value (value)
    , m_parameterType ("String")
{
}


/*
    Int = 2,
            UInt = 3,
            LongLong = 4,
            ULongLong = 5,
            Double = 6,
            Char = 7,
            Map = 8,
            List = 9,
            String = 10,
            StringList = 11,
            ByteArray = 12,
            BitArray = 13,
            Date = 14,
            Time = 15,
            DateTime = 16,
            Url = 17,
            Locale = 18,
            Rect = 19,
            RectF = 20,
            Size = 21,
            SizeF = 22,
            Line = 23,
            LineF = 24,
            Point = 25,
            PointF = 26,
            RegExp = 27,
            Hash = 28,
            EasingCurve = 29,
            LastCoreType = EasingCurve,

            // value 62 is internally reserved
        #ifdef QT3_SUPPORT
            ColorGroup = 63,
        #endif
            Font = 64,
            Pixmap = 65,
            Brush = 66,
            Color = 67,
            Palette = 68,
            Icon = 69,
            Image = 70,
            Polygon = 71,
            Region = 72,
            Bitmap = 73,
            Cursor = 74,
            SizePolicy = 75,
            KeySequence = 76,
            Pen = 77,
            TextLength = 78,
            TextFormat = 79,
            Matrix = 80,
            Transform = 81,
            Matrix4x4 = 82,
            Vector2D = 83,
            Vector3D = 84,
            Vector4D = 85,
            Quaternion = 86,
            LastGuiType = Quaternion,

            UserType = 127,
        #ifdef QT3_SUPPORT
            IconSet = Icon,
            CString = ByteArray,
            PointArray = Polygon,
        #endif
            LastType = 0xffffffff
}
  */
TaskParameter::~TaskParameter()
{
}

TaskParameterPtr TaskParameter::create (const QString & name, QUrl value, const QString & type)
{
    TaskParameterPtr ptr (new TaskParameter (name, value, type) );
    ptr->_this = ptr.toWeakRef();
    return ptr;
}

TaskParameterPtr TaskParameter::create (const QString & name, QImage value)
{
    TaskParameterPtr ptr (new TaskParameter (name, value) );
    ptr->_this = ptr.toWeakRef();
    return ptr;
}

TaskParameterPtr TaskParameter::create (const QString & name, QVariant value)
{
    TaskParameterPtr ptr (new TaskParameter (name, value) );
    ptr->_this = ptr.toWeakRef();
    return ptr;
}

TaskParameterPtr TaskParameter::create (const QString & name, double value)
{
    TaskParameterPtr ptr (new TaskParameter (name, value) );
    ptr->_this = ptr.toWeakRef();
    return ptr;
}

TaskParameterPtr TaskParameter::create (const QString & name, int value)
{
    TaskParameterPtr ptr (new TaskParameter (name, value) );
    ptr->_this = ptr.toWeakRef();
    return ptr;
}

TaskParameterPtr TaskParameter::create (const QString & name, QString value)
{
    TaskParameterPtr ptr (new TaskParameter (name, value) );
    ptr->_this = ptr.toWeakRef();
    return ptr;
}

QMap<Task::TaskStatus, QString> Task::taskStatusString;

Task::taskStatusStringInit::taskStatusStringInit()
{
    Task::taskStatusString[Task::New] = "New";
    Task::taskStatusString[Task::Done] = "Done";
    Task::taskStatusString[Task::Accepted] = "Accepted";
    Task::taskStatusString[Task::Fail] = "Fail";
    Task::taskStatusString[Task::Building] = "Building";
    Task::taskStatusString[Task::RefreshForm] = "RefreshForm";
}

Task::taskStatusStringInit Task::initer;

TaskPtr Task::create (const QString & executorID, const QVariantMap & paramsMap)
{
    TaskPtr t = Task::create();
    t->setId (-1);
    t->setDate (0);
    t->setSpan (0);
    t->setUserMail (ApplicationContext::instance()->getUserName() );
    t->setStatus (Task::New);
    t->setResult ("");
    t->setExecutorID (executorID);

    QMapIterator<QString, QVariant> it (paramsMap);
    while (it.hasNext() ) {
        it.next();
        t->addParam (it.key(), it.value() );
    }
    return t;
}


void Task::addParam (const QString & name, const QVariant & v)
{
    TaskParameterPtr param;
    if (v.type() == QVariant::Int) {
        param = TaskParameter::create (name, v.toInt() );
    }
    if (v.type() == QVariant::String) {
        param = TaskParameter::create (name, v.toString() );
    }
    if (v.type() == QVariant::Double) {
        param = TaskParameter::create (name, v.toDouble() );
    }
    if (v.type() == QVariant::Image) {
        //QUrl urlP (m_tempImageService.add(v.value<QImage>()));
        param = TaskParameter::create (name, v.value<QImage>() );
    }
    if (v.canConvert<AnnotationGlobalPtr>() ) {
        param = TaskParameter::create (name, v);
    }
    if (param.isNull() ) {
        param = TaskParameter::create (name, v);
    }
    addParam (param);
}

void Task::setParam (const QString & name, const QVariant & v)
{
    TaskParameterPtr param;
    if (v.type() == QVariant::Int) {
        param = TaskParameter::create (name, v.toInt() );
    }
    if (v.type() == QVariant::String) {
        param = TaskParameter::create (name, v.toString() );
    }
    if (v.type() == QVariant::Double) {
        param = TaskParameter::create (name, v.toDouble() );
    }
    if (v.type() == QVariant::Image) {
        //QUrl urlP (m_tempImageService.add(v.value<QImage>()));
        param = TaskParameter::create (name, v.value<QImage>() );
    }
    if (v.canConvert<AnnotationGlobalPtr>() ) {
        param = TaskParameter::create (name, v);
    }
    if (param.isNull() ) {
        param = TaskParameter::create (name, v);
    }
    setParam (param);
}

void Task::setParam(TaskParameterPtr arg)
{
    bool found = false;
    QListIterator<TaskParameterPtr> it(m_params);
    while(!found && it.hasNext()) {
        TaskParameterPtr p = it.next();
        if(p->getName() == arg->getName()) {
            found = true;
            p->setValue(arg->getValue());
        }
    }
}


