#ifndef WEBCLIENT_HPP
#define WEBCLIENT_HPP

#include <DoQuBook/DoQuBook_global.hpp>

#include <QObject>
#include <QByteArray>
#include <QUrl>
#include <QNetworkRequest>



#include <QtNetwork>
#include <QString>
#include <iostream>

#ifdef HAVE_CPP11
#include <future>
#include <thread>
#endif

#include <QSharedPointer>
#include <DoQuBook/MarshallableFormater.hpp>

namespace DoQuBook
{

  class DoQuBookSHARED_EXPORT WebClient : public QObject
  {
      Q_OBJECT
    public:
      explicit WebClient (QObject * parent = 0);
      enum Method {
        GET,
        POST,
        PUT,
        DEL
      };
      enum ContentType {
        JSON,
        XML,
        JPEG_MULTIPART,
        PNG_MULTIPART,
        TIFF_MULTIPART,
        XML_MULTIPART,
        ZIP_MULTIPART
      };

      void setContentType (ContentType t);

      QByteArray sendSynchronous  (const QUrl & url, Method m, const QByteArray & content );
      QByteArray sendSynchronous (const QUrl & url, Method m, const QByteArray & content, const QString & fileName );
      QByteArray postJpegFile  (const QUrl & url, const QByteArray & content, const QString & fileName );
      bool isAvailable (const QUrl & url);

      QByteArray gUncompress (const QByteArray & data);
      QByteArray gCompress (const QByteArray & data);


    signals:

    public slots:

    private :
      QString contentTypeToString();
      QByteArray boundary();
      void prepareRequestForMultiPart (QNetworkRequest & request, const QByteArray & boundary, const QByteArray & data);
      QByteArray createFormDataBytes (const QByteArray & bytes, const QByteArray & boundary, const QString & fileName );



    private :
      ContentType m_contentType;
  };




  template <class T>
  class Unmarshall {
    public :
      Unmarshall(MarshallableFormaterPtr &pMarshaller)
        : _marshaller(pMarshaller) {}
    protected :
      virtual void unmarshall (QNetworkReply *t) {
        QSharedPointer<T> m = T::create();
        _marshaller->unMarshall(t->readAll(), m);
        onReceived(m);
      }
      virtual void onReceived(QSharedPointer<T> m) = 0;
     private :
      MarshallableFormaterPtr _marshaller;

  };

  class AsyncWebClientQtBase : public QObject {
      Q_OBJECT
    public :
      void get(const QString &url) {
        QNetworkRequest  requete (url);
        requete.setHeader (QNetworkRequest::ContentTypeHeader, "application/json");
        requete.setRawHeader ("Accept", "application/json");
        connect(&_m, SIGNAL(finished(QNetworkReply*)), this, SLOT(process(QNetworkReply*)));
        _m.get (requete);
      }

    private slots :
      virtual void process (QNetworkReply *r) = 0;
    private :
      QNetworkAccessManager _m;
  };

  template <class T>
  class AsyncWebClientQt : public AsyncWebClientQtBase, public Unmarshall<T> {
    public :
      virtual void process (QNetworkReply *r) {
        Unmarshall<T>::unmarshall(r);
      }

      virtual void onReceived(QSharedPointer<T> m) = 0;
  };



#ifdef HAVE_CPP11
  class AsyncWebClientHelper {

    public :
      std::future<QByteArray> get(const QString &url, std::function<void (const QByteArray&)> onSuccess);

    private :
      std::promise<QByteArray> p;
      std::thread t;

  };
#endif

}

#endif // WEBCLIENT_HPP
