include(dqb_plugin_functions)
include(dqb_app_functions)
include(dqb_doxygen)


###########################################################################
###########################################################################
###########################################################################
###### Register a list of plugins for compilation depending on if the 
###### necessary libs are installed
###########################################################################
###########################################################################
###########################################################################
macro(registerPluginWithLibs PLUGINS LIBS)

SET(dependencies_mannaged true)
SET(HAVE_ALL_LIBS true)

FOREACH (LIB ${LIBS})
        message("==> Checking for library ${LIB}.")
        FIND_PACKAGE(${LIB} QUIET)
		
        if(NOT ${${LIB}_FOUND})
            message("       -- ${LIB} not found.")
            SET(HAVE_ALL_LIBS false)
        endif(NOT ${${LIB}_FOUND})

		## Clean up since we only want to check if libs are installed
		set(${LIB}_FOUND false)
		SET(${LIB}_INCLUDE_DIRS "")
		SET(${LIB}_LIBS "")
		SET(${LIB}_LIBRARIES "")
		SET(${LIB}_DEFINITIONS "")
		
ENDFOREACH(LIB)

if(${HAVE_ALL_LIBS})

FOREACH(PLUGIN ${PLUGINS})
    message("       -- Registering plugin ${PLUGIN}.")
ENDFOREACH()

FOREACH(PLUGIN ${PLUGINS})
    ADD_SUBDIRECTORY(${PLUGIN})
	get_directory_property(docTarget DIRECTORY ${PLUGIN} DEFINITION documentationTarget)
	set(docTargets ${docTargets} ${docTarget})

	get_directory_property(docFolder DIRECTORY ${PLUGIN} DEFINITION documentationFolder)
	set(docFolders ${docFolders} ${docFolder})
ENDFOREACH()
endif(${HAVE_ALL_LIBS})

if(NOT ${HAVE_ALL_LIBS})
foreach(PLUGIN ${PLUGINS})
    message("       -- Missing library for plugin ${PLUGIN}.")
endforeach()
endif(NOT ${HAVE_ALL_LIBS})

endmacro(registerPluginWithLibs)



###########################################################################











###########################################################################
###########################################################################
###########################################################################
##### DoQuBook Plugin Tests
###########################################################################
###########################################################################
###########################################################################
macro(doqubook_pluginTest PLUGIN_NAME TESTS_HEADERS TESTS_SOURCES test_RCS)
	
if(WANT_UNIT_TESTS)
	SET(EXECUTABLE_OUTPUT_PATH "${CMAKE_BINARY_DIR}/tests")
		
	SET(CMAKE_CXX_FLAGS "-g -Wuninitialized -O0 ${CMAKE_CXX_FLAGS}")
		
	FIND_PACKAGE(Qt5Test QUIET)

	if(Qt5Test_FOUND)
	  message("-- Found Qt5: ")
	    get_target_property(QtTest_location Qt5::Test LOCATION)
	    message("\t Qt5::Test: " ${QtTest_location})
	    INCLUDE_DIRECTORIES(${Qt5Test_INCLUDE_DIRS})
	    ADD_DEFINITIONS(${Qt5Test_DEFINITIONS})
	else()
					message("-- Did not find Qt5Test ! falling back to Qt4 ")
	        set(QT_USE_QTTEST true)
	        SET(QT_USE_QTTEST  TRUE )
	        SET(QT_USE_QTGUI   TRUE )
					set(QT_USE_QTNETWORK true)
		
		set(QT_USE_QTOPENGL TRUE)
		INCLUDE( ${QT_USE_FILE} )

	        include_directories(${QT_QTTEST_INCLUDE_DIR})
	endif()
	
include(CTest)
enable_testing()

SET(test_libs ${QT_LIBRARIES} ${QT_QTTEST_LIBRARY})

include_directories(${QT_QTTEST_INCLUDE_DIR})
include_directories(./src)
include_directories(${CMAKE_CURRENT_BINARY_DIR})


set(headers_MOC_HPPS ${TESTS_HEADERS})


SET(testName Test${PLUGIN_NAME})

if(Qt5Core_FOUND AND Qt5Widgets_FOUND AND Qt5Network_FOUND)
else()
	QT4_ADD_RESOURCES(test_CRCS ${test_RCS})
	QT4_WRAP_CPP(headers_MOCED_HPPS ${headers_MOC_HPPS})
endif()

ADD_CUSTOM_TARGET(${testName}moc ALL DEPENDS ${${testName}_MOC_SRCS})
add_executable(${testName}
    ${TESTS_SOURCES}
    ${TESTS_HEADERS}
    ${headers_MOCED_HPPS}
    ${test_CRCS})

add_dependencies(${testName} ${testName}moc)



if(Qt5Core_FOUND AND Qt5Widgets_FOUND AND Qt5Network_FOUND)
    #QT5_USE_MODULES(${NAME} Core Widgets Network Concurrent)
	set(test_libs ${test_libs} ${Qt5Core_LIBRARIES} ${Qt5Widgets_LIBRARIES} ${Qt5Concurrent_LIBRARIES}  ${Qt5Network_LIBRARIES} ${Qt5Test_LIBRARIES})
endif()

message(${PLUGIN_NAME})
target_link_libraries(${testName} ${PLUGIN_NAME} ${test_libs})

add_test(${testName} "${CMAKE_BINARY_DIR}/tests/${testName}")
endif(WANT_UNIT_TESTS)
endmacro(doqubook_pluginTest)


###########################################################################
###########################################################################
###########################################################################
##### REGISTER DATA IMGS FILES FO A DoQuBook Plugin Test
###########################################################################
###########################################################################
###########################################################################
macro(doqubook_registerData PLUGIN_NAME TEST_DATA CONFIG_IN)

SET(DATA_DIR ${TEST_DATA})

CONFIGURE_FILE(${CONFIG_IN}
               ${CMAKE_CURRENT_SOURCE_DIR}/test/Config.hpp
               @ONLY)
#if(WANT_UNIT_TESTS)
#ADD_CUSTOM_TARGET(Install${PLUGIN_NAME}DataFiles ALL
#    COMMAND ${CMAKE_COMMAND} -E make_directory ${DATA_DEST_DIR}
#    COMMAND ${CMAKE_COMMAND} -E copy_directory ${TEST_DATA} ${DATA_DEST_DIR}
#    WORKING_DIRECTORY ${CMAKE_CURRENT_BINARY_DIR}
#)
#endif(WANT_UNIT_TESTS)
endmacro(doqubook_registerData)


###########################################################################
###########################################################################
###########################################################################
##### Add a plugin dependency to the current plugin
###########################################################################
###########################################################################
###########################################################################
macro(doqubook_pluginDependency PLUGIN_NAME PATH_TO_PLUGIN)
if(NOT dependencies_mannaged)
    ADD_SUBDIRECTORY(${PATH_TO_PLUGIN} "${CMAKE_BINARY_DIR}/${PLUGIN_NAME}")
endif()
endmacro(doqubook_pluginDependency)





macro(macx_doqubook_app NAME ICONE RESOURCES SOURCES MOC_HEADERS HEADERS DEPENDENCIES XIBS)

	OPTION(CPP11 "Activate CPP11 features" OFF)

	  IF( ${CMAKE_CXX_COMPILER} MATCHES "cl" AND NOT ${CMAKE_CXX_COMPILER} MATCHES "clang" )
			add_definitions(-DIS_CL_COMPILER)
			SET(CMAKE_CXX_FLAGS "-EHsc")
	  ELSE()
	#	SET(CMAKE_CXX_FLAGS "-g -Wuninitialized -O0 "${CMAKE_CXX_FLAGS})
	  ENDIF()

		#SET(CMAKE_CXX_FLAGS "-g -Wuninitialized -O0 "${CMAKE_CXX_FLAGS})



	############################################
	# find the libs

	#QJSon
	find_package(QJSON REQUIRED)
	SET(LIBS ${LIBS} ${QJSON_LIBRARIES})
	INCLUDE_DIRECTORIES(${QJSON_INCLUDE_DIR})

	#Qt
	FIND_PACKAGE(Qt5Core QUIET)
	FIND_PACKAGE(Qt5Widgets QUIET)
	FIND_PACKAGE(Qt5Network QUIET)
	FIND_PACKAGE(Qt5WebKit QUIET)
	FIND_PACKAGE(Qt5WebKitWidgets QUIET)

	if(Qt5Core_FOUND AND Qt5Widgets_FOUND AND Qt5Network_FOUND AND Qt5WebKit_FOUND AND Qt5WebKitWidgets_FOUND)
	  message("-- Found Qt5: ")
	    get_target_property(QtCore_location Qt5::Core LOCATION)
	    message("\t Qt5::Core: " ${QtCore_location})
	    get_target_property(QtWidgets_location Qt5::Widgets LOCATION)
	    message("\t Qt5::Widgets: " ${QtWidgets_location})
	    get_target_property(QtNetwork_location Qt5::Network LOCATION)
	    message("\t Qt5::Network: " ${QtNetwork_location})
	    get_target_property(QtWebKit_location Qt5::WebKit LOCATION)
	    message("\t Qt5::Webkit: " ${QtWebKit_location})
	    get_target_property(QtWebKitWidgets_location Qt5::WebKitWidgets LOCATION)
	    message("\t Qt5::WebKitWidgets: " ${QtWebKitWidgets_location})



	  INCLUDE_DIRECTORIES(${Qt5Widgets_INCLUDE_DIRS})
	  ADD_DEFINITIONS(${Qt5Widgets_DEFINITIONS})

	  INCLUDE_DIRECTORIES(${Qt5Network_INCLUDE_DIRS})
	  ADD_DEFINITIONS(${Qt5Network_DEFINITIONS})

	  INCLUDE_DIRECTORIES(${Qt5WebKit_INCLUDE_DIRS})
	  ADD_DEFINITIONS(${Qt5WebKit_DEFINITIONS})


	  INCLUDE_DIRECTORIES(${Qt5WebKitWidgets_INCLUDE_DIRS})
	  ADD_DEFINITIONS(${Qt5WebKitWidgets_DEFINITIONS})

		QT5_ADD_RESOURCES( APP_RESOURCES ${RESOURCES} )

		QT5_WRAP_UI(UIS_HDRS
		    ${UICS}
		)

		INCLUDE_DIRECTORIES(${CMAKE_CURRENT_BINARY_DIR})

	  # Tell CMake to run moc when necessary:
	  set(CMAKE_AUTOMOC ON)


		if(APPLE AND CPP11)
		    message("-- Activating CPP11 features.")
		    SET(CMAKE_CXX_FLAGS "-std=c++11 -stdlib=libc++" ${CMAKE_CXX_FLAGS})
		    add_definitions(-DHAVE_CPP11)
		 endif()

	ELSE()
	  message ("Qt5 not found, searching for Qt4")
	  # Find Qt4
	  FIND_PACKAGE( Qt4 REQUIRED )

	  # Include the cmake file needed to use qt4
	  INCLUDE( ${QT_USE_FILE} )
	  set(QT_USE_QTTEST true)
	  set(QT_USE_QTNETWORK true)

	  # Qt : moced .cpps
		#Preparation des sources
		QT4_ADD_RESOURCES( APP_RESOURCES ${RESOURCES} )

		# prepare Qt HEADERS
		QT4_WRAP_CPP(MOC_SOURCES ${MOC_HEADERS})

		IF (WIN32)
		SET(QT_DEFINITIONS -DQT_DLL)
		ENDIF(WIN32)
	ENDIF()


if(APPLE)
	#Just a ObjC Test
	FIND_LIBRARY(FOUNDATION_LIBRARY Foundation)
	message(${FOUNDATION_LIBRARY})
	set(LIBS ${LIBS} ${FOUNDATION_LIBRARY})
	FIND_LIBRARY(Cocoa_LIBRARY Cocoa)
	message(${Cocoa_LIBRARY})
	set(LIBS ${LIBS} ${Cocoa_LIBRARY})
endif()







	#DoQuBook
	FIND_PACKAGE(DoQuBook REQUIRED)

	FOREACH (LIB ${DEPENDENCIES})
		FIND_PACKAGE(${LIB} REQUIRED)

		SET(${NAME}_EXTRA_DIRS ${${LIB}_INCLUDE_DIRS} ${${NAME}_EXTRA_DIRS})
		message(DIRS : ${${NAME}_EXTRA_DIRS})
		SET(${NAME}_EXTRA_LIBS ${${LIB}_LIBS} ${${LIB}_LIBRARIES} ${${NAME}_EXTRA_LIBS})
		SET(${NAME}_DEFINITIONS ${${LIB}_DEFINITIONS} ${${NAME}_DEFINITIONS})
	ENDFOREACH(LIB)

	INCLUDE_DIRECTORIES(${DoQuBook_INCLUDE_DIRS} ${QT_QTCORE_INCLUDE_DIR} ${QT_QTGUI_INCLUDE_DIR} ${QT_QTOPENGL_INCLUDE_DIR} ${QT_QTDESIGNER_INCLUDE_DIR} ${${NAME}_EXTRA_DIRS})
	include_directories(${QT_QTNETWORK_INCLUDE_DIR})
	INCLUDE_DIRECTORIES(${DoQuBook_INCLUDE_DIRS})

	# Add some definitions
	add_definitions(${QT_DEFINITIONS})
	ADD_DEFINITIONS(${${NAME}_DEFINITIONS})

	SET(LIBS ${LIBS} ${QT_LIBRARIES} ${DoQuBook_LIBRARIES} ${QT_QTXML_LIBRARY} ${QT_QTNETWORK_LIBRARY} ${QT_QTWEBKIT_LIBRARY} ${${NAME}_EXTRA_LIBS})


	###########################################
	SET(ALL_SOURCES ${SOURCES} ${MOC_SOURCES} ${MOC_HEADERS} ${HEADERS} ${APP_RESOURCES} ${UIS_HDRS})

	############################################
	# The icon
	# Pour le bundle osx on ajoute l'icone
	IF(APPLE)
	# set how it shows up in the Info.plist file
	SET(MACOSX_BUNDLE_ICON_FILE ${ICONE})
	# set where in the bundle to put the icns file
	SET_SOURCE_FILES_PROPERTIES(${CMAKE_CURRENT_SOURCE_DIR}/${ICONE}.icns PROPERTIES MACOSX_PACKAGE_LOCATION Resources)
	#INSTALL(FILES ${CMAKE_CURRENT_SOURCE_DIR}/${ICONE} DESTINATION  MACOSX_PACKAGE_LOCATION COMPONENT Resources)
	# include the icns file in the target
	SET(ALL_SOURCES ${ALL_SOURCES} ${CMAKE_CURRENT_SOURCE_DIR}/${ICONE}.icns)
	ENDIF(APPLE)

	if(WIN32)
	 FILE(WRITE ${CMAKE_SOURCE_DIR}/win.rc   "IDR_MAINFRAME           ICON                    \"${ICONE}.ico\"")
	 SET(ALL_SOURCES ${ALL_SOURCES} ${CMAKE_SOURCE_DIR}/win.rc)
	endif()

	############################################
	# the exe
	if(${IS_CORE_APP})
	ADD_EXECUTABLE(${NAME}
	        ${ALL_SOURCES}
	)
	else()
	ADD_EXECUTABLE(${NAME} MACOSX_BUNDLE
	        ${ALL_SOURCES}
	)
	endif()

	if(APPLE)
	set(XIBS View)
	message(${XIBS})
	find_program(IBTOOL ibtool HINTS "/usr/bin" "${OSX_DEVELOPER_ROOT}/usr/bin")
	if (${IBTOOL} STREQUAL "IBTOOL-NOTFOUND")
	  message(SEND_ERROR "ibtool can not be found and is needed to compile the .xib files. It should have been installed with 
	                    the Apple developer tools. The default system paths were searched in addition to ${OSX_DEVELOPER_ROOT}/usr/bin")
	endif()

	# Make sure the 'Resources' Directory is correctly created before we build
	add_custom_command (TARGET ${NAME} PRE_BUILD
	                      COMMAND mkdir -p ${CMAKE_CURRENT_BINARY_DIR}/${NAME}.app/Contents/Resources)

	# Compile the .xib files using the 'ibtool' program with the destination being the app package
	foreach(xib ${XIBS})
	  add_custom_command (TARGET ${NAME} POST_BUILD 
	                      COMMAND ${IBTOOL} --errors --warnings --notices --output-format human-readable-text 
	                              --compile ${CMAKE_CURRENT_BINARY_DIR}/${NAME}.app/Contents/Resources/${xib}.nib 
	                             ${CMAKE_SOURCE_DIR}/src/${xib}.xib
	                      COMMENT "Compiling ${SimpleCocoaVTK_SOURCE_DIR}/${xib}")

	endforeach()
	endif()



	if(Qt5Core_FOUND AND Qt5Widgets_FOUND AND Qt5Network_FOUND AND Qt5WebKit_FOUND)
	    QT5_USE_MODULES(${NAME} Widgets Network WebKit WebKitWidgets)
	endif()


	TARGET_LINK_LIBRARIES(${NAME} ${LIBS})


	#--------------------------------------------------------------------------------
	# def des variable necesaires :
	# APPS : est utilisÃ© pour CPack
	SET(plugin_dest_dir bin) 
	SET(qtconf_dest_dir bin)
	SET(APPS "\${CMAKE_INSTALL_PREFIX}/bin/${NAME}")
	IF(APPLE)
	SET(plugin_dest_dir ${NAME}.app/Contents)
	SET(qtconf_dest_dir ${NAME}.app/Contents/Resources)
	SET(APPS "\${CMAKE_INSTALL_PREFIX}/${NAME}.app")
	ENDIF(APPLE)
	IF(WIN32)
	SET(plugin_dest_dir bin) 
	SET(qtconf_dest_dir bin) 
	SET(APPS "\${CMAKE_INSTALL_PREFIX}/bin/${NAME}.exe")
	ENDIF(WIN32)


	#--------------------------------------------------------------------------------
	# Installation, sure apple, le bundle se trouve a la racine,
	# sur les autres plate-forme dans bin
	INSTALL(TARGETS ${NAME}
	BUNDLE DESTINATION . COMPONENT Runtime
	RUNTIME DESTINATION bin COMPONENT Runtime
	)

	if(APPLE)
	set(DoQuBook_plugins /Library/Frameworks/DoQuBook.framework/PlugIns/)
	endif(APPLE)
	if(WIN32)
	set(DoQuBook_plugins "C:/DoQuBook_plugins/")
	endif(WIN32)

	# We do not install to Application on mac if it is a core app
	if(APPLE AND NOT ${IS_CORE_APP})
	    SET(MY_OWN_INSTALL_PREFIX "/Applications" CACHE PATH "Prefix prepended to install directories")
	    SET(CMAKE_INSTALL_PREFIX "${MY_OWN_INSTALL_PREFIX}" CACHE INTERNAL "Prefix prepended to install directories" FORCE)
	endif()

	# We do not install to Application on mac if it is a core app
	if(APPLE AND NOT ${IS_CORE_APP})
	INSTALL(DIRECTORY ${DoQuBook_plugins} 
			DESTINATION ${plugin_dest_dir}/DoQuBook_plugins 
			COMPONENT Runtime
			PATTERN "*.lib" EXCLUDE)


	# Sort libs between .framework and .dylibs (.so or .dll), other file types are left behind
	#message(LIBS : ${LIBS})
	SET(BUNDLE_LIBS )
	SET(BUNDLE_FRAMEWORKS )
	FOREACH(item ${LIBS})
	  if(WIN32)
		STRING(REPLACE "/lib/" "/bin/" item ${item})
		STRING(REPLACE ".lib" ".dll" item ${item})
	  endif()
	  #message("Testing item ext : "${CMAKE_SHARED_LIBRARY_SUFFIX}"   item "${item})
	  STRING(REGEX MATCH ".*${CMAKE_SHARED_LIBRARY_SUFFIX}" lib ${item})
	  IF(lib)
		if(EXISTS ${lib})
			LIST(APPEND BUNDLE_LIBS ${lib})
		endif()
	  ENDIF(lib)
	  if(APPLE)
		  STRING(REGEX MATCH ".*so" lib ${item})
		  IF(lib)
			if(EXISTS ${lib})
				LIST(APPEND BUNDLE_LIBS ${lib})
			endif()
		  ENDIF(lib)
	  endif()
	  STRING(REGEX MATCH ".*framework" framework ${item})
	  if(framework AND APPLE)
		LIST(APPEND BUNDLE_FRAMEWORKS ${framework})
	  endif(framework AND APPLE)
	ENDFOREACH(item ${LIBS})

	INSTALL(FILES ${BUNDLE_LIBS}
				  DESTINATION ${plugin_dest_dir}/MacOS COMPONENT Runtime
	)
	endif()

	set(QT5_BUILD 1)
	if(QT5_BUILD)
	 get_target_property(QMAKE_EXECUTABLE Qt5::qmake LOCATION)
	  function(QUERY_QMAKE VAR RESULT)
	    exec_program(${QMAKE_EXECUTABLE} ARGS "-query ${VAR}" RETURN_VALUE return_code OUTPUT_VARIABLE output )
	    if(NOT return_code)
	      file(TO_CMAKE_PATH "${output}" output)
	      set(${RESULT} ${output} PARENT_SCOPE)
	    endif(NOT return_code)
	  endfunction(QUERY_QMAKE)

	  query_qmake(QT_INSTALL_PLUGINS QT_PLUGINS_DIR)
		message("ST PLUGINS DIR : " ${QT_PLUGINS_DIR})
	  query_qmake(QT_INSTALL_IMPORTS QT_IMPORTS_DIR)
		message("QT_INSTALL_IMPORTS : " ${QT_IMPORTS_DIR})
	  query_qmake(QT_HOST_DATA QT_DATA_DIR)
		message("QT_HOST_DATA : " ${QT_DATA_DIR})
	  set(QT_MKSPECS_DIR ${QT_DATA_DIR}/mkspecs)
	endif(QT5_BUILD)




	#--------------------------------------------------------------------------------
	# On copie le dossier de plugins QT dans le bundle courant
	if(APPLE AND NOT ${IS_CORE_APP})
		INSTALL(DIRECTORY "${QT_PLUGINS_DIR}/imageformats" DESTINATION ${plugin_dest_dir}/plugins COMPONENT Runtime)
		INSTALL(DIRECTORY "${QT_PLUGINS_DIR}/platforms" DESTINATION ${plugin_dest_dir}/plugins COMPONENT Runtime)
	endif()

	#--------------------------------------------------------------------------------
	# installation du qt.conf
	# on insert du code cmake pour Ã©crire le fichier dans le script d'installation
	if(APPLE AND NOT ${IS_CORE_APP})
	INSTALL(CODE "
	file(WRITE \"\${CMAKE_INSTALL_PREFIX}/${qtconf_dest_dir}/qt.conf\" \"\")
	" COMPONENT Runtime)

	#--------------------------------------------------------------------------------
	# On copie le qt_menu.nib de QT dans le bundle courant
	if(NOT QT5_BUILD)
	install (DIRECTORY ${QT_LIBRARY_DIR}/QtGui.framework/Resources/qt_menu.nib DESTINATION ${NAME}.app/Contents/Resources)
	endif()
	endif()

	#--------------------------------------------------------------------------------
	# Grace a BundleUtilities on peux copier les libs nÃ©cessaire dans la partie
	# Frameworks du bundle
	if(APPLE AND NOT ${IS_CORE_APP})
	# Répèrtoire à regarder pour les dépendance
	#SET(DIRS ${QT_LIBRARY_DIRS} ${BUNDLE_LIBS} ${DoQuBook_plugins}/lib /usr/local/lib/ /usr/local)
	SET(DIRS ${QT_LIBRARY_DIRS} ${DoQuBook_plugins}/lib /usr/local/lib/ /usr/local /Library/Frameworks /usr/lib)
	# Add "C:/Program Files/Microsoft Visual Studio 9.0/VC/redist/Debug_NonRedist/x86/Microsoft.VC90.DebugCRT" ?

	# on insert du code cmake pour utiliser BundleUtilities dans le script d'installation
	INSTALL(CODE "
	file(GLOB_RECURSE QTPLUGINS
	\"\${CMAKE_INSTALL_PREFIX}/${plugin_dest_dir}/plugins/*${CMAKE_SHARED_LIBRARY_SUFFIX}\")
	file(GLOB_RECURSE DOQUBOOKPLUGINS
	\"\${CMAKE_INSTALL_PREFIX}/${plugin_dest_dir}/DoQuBook_plugins/*${CMAKE_SHARED_LIBRARY_SUFFIX}\")
	include(BundleUtilities)
	SET(PLUGINS \${QTPLUGINS} \${DOQUBOOKPLUGINS})
	fixup_bundle(\"${APPS}\" \"\${PLUGINS}\" \"${DIRS}\")
	" COMPONENT Runtime)
	endif()




	###### CPACK 
	 set(CPACK_PACKAGE_NAME ${NAME})

	set(CPACK_PACKAGE_VENDOR ${${NAME}_VENDOR})
	set(${NAME}_VERSION ${${NAME}_MAJOR_VERSION}.${${NAME}_MINOR_VERSION}.${${NAME}_PATCH_VERSION})
	SET(CPACK_PACKAGE_VERSION "${${NAME}_VERSION}")
	SET(CPACK_PACKAGE_VERSION_MAJOR "${${NAME}_MAJOR_VERSION}")
	SET(CPACK_PACKAGE_VERSION_MINOR "${${NAME}_MINOR_VERSION}")
	SET(CPACK_PACKAGE_VERSION_PATCH "${${NAME}_PATCH_VERSION}")
	set(CPACK_PACKAGE_INSTALL_DIRECTORY ${NAME})
	SET(CPACK_PACKAGE_DESCRIPTION_SUMMARY ${${NAME}_DESCRIPTION})

	if(APPLE)
		SET(CPACK_GENERATOR "DragNDrop")
	endif()

	 if(WIN32)
		set(CPACK_NSIS_PACKAGE_NAME "${NAME}")
		set(CPACK_NSIS_DISPLAY_NAME "${NAME}")
		set(CPACK_NSIS_MODIFY_PATH "ON")
		set(CPACK_PACKAGE_EXECUTABLES "${NAME}" "${NAME}")
		set(CPACK_NSIS_MENU_LINKS "labri.digidoc.com" "Homepage for ${EWS_APP_NAME}")
		set(CPACK_NSIS_INSTALLED_ICON_NAME bin\\\\${NAME}.exe)
	 endif()

	IF(UNIX AND NOT APPLE)
		SET(CPACK_GENERATOR "DEB")
		SET(CPACK_DEBIAN_PACKAGE_MAINTAINER ${${NAME}_AUTHOR})
		SET(CPACK_PACKAGE_CONTACT "digidoc@gmail.com")

		IF(NOT DEFINED ${NAME}_DEB_DEPENDS)
			SET(CPACK_DEBIAN_PACKAGE_DEPENDS "libdoqubook-dev")
		ENDIF(NOT DEFINED ${NAME}_DEB_DEPENDS)
		IF(DEFINED ${NAME}_DEB_DEPENDS)
			SET(CPACK_DEBIAN_PACKAGE_DEPENDS "libdoqubook-dev,${${NAME}_DEB_DEPENDS}")
		ENDIF(DEFINED ${NAME}_DEB_DEPENDS)

		SET(CPACK_DEBIAN_PACKAGE_SECTION "research")

		SET(SERVER sd-22392.dedibox.fr)
		SET(SERVER_REPO_BIN_PATH /var/www/digidoc-repo/binary/)
		SET(SERVER_REPO_PATH /var/www/digidoc-repo/)
		SET(ON_SERVER_USERNAME zoolonly)

		message(=> New target : deploydeb${NAME})
		ADD_CUSTOM_TARGET(deploydeb${NAME}
			 COMMAND ${CMAKE_COMMAND} -E echo "Deploying Generated Deb Files..."
			 COMMAND ${CMAKE_COMMAND} -E echo "  --- to :" ${SERVER}:${SERVER_REPO_PATH}
			 COMMAND sh ${CMAKE_CURRENT_SOURCE_DIR}/../../Tools/publishDebs.sh
			 COMMAND ${CMAKE_COMMAND} -E echo "Done."
			 WORKING_DIRECTORY ${CMAKE_CURRENT_BINARY_DIR}
		)
	ENDIF(UNIX AND NOT APPLE)

	# on utilise cpack : "cpack -G DragNDrop CPackConfig.cmake" sur mac os x
	# le CPackConfig.cmake est crÃ©er par un premier cmake
	# Command osx /Applications/CMake\ 2.8-1.app/Contents/bin/cpack -G DragNDrop CPackConfig.cmake
	#SET(CPACK_GENERATOR "DragNDrop")
	#set(CPACK_BINARY_DRAGNDROP ON)
	include(CPack)

endmacro(macx_doqubook_app)
