
# Usage :
# include(UseQt)
#
# UseAllQt5(DQB_QT "${DoQuBook_moc_headers}" "${DoQuBook_QRC}" "${DoQuBook_UIS}")
# SET(sources ${sources}  ${DQB_QT_GENERATED_SRCS})
# TARGET_LINK_LIBRARIES(TARGET ${OTHER_LIBS} ${DQB_QT_LIBS})
#
macro(UseQt5 MODULES QRCS UIS io_SOURCES io_LIBS)


    if(NOT DEFINED qt5_found)
        # First look up for Qt5 modules in Modules
        # Check if all modules ar found first and if so display messages
        set(all_modules_found true)
        set(found_modules )
        set(not_found_modules )
        foreach(module ${MODULES})
            find_package(Qt5${module} QUIET)
            if(Qt5${module}_FOUND)
                set(found_modules ${found_modules} ${module})
            else()
                set(all_modules_found false)
                set(not_found_modules ${not_found_modules} ${module})
            endif()
        endforeach()

        # display message
        # and make the includes just once
        if(${all_modules_found})
            message("-- Found Qt5: ")
            set(CMAKE_AUTOMOC ON)
            # Really needed ?
            if(APPLE)
                set(CMAKE_AUTOMOC_MOC_OPTIONS "-F/Library/Frameworks")
            endif(APPLE)
            foreach(module ${found_modules})
                get_target_property(Qt${module}_location Qt5::${module} LOCATION)
                message("\t Qt5::${module}: ${Qt${module}_location}")
                include_directories(${Qt5${module}_INCLUDE_DIRS})
                add_definitions(${Qt5${module}_DEFINITIONS})
            endforeach()
            set(USEQT5 true)
        endif()
        if(${all_modules_found})
            SET(qt5_found true)
						QT5_ADD_RESOURCES( APP_RESOURCES ${QRCS} )
						QT5_WRAP_UI(UIS_HDRS ${UIS})
						
						INCLUDE_DIRECTORIES(${CMAKE_CURRENT_BINARY_DIR})
        endif()
    endif()


    # Will contain all needed Qt Libs
    set(QT_ALL_LIBS )
    if(${USEQT5})
        add_definitions(-DHAVE_QT5)
        add_definitions(${QT_DEFINITIONS})
        add_definitions(-DQT_PLUGIN)
        add_definitions(-DQT_SHARED)

        foreach(module ${found_modules})
            #QT5_USE_MODULES(${NAME} Core Widgets Network Concurrent)
            set(QT_ALL_LIBS ${QT_ALL_LIBS} ${Qt5${module}_LIBRARIES})
        endforeach()
    endif()
    set(${io_LIBS} ${QT_ALL_LIBS})
		set(${io_SOURCES} ${UIS_HDRS} ${APP_RESOURCES})
endmacro(UseQt5)

macro(UseAllQt5 MOC_HEADERS QRCS UIS io_SOURCES io_LIBS)
	
    set(modules Core Widgets Network Concurrent WebKit WebKitWidgets Xml XmlPatterns PrintSupport)
    UseQt5("${modules}" "${QRCS}" "${UIS}" ${io_SOURCES} ${io_LIBS})

    # Automoc !!
    set(${output}_GENERATED_SRCS  )
endmacro(UseAllQt5)

macro(UseAllQt4 MOC_HEADERS QRCS UIS io_SOURCES io_LIBS)
    message("Qt5 not found, searching for Qt4")
    # Find Qt4
    FIND_PACKAGE(Qt4 REQUIRED)

    # Include the cmake file needed to use qt4
    set(QT_USE_QTTEST true)
    set(QT_USE_QTNETWORK true)
    set(QT_USE_QTXML TRUE)
    INCLUDE(${QT_USE_FILE})

    add_definitions(${QT_DEFINITIONS})
    add_definitions(-DQT_PLUGIN)
    add_definitions(-DQT_SHARED)
    add_definitions(-DQT_NO_DEBUG)


    # Qt : moced .cpps
    qt4_wrap_cpp(qt4_moced ${DoQuBook_moc_headers})
    QT4_ADD_RESOURCES(qt4_CRCS ${DoQuBook_QRC})

    set(${output}_qt4_found true)
    set(${io_LIBS} ${io_LIBS} ${QT_LIBRARIES} ${QT_QTNETWORK_LIBRARY})
    set(${io_SOURCES} ${io_SOURCES} ${DoQuBook_moced} ${DoQuBook_CRCS})
endmacro(UseAllQt4)


# Usage :
# include(UseQt)
#
# UseAllQt("${DoQuBook_moc_headers}" "${DoQuBook_QRC}" "${DoQuBook_UIS}" qt_sources qt_libs)
# SET(sources ${sources}  ${qt_sources})
# TARGET_LINK_LIBRARIES(TARGET ${OTHER_LIBS} ${qt_libs})
#
macro(UseAllQt MOC_HEADERS QRCS UIS io_SOURCES io_LIBS)
    set(CMAKE_AUTOMOC ON)
    UseAllQt5("${MOC_HEADERS}" "${QRCS}" "${UIS}" "${io_SOURCES}" "${io_LIBS}")
    if(NOT ${qt5_found})
        UseAllQt4("${MOC_HEADERS}" "${QRCS}" "${UIS}" "${io_SOURCES}" "${io_LIBS}")
    endif()
endmacro(UseAllQt)

# Usage :
#
#    set(MOC_SOURCES ./test.cpp) if Q_OBJECT in test.cpp
#    set(SOURCES ./test.pp)
#    set(MOC_HEADERS ./test.hpp) if Q_OBJECT in test.hpp
#    set(HEADERS ./testhead.h)
#    set(QRCS ./test.qrc)
#    AddQtTest(testName "${MOC_SOURCES}" "${SOURCES}" "${MOC_HEADERS}" "${HEADERS}" "${QRCS}")
macro(AddQtTest NAME MOC_SOURCES SOURCES MOC_HEADERS HEADERS QRCS LIBS)
    set(CMAKE_AUTOMOC ON)
    set(EXECUTABLE_OUTPUT_PATH "${CMAKE_BINARY_DIR}/tests")
    set(CMAKE_CXX_FLAGS "-g -Wuninitialized -O0 ${CMAKE_CXX_FLAGS}")
    include(CTest)
    enable_testing()

    if(NOT DEFINED QtTest_FOUND)
        FIND_PACKAGE(Qt5Test QUIET)

        if(Qt5Test_FOUND)
            set(QtTest_FOUND true)
            message("-- Found Qt5: ")
            get_target_property(QtTest_location Qt5::Test LOCATION)
            message("\t Qt5::Test: ${QtTest_location}")
            INCLUDE_DIRECTORIES(${Qt5Test_INCLUDE_DIRS})
            ADD_DEFINITIONS(${Qt5Test_DEFINITIONS})
        else()
            set(QT_USE_QTTEST true)
            SET( QT_USE_QTTEST  TRUE )
            SET( QT_USE_QTGUI   TRUE )
            include_directories(${QT_QTTEST_INCLUDE_DIR})
        endif()
    endif()


    set(UIS )
    UseAllQt("${MOC_HEADERS}" "${QRCS}" "${UIS}" ${NAME}_qt_sources ${NAME}_qt_libs)
    set(${NAME}_all_sources ${SOURCES} ${HEADERS} ${MOC_HEADERS} ${${NAME}_qt_sources})
    if(NOT ${qt5_found})
        QT4_WRAP_CPP(${NAME}_MOCED_CPPS ${MOC_SOURCES})
        ADD_CUSTOM_TARGET(${NAME}moc ALL DEPENDS ${${NAME}_MOCED_CPPS})
        set(${NAME}_all_sources ${${NAME}_all_sources} ${${NAME}_MOCED_CPPS})
    endif()
    set(test_libs ${LIBS} ${test_libs} ${${NAME}_qt_libs})

    add_executable(${NAME} ${${NAME}_all_sources} ${SOURCES})
    add_dependencies(${NAME} ${NAME}moc)

    target_link_libraries(${NAME} ${test_libs})

    if(Qt5Core_FOUND AND Qt5Widgets_FOUND AND Qt5Network_FOUND)
        QT5_USE_MODULES(${NAME} Widgets Network Test Concurrent)
    endif()

    add_test(${NAME} ${NAME})

endmacro(AddQtTest)

