include(dqb_UseQt)
include(dqb_targets_functions)
include(dqb_cpack)

###########################################################################
###########################################################################
###########################################################################
#####  DoQuBook APP
###########################################################################
###########################################################################
###########################################################################

## ICONE filename but not the path [path must be CMAKE_CURRENT_SOURCE_DIR]
## Example : Icon for files (Icon.icns, Icon.png, Icon.ico)
macro(private_doqubook_app NAME ICONE RESOURCES SOURCES MOC_HEADERS HEADERS UICS DEPENDENCIES IS_CORE_APP)

  IF( ${CMAKE_CXX_COMPILER} MATCHES "cl" AND NOT ${CMAKE_CXX_COMPILER} MATCHES "clang" )
		add_definitions(-DIS_CL_COMPILER)
		SET(CMAKE_CXX_FLAGS "-EHsc")
  ENDIF()


dqb_prepare_target(${NAME} "${HEADERS}" "${MOC_HEADERS}" "${SOURCES}" "${LIBS}")


############################################
# The icon
# Pour le bundle osx on ajoute l'icone
IF(APPLE)
	# set how it shows up in the Info.plist file
	SET(MACOSX_BUNDLE_ICON_FILE ${ICONE})
	# set where in the bundle to put the icns file
	SET_SOURCE_FILES_PROPERTIES(${CMAKE_CURRENT_SOURCE_DIR}/${ICONE}.icns PROPERTIES MACOSX_PACKAGE_LOCATION Resources)
	#INSTALL(FILES ${CMAKE_CURRENT_SOURCE_DIR}/${ICONE} DESTINATION  MACOSX_PACKAGE_LOCATION COMPONENT Resources)
	# include the icns file in the target
	SET(ALL_SOURCES ${ALL_SOURCES} ${CMAKE_CURRENT_SOURCE_DIR}/${ICONE}.icns)
ENDIF(APPLE)

if(WIN32)
 FILE(WRITE ${CMAKE_SOURCE_DIR}/win.rc   "IDR_MAINFRAME           ICON                    \"${ICONE}.ico\"")
 SET(ALL_SOURCES ${ALL_SOURCES} ${CMAKE_SOURCE_DIR}/win.rc)
endif()

############################################
# the exe
if(${IS_CORE_APP})
	ADD_EXECUTABLE(${NAME} ${ALL_SOURCES})
        add_custom_command(TARGET ${NAME}
            POST_BUILD COMMAND
            ${CMAKE_INSTALL_NAME_TOOL} -add_rpath "/Applications/QtSDK/Desktop/Qt/5.0/5.0.0/clang_64/lib/" ${NAME})
else()
        ADD_EXECUTABLE(${NAME}
                MACOSX_BUNDLE ${ALL_SOURCES}
        )
endif()

dqb_target_link_libraries(${NAME})



#--------------------------------------------------------------------------------
# def des variable necesaires :
# APPS : est utilisÃ© pour CPack
SET(plugin_dest_dir bin) 
SET(qtconf_dest_dir bin)
SET(APPS "\${CMAKE_INSTALL_PREFIX}/bin/${NAME}")
IF(APPLE)
	SET(plugin_dest_dir ${NAME}.app/Contents)
	SET(qtconf_dest_dir ${NAME}.app/Contents/Resources)
	SET(APPS "\${CMAKE_INSTALL_PREFIX}/${NAME}.app")
ENDIF(APPLE)
IF(WIN32)
	SET(plugin_dest_dir bin) 
	SET(qtconf_dest_dir bin) 
	SET(APPS "\${CMAKE_INSTALL_PREFIX}/bin/${NAME}.exe")
ENDIF(WIN32)


#--------------------------------------------------------------------------------
# Installation, sur apple, le bundle se trouve a la racine, sur les autres plate-forme dans bin
INSTALL(TARGETS ${NAME}
BUNDLE DESTINATION . COMPONENT Runtime
RUNTIME DESTINATION bin COMPONENT Runtime)

if(APPLE)
	set(DoQuBook_plugins /Library/Frameworks/DoQuBook.framework/PlugIns/)
endif(APPLE)
if(WIN32)
	set(DoQuBook_plugins "C:/DoQuBook_plugins/")
endif(WIN32)

# We do not install to Application on mac if it is a core app
if(APPLE AND NOT ${IS_CORE_APP})
    SET(MY_OWN_INSTALL_PREFIX "/Applications" CACHE PATH "Prefix prepended to install directories")
    SET(CMAKE_INSTALL_PREFIX "${MY_OWN_INSTALL_PREFIX}" CACHE INTERNAL "Prefix prepended to install directories" FORCE)
endif()

# We do not install plugins in Application on mac if it is a core app
if(APPLE AND NOT ${IS_CORE_APP})
	INSTALL(DIRECTORY ${DoQuBook_plugins} 
					DESTINATION ${plugin_dest_dir}/DoQuBook_plugins 
					COMPONENT Runtime
					PATTERN "*.lib" EXCLUDE)

	# Sort libs between .framework and .dylibs (.so or .dll), other file types are left behind
SET(BUNDLE_LIBS )
SET(BUNDLE_FRAMEWORKS )
FOREACH(item ${LIBS})
  if(WIN32)
	STRING(REPLACE "/lib/" "/bin/" item ${item})
	STRING(REPLACE ".lib" ".dll" item ${item})
  endif()
  #message("Testing item ext : "${CMAKE_SHARED_LIBRARY_SUFFIX}"   item "${item})
  STRING(REGEX MATCH ".*${CMAKE_SHARED_LIBRARY_SUFFIX}" lib ${item})
  IF(lib)
	if(EXISTS ${lib})
		LIST(APPEND BUNDLE_LIBS ${lib})
	endif()
  ENDIF(lib)
  if(APPLE)
	  STRING(REGEX MATCH ".*so" lib ${item})
	  IF(lib)
		if(EXISTS ${lib})
			LIST(APPEND BUNDLE_LIBS ${lib})
		endif()
	  ENDIF(lib)
  endif()
  STRING(REGEX MATCH ".*framework" framework ${item})
  if(framework AND APPLE)
	LIST(APPEND BUNDLE_FRAMEWORKS ${framework})
  endif(framework AND APPLE)
ENDFOREACH(item ${LIBS})

INSTALL(FILES ${BUNDLE_LIBS}
			  DESTINATION ${plugin_dest_dir}/MacOS COMPONENT Runtime
)
endif()

if(DEFINED qt5_found)
  get_target_property(QMAKE_EXECUTABLE Qt5::qmake LOCATION)
  function(QUERY_QMAKE VAR RESULT)
    exec_program(${QMAKE_EXECUTABLE} ARGS "-query ${VAR}" RETURN_VALUE return_code OUTPUT_VARIABLE output )
    if(NOT return_code)
      file(TO_CMAKE_PATH "${output}" output)
      set(${RESULT} ${output} PARENT_SCOPE)
    endif(NOT return_code)
  endfunction(QUERY_QMAKE)

  query_qmake(QT_INSTALL_PLUGINS QT_PLUGINS_DIR)
	#message("QT PLUGINS DIR : " ${QT_PLUGINS_DIR})
  query_qmake(QT_INSTALL_IMPORTS QT_IMPORTS_DIR)
	#message("QT_INSTALL_IMPORTS : " ${QT_IMPORTS_DIR})
  query_qmake(QT_HOST_DATA QT_DATA_DIR)
	#message("QT_HOST_DATA : " ${QT_DATA_DIR})
  set(QT_MKSPECS_DIR ${QT_DATA_DIR}/mkspecs)
endif()




#--------------------------------------------------------------------------------
# On copie le dossier de plugins QT dans le bundle courant
if(APPLE AND NOT ${IS_CORE_APP})
	INSTALL(DIRECTORY "${QT_PLUGINS_DIR}/imageformats" DESTINATION ${plugin_dest_dir}/plugins COMPONENT Runtime)
	INSTALL(DIRECTORY "${QT_PLUGINS_DIR}/platforms" DESTINATION ${plugin_dest_dir}/plugins COMPONENT Runtime)
endif()


#--------------------------------------------------------------------------------
# installation du qt.conf
# on insert du code cmake pour Ã©crire le fichier dans le script d'installation
if(APPLE AND NOT ${IS_CORE_APP})
INSTALL(CODE "
file(WRITE \"\${CMAKE_INSTALL_PREFIX}/${qtconf_dest_dir}/qt.conf\" \"\")
" COMPONENT Runtime)

#--------------------------------------------------------------------------------
# On copie le qt_menu.nib de QT dans le bundle courant
if(NOT DEFINED qt5_found)
install (DIRECTORY ${QT_LIBRARY_DIR}/QtGui.framework/Resources/qt_menu.nib DESTINATION ${NAME}.app/Contents/Resources)
endif()
endif()

#--------------------------------------------------------------------------------
# Grace a BundleUtilities on peux copier les libs nÃ©cessaire dans la partie
# Frameworks du bundle
if(APPLE AND NOT ${IS_CORE_APP})
# Répèrtoire à regarder pour les dépendance
#SET(DIRS ${QT_LIBRARY_DIRS} ${BUNDLE_LIBS} ${DoQuBook_plugins}/lib /usr/local/lib/ /usr/local)
SET(DIRS ${QT_LIBRARY_DIRS} ${DoQuBook_plugins}/lib /usr/local/lib/ /usr/local /Library/Frameworks /usr/lib)
# Add "C:/Program Files/Microsoft Visual Studio 9.0/VC/redist/Debug_NonRedist/x86/Microsoft.VC90.DebugCRT" ?

# on insert du code cmake pour utiliser BundleUtilities dans le script d'installation
INSTALL(CODE "
file(GLOB_RECURSE QTPLUGINS
\"\${CMAKE_INSTALL_PREFIX}/${plugin_dest_dir}/plugins/*${CMAKE_SHARED_LIBRARY_SUFFIX}\")
file(GLOB_RECURSE DOQUBOOKPLUGINS
\"\${CMAKE_INSTALL_PREFIX}/${plugin_dest_dir}/DoQuBook_plugins/*${CMAKE_SHARED_LIBRARY_SUFFIX}\")
include(BundleUtilities)
SET(PLUGINS \${QTPLUGINS} \${DOQUBOOKPLUGINS})
fixup_bundle(\"${APPS}\" \"\${PLUGINS}\" \"${DIRS}\")
" COMPONENT Runtime)
endif()




###### CPACK 
 set(CPACK_PACKAGE_NAME ${NAME})

set(CPACK_PACKAGE_VENDOR ${${NAME}_VENDOR})
set(${NAME}_VERSION ${${NAME}_MAJOR_VERSION}.${${NAME}_MINOR_VERSION}.${${NAME}_PATCH_VERSION})
SET(CPACK_PACKAGE_VERSION "${${NAME}_VERSION}")
SET(CPACK_PACKAGE_VERSION_MAJOR "${${NAME}_MAJOR_VERSION}")
SET(CPACK_PACKAGE_VERSION_MINOR "${${NAME}_MINOR_VERSION}")
SET(CPACK_PACKAGE_VERSION_PATCH "${${NAME}_PATCH_VERSION}")
set(CPACK_PACKAGE_INSTALL_DIRECTORY ${NAME})
SET(CPACK_PACKAGE_DESCRIPTION_SUMMARY ${${NAME}_DESCRIPTION})

if(APPLE)
	SET(CPACK_GENERATOR "DragNDrop")
endif()
 
 if(WIN32)
	set(CPACK_NSIS_PACKAGE_NAME "${NAME}")
	set(CPACK_NSIS_DISPLAY_NAME "${NAME}")
	set(CPACK_NSIS_MODIFY_PATH "ON")
	set(CPACK_PACKAGE_EXECUTABLES "${NAME}" "${NAME}")
	set(CPACK_NSIS_MENU_LINKS "labri.digidoc.com" "Homepage for ${EWS_APP_NAME}")
	set(CPACK_NSIS_INSTALLED_ICON_NAME bin\\\\${NAME}.exe)
 endif()

IF(UNIX AND NOT APPLE)
	SET(CPACK_GENERATOR "DEB")
	SET(CPACK_DEBIAN_PACKAGE_MAINTAINER ${${NAME}_AUTHOR})
	SET(CPACK_PACKAGE_CONTACT "digidoc@gmail.com")
	
	IF(NOT DEFINED ${NAME}_DEB_DEPENDS)
		SET(CPACK_DEBIAN_PACKAGE_DEPENDS "libdoqubook-dev")
	ENDIF(NOT DEFINED ${NAME}_DEB_DEPENDS)
	IF(DEFINED ${NAME}_DEB_DEPENDS)
		SET(CPACK_DEBIAN_PACKAGE_DEPENDS "libdoqubook-dev,${${NAME}_DEB_DEPENDS}")
	ENDIF(DEFINED ${NAME}_DEB_DEPENDS)

	SET(CPACK_DEBIAN_PACKAGE_SECTION "research")


ENDIF(UNIX AND NOT APPLE)
 
# on utilise cpack : "cpack -G DragNDrop CPackConfig.cmake" sur mac os x
# le CPackConfig.cmake est crÃ©er par un premier cmake
# Command osx /Applications/CMake\ 2.8-1.app/Contents/bin/cpack -G DragNDrop CPackConfig.cmake
#SET(CPACK_GENERATOR "DragNDrop")
#set(CPACK_BINARY_DRAGNDROP ON)
include(CPack)

endmacro(private_doqubook_app)

macro(core_doqubook_app NAME ICONE RESOURCES SOURCES MOC_HEADERS HEADERS DEPENDENCIES)
	SET(UICS 
		)
	private_doqubook_app("${NAME}" "${ICONE}" "${RESOURCES}" "${SOURCES}" "${MOC_HEADERS}"  "${UICS}" "${HEADERS}" "${DEPENDENCIES}"  TRUE)
endmacro(core_doqubook_app)

macro(doqubook_app NAME ICONE RESOURCES SOURCES MOC_HEADERS HEADERS UICS DEPENDENCIES)
	private_doqubook_app("${NAME}" "${ICONE}" "${RESOURCES}" "${SOURCES}" "${MOC_HEADERS}" "${HEADERS}" "${UICS}" "${DEPENDENCIES}" FALSE)
endmacro(doqubook_app)
