include(dqb_UseQt)
include(dqb_targets_functions)
include(dqb_cpack)

###########################################################################

## Used by Qt5 and Qt4 plugins :
function(dqb_plugin_auto_conf )
	set(VERSION ${${PROJECT_NAME}_MAJOR_VERSION}.${${PROJECT_NAME}_MINOR_VERSION}.${${PROJECT_NAME}_PATCH_VERSION})
	
	set(DESCRIPTION ${${PROJECT_NAME}_DESCRIPTION} )
	set(AUTHOR ${${PROJECT_NAME}_AUTHOR} )
	set(ALGORITHM_NAME ${PROJECT_NAME} )
	set(ORGANIZATION_NAME ${${PROJECT_NAME}_VENDOR} )
	
	set(PLUGIN_ID "${${PROJECT_NAME}_VENDOR}.${PROJECT_NAME}.${VERSION}" )
	INCLUDE_DIRECTORIES(${CMAKE_CURRENT_BINARY_DIR})
	CONFIGURE_FILE(${CMAKE_CURRENT_SOURCE_DIR}/src/PluginConfig.hpp.in
                       ${CMAKE_CURRENT_SOURCE_DIR}/src/PluginConfig.hpp
	               @ONLY)
	CONFIGURE_FILE(${CMAKE_CURRENT_SOURCE_DIR}/src/plugin.json.in
								               ${CMAKE_CURRENT_BINARY_DIR}/plugin.json
								               @ONLY)
endfunction()

###########################################################################
###### Creates a DoQuBook_Plugin (with Qt and DoQuBook)
###########################################################################
###########################################################################
###########################################################################
# The plugin MACRO
macro(doqubook_plugin NAME HEADERS MOC_HEADERS SOURCES LIBS)

  
SET(LIBRARY_OUTPUT_PATH "${CMAKE_BINARY_DIR}/plugins")
SET(EXECUTABLE_OUTPUT_PATH "${CMAKE_BINARY_DIR}/tests")

dqb_prepare_target(${NAME} "${HEADERS}" "${MOC_HEADERS}" "${SOURCES}" "${LIBS}")

add_definitions(-Dplugin_LIBRARY)

############################################



############################################
# the target
ADD_LIBRARY(${NAME} SHARED ${ALL_SOURCES})

dqb_target_link_libraries(${NAME})

############################################
# installation rules
if(APPLE)
	set(DoQuBook_PLUGIN_DEST /Library/Frameworks/DoQuBook.framework/PlugIns)
endif(APPLE)
if(UNIX AND NOT APPLE)
	set(DoQuBook_PLUGIN_DEST bin/DoQuBook_plugins)
endif(UNIX AND NOT APPLE)

if(WIN32)
	set(DoQuBook_PLUGIN_DEST "C:/Program Files/DoQuBook_plugins")
	set(CPACK_NSIS_INSTALL_ROOT "C:/Program Files/DoQuBook_plugins")
	set(CPACK_PACKAGE_INSTALL_DIRECTORY "")
	SET(CMAKE_INSTALL_PREFIX "C:/Program Files/DoQuBook_plugins" CACHE PATH "DoQuBook Plugins install prefix" FORCE)
endif(WIN32)

INSTALL(TARGETS ${NAME}
  RUNTIME DESTINATION ${DoQuBook_PLUGIN_DEST}
  LIBRARY DESTINATION ${DoQuBook_PLUGIN_DEST}
)


############################
# Packaging
if(WIN32 OR APPLE)
# Install le libs for auto packaging and fixup_bundle lookpath
# Sort libs between .framework and .dylibs (.so or .dll), other file types are left behind
if(NOT TARGET package)
	ADD_CUSTOM_TARGET(package)
endif()

ADD_CUSTOM_COMMAND(TARGET package
	COMMAND ${CMAKE_COMMAND} -E make_directory ${CMAKE_BINARY_DIR}/plugin_libs
	DEPENDS ${NAME}
)

SET(BUNDLE_LIBS )
SET(BUNDLE_FRAMEWORKS )
FOREACH(item ${${NAME}_ALL_LIBS})
  STRING(REGEX MATCH ".*${CMAKE_SHARED_LIBRARY_SUFFIX}" lib ${item})
  IF(lib)
    LIST(APPEND BUNDLE_LIBS ${lib})
  ENDIF(lib)
  if(APPLE)
	  STRING(REGEX MATCH ".*so" lib ${item})
	  IF(lib)
		if(EXISTS ${lib})
			LIST(APPEND BUNDLE_LIBS ${lib})
		endif()
	  ENDIF(lib)
  endif()
  STRING(REGEX MATCH ".*framework" framework ${item})
  if(framework AND APPLE)
	LIST(APPEND BUNDLE_FRAMEWORKS ${framework})
  endif(framework AND APPLE)
ENDFOREACH(item ${LIBS})

foreach(i ${BUNDLE_LIBS})
add_custom_command(TARGET package
	COMMAND ${CMAKE_COMMAND} -E copy ${i} ${CMAKE_BINARY_DIR}/plugin_libs/
	DEPENDS ${NAME}
)
endforeach()

else()
	dqb_package(${NAME})
endif()


############################################
doxygen(${NAME})

endmacro(doqubook_plugin)
