macro(dqb_prepare_target NAME HEADERS MOC_HEADERS SOURCES LIBS)

PROJECT(${NAME})

include_directories(./src/)

SET(CMAKE_CXX_FLAGS "-g -Wuninitialized -O0 ${CMAKE_CXX_FLAGS}")

OPTION(CPP11 "Activate CPP11 features" OFF)
if(CPP11)
		add_definitions(-DHAVE_CPP11)
endif()

set(CMAKE_INCLUDE_CURRENT_DIR ON)

# some cmake conf stuff
SET(CMAKE_ALLOW_LOOSE_LOOP_CONSTRUCTS TRUE)



############################################
### lets find the required libs.

#DoQubook
find_package(DoQuBook REQUIRED)

#Activate OpenCVConverters if needed (if using OpenCV)
foreach(LIB ${LIBS})
    string(COMPARE EQUAL "${LIB}" OpenCV _cmp)
    if(${_cmp})
        set(ALL_LIBS ${LIBS} DoQuBookOpenCV)
    endif()
endforeach()

# For each LIB in ALL_LIBS (DoQuBookOpenCv + LIBS) include dirs, use libs ans add definitions
# Only works if LIB has a FindLIB.cmake in CMAKE_MODULE_PATH(S)
foreach (LIB ${ALL_LIBS})
	find_package(${LIB} REQUIRED)
	set(${NAME}_EXTRA_DIRS ${${LIB}_INCLUDE_DIRS} ${${NAME}_EXTRA_DIRS})
	set(${NAME}_EXTRA_LIBS ${${LIB}_LIBS} ${${LIB}_LIBRARIES} ${${NAME}_EXTRA_LIBS})
	set(${NAME}_DEFINITIONS ${${LIB}_DEFINITIONS} ${${NAME}_DEFINITIONS})
endforeach(LIB)
include_directories(${DoQuBook_INCLUDE_DIRS} ${QT_QTCORE_INCLUDE_DIR} ${QT_QTGUI_INCLUDE_DIR} ${QT_QTOPENGL_INCLUDE_DIR} ${QT_QTDESIGNER_INCLUDE_DIR} ${${NAME}_EXTRA_DIRS})

#Qt 
set(UIS ) # no .ui for plugins ?
UseAllQt("${MOC_HEADERS}" "${RESOURCES}" "${UIS}" qt_sources qt_libs)
set(ALL_SOURCES ${SOURCES} ${MOC_HEADERS} ${HEADERS}  ${sources}  ${qt_sources})
set(${NAME}_EXTRA_LIBS ${${NAME}_EXTRA_LIBS} ${qt_libs})
#


# Override
ADD_DEFINITIONS(${${NAME}_DEFINITIONS})


endmacro()




macro(dqb_target_link_libraries NAME)
	set(${NAME}_ALL_LIBS ${DoQuBook_LIBRARIES} ${${NAME}_EXTRA_LIBS})
	TARGET_LINK_LIBRARIES(${NAME} ${${NAME}_ALL_LIBS} ${${NAME}_TARGET_DEPENDENCIES})
endmacro()
