#ifndef GenericCRUDServiceTest_HPP
#define GenericCRUDServiceTest_HPP

#include <QObject>
#include <DoQuBook/Repository.hpp>
#include <DoQuBook/ApplicationContext.hpp>
#include <DoQuBook/PluginLoader.hpp>

#include <Config.hpp>

#include <QtTest>

namespace DoQuBook {
  namespace TestsUtils {

    template <class T,class IdType>
    class GenericCRUDServiceTest {
      public :
        typedef QSharedPointer<CRUDService<T, IdType > > CRUDServicePtr;


        GenericCRUDServiceTest(){}

        virtual void modifyEntity(QSharedPointer<T>  &e) = 0;
        virtual void verifyUpdated(QSharedPointer<T>  &e) =0;


        void testGetAddGetUpdateDel (QSharedPointer<T> &e,CRUDServicePtr &srv){
          {
            QSharedPointer<T>  entityOnServer = srv->get (e->getId());
            QVERIFY (entityOnServer.isNull());
          }
          // add the entity
          srv->add (e);
          {
            QSharedPointer<T> entityOnServer = srv->get (e->getId());
            QVERIFY (!entityOnServer.isNull());
            QVERIFY (entityOnServer->equals(*e));
          }
          // update :
          modifyEntity(e);
          srv->update(e);
          {
            QSharedPointer<T> entityOnServer = srv->get (e->getId());
            QVERIFY (!entityOnServer.isNull());
            verifyUpdated(entityOnServer);
          }

          bool deleted = srv->del(e->getId());
          QVERIFY(deleted);
          // Check if really deleted
          {
            QSharedPointer<T> entityOnServer = srv->get (e->getId());
            QVERIFY(entityOnServer.isNull());
          }
        }
    };
  }
}


#endif // GenericCRUDServiceTest_hpp
