#include "GenericTestPageService.hpp"

#include <QtTest>

using namespace  DoQuBook;
using namespace  DoQuBook::TestsUtils;


GenericTestPageService::GenericTestPageService(QObject* parent)
{
  DoQuBook::ApplicationContext::instance()->deactivateInternetPlugins();

  DoQuBook::PluginLoader *loader = PluginLoader::instance();
  loader->setPluginDirPath("./plugins/");
}

GenericCRUDServiceTest<Page, int>::CRUDServicePtr GenericTestPageService::buildService()  {
  GenericCRUDServiceTest<Page, int>::CRUDServicePtr srv = pageService;
  return srv;
}

QSharedPointer<Page> GenericTestPageService::buildEntity() {
  PagePtr page = Page::create();
  page->setImage(DATA_DIR+"/img2.png");
  return page;
}

void GenericTestPageService::modifyEntity(QSharedPointer<Page>& e) {
  e->setName("newname");
}

void GenericTestPageService::verifyUpdated(QSharedPointer<Page>& e) {
  QVERIFY(e->getName()=="newname");
}

void GenericTestPageService::init()
{
  PagePtr page = Page::create();
  page->setImage(DATA_DIR+"/img2.png");
  pageService->add(page);
  addedPageIds << page->getId();
}

void GenericTestPageService::cleanup()
{
  foreach (int id, addedPageIds) {
    pageService->del(id);
    // verify it is deleted
    PagePtr existingPage = pageService->get(id);
    QVERIFY(existingPage.isNull());
  }
  addedPageIds.clear();
}

void GenericTestPageService::testVerifyAddedPageinInit()
{
  foreach (int id, addedPageIds) {
    PagePtr existingPage = pageService->get(id);
    QVERIFY(!existingPage.isNull());
  }
}

void GenericTestPageService::testGenericAddGetUpdateDel()
{
  PagePtr p = buildEntity();
  CRUDServicePtr srv = buildService();
  testGetAddGetUpdateDel(p, srv);
}

void GenericTestPageService::testFindByTags()
{
  int pageid = addedPageIds.at(0);
  PagePtr existingPage = pageService->get(pageid);
  QVERIFY(!existingPage.isNull());

  QString tag = "unittest";
  pageService->addTag(existingPage, tag);

  // Verify that the new tag exists on the repo
  QStringList tags = pageService->existingTags();
  qDebug() << tags;
  QVERIFY(tags.contains(tag));

  // Verify that the page can be found by its tag
  meta_iterator<PagePtr>::IteratorPtr it = pageService->findByTag(tag);
  QList<int> pageids;
  while(it->hasNext()) {
    PagePtr p = it->next();
    qDebug() << p->getId() << " : " << p->getName();
    pageids << p->getId();
  }

  QVERIFY(pageids.contains(pageid));

  // remove the tag from the page
  pageService->removeTag(existingPage, tag);

  // verify that the page con NOT be found anymore
  it = pageService->findByTag(tag);
  pageids.clear();
  while(it->hasNext()) {
    PagePtr p = it->next();
    qDebug() << p->getId() << " : " << p->getName();
    pageids << p->getId();
  }

  QVERIFY(!pageids.contains(pageid));

  // verify that the tag does not exists anymore.
  tags = pageService->existingTags();
  qDebug() << tags;
  QVERIFY(!tags.contains(tag));
}
