#ifndef TESTUTILS_HPP
#define TESTUTILS_HPP

#include <QTextStream>
#include <QString>
#include <QTest>
#include <QFile>

#include <DoQuBook/AnnotatedPolygon.hpp>
#include <DoQuBook/AnnotationLocal.hpp>
#include <DoQuBook/AnnotationGlobal.hpp>
#include <DoQuBook/Page.hpp>
#include <DoQuBook/Book.hpp>
#include "Config.hpp"

using namespace DoQuBook;

QString readFile(const QString &filePath)
{
    QFile file(filePath);
    if(!file.open(QIODevice::ReadOnly))
        throw "could not open file";

    QTextStream in(&file);
    QString realJson;
    while (!in.atEnd()) {
        realJson += in.readLine();
    }
    return realJson;
}

DoQuBook::AnnotatedPolygonPtr createPolygon()
{
    DoQuBook::AnnotatedPolygonPtr annotatedPolygon = AnnotatedPolygon::create();
    DoQuBook::PointPtr p = DoQuBook::Point::create();
    p->setX(1); p->setY(2);

    DoQuBook::PointPtr p2 = DoQuBook::Point::create();
    p2->setX(2); p2->setY(1);

    annotatedPolygon->addPoint(p);
    annotatedPolygon->addPoint(p2);
    annotatedPolygon->setValue(12);

    return annotatedPolygon;
}

DoQuBook::AnnotationGlobalPtr createAnnotationGlobal()
{
    DoQuBook::AnnotationGlobalPtr annLocal = DoQuBook::AnnotationGlobal::create();
    annLocal->setName("annGlobal");
    annLocal->setValue(10);

    return annLocal;
}

DoQuBook::AnnotationLocalPtr createAnnotationLocal()
{
    AnnotationLocalPtr annLocal = AnnotationLocal::create();
    AnnotatedPolygonPtr polygon = createPolygon();

    annLocal->addAnnotatedPolygon(polygon);
    annLocal->setName("annLocal");

    return annLocal;
}


DoQuBook::PagePtr createPage()
{
    DoQuBook::PagePtr page = DoQuBook::Page::create();

    page->setName("pagename");
    page->setImage(testsDatasPath+"/image_1_.png");

    return page;
}

DoQuBook::BookPtr createBook()
{
    DoQuBook::BookPtr book = DoQuBook::Book::create();
    DoQuBook::PagePtr page = createPage();
    book->addPage(page);

    book->setName("bookName");
    return book;
}

DoQuBook::BookPtr createBookWithOneEmptyPage()
{
    DoQuBook::BookPtr book = DoQuBook::Book::create();
    DoQuBook::PagePtr page = createPage();
    book->addPage(page);

    book->setName("bookName");
    return book;
}

#endif // TESTUTILS_HPP
