#include <QtTest>
#include <DoQuBook/AnnotatedPolygon.hpp>
#include <assert.h>
#include <QDebug>
#include <QObject>

#include <DoQuBook/MarshallableFormater.hpp>
#include <Utils.hpp>

#include <DoQuBook/PluginLoader.hpp>

using namespace DoQuBook;

namespace Tests {

class TestAnnotatedPolygon : public QObject {

    Q_OBJECT

public :
    static const int annotatedPolygonValue;

    TestAnnotatedPolygon()
        :_annotatedPolygon(NULL)
    {}

    ~TestAnnotatedPolygon()
    {
        if(_annotatedPolygon)
            delete _annotatedPolygon;
    }

private slots:

    void testGetSetValue();
    void testEqual();
    void testMarshalling();
    //void testExportJava();

    /**
         * Called before each test function is executed.
        */
    void init();

    /**
         * Called after every test function.
        */
    void cleanup();

protected :
    DoQuBook::AnnotatedPolygon * _annotatedPolygon;
};

}
using namespace Tests;

const int TestAnnotatedPolygon::annotatedPolygonValue = 13;

void TestAnnotatedPolygon::init()
{
    _annotatedPolygon = new AnnotatedPolygon();
    assert(_annotatedPolygon);
}

void TestAnnotatedPolygon::cleanup()
{
    if(_annotatedPolygon)
        delete _annotatedPolygon;
    _annotatedPolygon = NULL;
}

void TestAnnotatedPolygon::testGetSetValue()
{
    _annotatedPolygon->setValue(annotatedPolygonValue);

    QVERIFY(_annotatedPolygon->getValue() == annotatedPolygonValue);
}


void TestAnnotatedPolygon::testEqual()
{
    AnnotatedPolygonPtr annPtr = createPolygon();
    AnnotatedPolygonPtr annPtr2 = createPolygon();

    QVERIFY(annPtr->equals(*annPtr2));

}

void TestAnnotatedPolygon::testMarshalling()
{
    DoQuBook::AnnotatedPolygonPtr annP = createPolygon();
    MarshallableFormaterPtr jsonFormater = PluginLoader::instance()->getPlugin<MarshallableFormater>("labri.QJSONMarshallableFormater.0.0.1");
    QString dest ;
    jsonFormater->marshall(annP, &dest);

    DoQuBook::AnnotatedPolygonPtr annPoly = AnnotatedPolygonPtr::create();
    jsonFormater->unMarshall(QByteArray(dest.toStdString().c_str()), annPoly);



    QVERIFY(annP->equals(*annPoly));
}

QTEST_MAIN(TestAnnotatedPolygon)

#if QT_VERSION >= 0x050000
  #include "testAnnotatedPolygon.moc"
#else
  #include "moc_testAnnotatedPolygon.cxx"
#endif
