#include <QtTest>
#include <DoQuBook/AnnotationGlobal.hpp>
#include "testAnnotation.hpp"
#include <assert.h>
#include <QDebug>
#include <QObject>
#include <Utils.hpp>
#include <DoQuBook/MarshallableFormater.hpp>
#include <DoQuBook/PluginLoader.hpp>

using namespace DoQuBook;

namespace Tests {
class TestAnnotationGlobal : public QObject {

    Q_OBJECT

public :
    static const QString annotationName;
    static const int annotationValue;

    TestAnnotationGlobal ();

    ~TestAnnotationGlobal();

private slots:

    void testGetSetValue();
    void testEqual();
    void testMarshalling ();

    /**
         * Called before each test function is executed.
        */
    void init();

    /**
         * Called after every test function.
        */
    void cleanup();

protected :
    AnnotationGlobalPtr _annotation;
};
}

using namespace  Tests;

const QString TestAnnotationGlobal::annotationName = "global bleed through";
const int TestAnnotationGlobal::annotationValue = 13;

void TestAnnotationGlobal::init()
{
    _annotation = AnnotationGlobalPtr(new AnnotationGlobal(annotationName));
    assert(!_annotation.isNull());
}

void TestAnnotationGlobal::cleanup()
{
    _annotation.clear();
}

TestAnnotationGlobal::TestAnnotationGlobal():_annotation(NULL){}

TestAnnotationGlobal::~TestAnnotationGlobal()
{
  if(!_annotation.isNull())
    _annotation.clear();
}

void TestAnnotationGlobal::testGetSetValue()
{
  _annotation->setValue(annotationValue);

  QVERIFY(_annotation->getValue() == annotationValue);
}


void TestAnnotationGlobal::testEqual()
{
    AnnotationGlobalPtr annGlobal1 = createAnnotationGlobal();
    AnnotationGlobalPtr annGlobal2 = createAnnotationGlobal();
    QVERIFY(annGlobal1->equals(*annGlobal2));
    annGlobal2->setValue(annGlobal1->getValue().toInt()+1);
    QVERIFY(!annGlobal1->equals(*annGlobal2));

}

void TestAnnotationGlobal::testMarshalling()
{
    AnnotationGlobalPtr annGlobal = createAnnotationGlobal();

    QString marshalled;
    MarshallableFormaterPtr jsonFormater = PluginLoader::instance()->getPlugin<MarshallableFormater>("labri.QJSONMarshallableFormater.0.0.1");
    jsonFormater->marshall(annGlobal, &marshalled);

    MarshallableFormaterPtr jsonImporter = PluginLoader::instance()->getPlugin<MarshallableFormater>("labri.QJSONMarshallableFormater.0.0.1");
    DoQuBook::AnnotationGlobalPtr annGlobal2 = AnnotationGlobal::create();
    jsonImporter->unMarshall(QByteArray(marshalled.toStdString().c_str()), annGlobal2);

    QVERIFY(annGlobal->equals(*annGlobal2));
}


QTEST_MAIN(TestAnnotationGlobal)


#ifdef HAVE_QT5
  #include "testAnnotationGlobal.moc"
#else
  #include "moc_testAnnotationGlobal.cxx"
#endif

