#include <QtTest>
#include "Utils.hpp"
#include <DoQuBook/ApplicationContext.hpp>
#include <iostream>

using namespace DoQuBook;

namespace Tests {

class TestConcurencyAndAutoUpdates : public QObject {

    Q_OBJECT

public :

    TestConcurencyAndAutoUpdates() {}

    ~TestConcurencyAndAutoUpdates()
    {}

private slots:

    void testConcurency();
    void testAutoUpdate();

    /**
         * Called before each test function is executed.
        */
    void init();

    /**
         * Called after every test function.
        */
    void cleanup();

protected :
    QList<BookPtr> _toBeCleanedUpBooks;
    BookServicePtr _bookService;
};

}

using namespace  Tests;
QTEST_MAIN(TestConcurencyAndAutoUpdates)


void TestConcurencyAndAutoUpdates::init()
{
    /*RepositoryPtr repo = ApplicationContext::instance()->getRepository("LocalhostDev");
    _bookService = repo->getBookService();

    BookPtr b = createBookWithOneEmptyPage();
    _bookService->put (b);

    QVERIFY(b->getId() != -1);

    _toBeCleanedUpBooks.append(b);*/
}

void TestConcurencyAndAutoUpdates::cleanup()
{
    /*foreach(BookPtr b, _toBeCleanedUpBooks) {
        _bookService->del (b);
    }
    _toBeCleanedUpBooks.clear();*/
}

class AutoUpdateBook : public QObject {

    Q_OBJECT

public:
    AutoUpdateBook(BookServicePtr srv, BookPtr b, bool &ann)
        : _srv (srv), _b (b), _ann (ann) {}

public slots :
    virtual void updateBook()
    {
        BookPtr b2 = _srv->get(_b->getId());
        _b->merge(b2);

        PagePtr page = _b->getPages().at(0);
        bool ann = page->getAnnotationGlobal("Global_IsWhitePage")->getValue().toBool();
        // qWarning() << ann;
        QVERIFY(ann==_ann);
    }

public:
    BookServicePtr _srv;
    BookPtr _b;
    bool &_ann;
};


class AutoModfifyBook : public QObject {

    Q_OBJECT

public:
    AutoModfifyBook (BookServicePtr srv, BookPtr b,  bool &ann)
        : _srv (srv), _b (b), _ann (ann) {}

public slots :
    virtual void updateBook()
    {
        PagePtr page = _b->getPages().at(0);
        _ann = !_ann;
        page->addGlobalAnnotation("Global_IsWhitePage", _ann);
        _srv->update(_b);
    }

public:
    BookServicePtr _srv;
    BookPtr _b;
     bool &_ann;
};



void TestConcurencyAndAutoUpdates::testConcurency()
{
    int toBeCleanedBookId = _toBeCleanedUpBooks.at(0)->getId();
    /// Extract 2 different books at same state.
    BookPtr b = _bookService->get (toBeCleanedBookId);
    BookPtr b2 = _bookService->get (toBeCleanedBookId);
    int version = b2->getVersion();
    /// Their page at index 0
    PagePtr page = b->getPages().at(0);
    PagePtr page2 = b2->getPages().at(0);
    /// We add an annotation to the first book
    page->addGlobalAnnotation("Global_IsWhitePage", false);
    /// But delete it on the second at the same time
    page2->deleteGlobalAnnotation("Global_IsWhitePage");

    /// We update the book b (b should be valid (ie : onTheServer.version == b.version)
    _bookService->update(b);
    /// The version on the server is updated (+1)
    version++;

    /// We update the second book (b2 is not valid and has to be merged)
    _bookService->update(b2);
    /// b2 is NOT valid, no Annotation should be ERASED !
    /// But fields are still updated.
    // qDebug() << b2->getPages().at(0)->getAnnotationGlobal(Defects::Global_IsWhitePage)->getValue().toBool();
    QVERIFY(b2->getPages().at(0)->getAnnotationGlobal("Global_IsWhitePage")->getValue().toBool() == false);
    QVERIFY(page2->getAnnotationGlobal("Global_IsWhitePage")->getValue().toBool() == false);
    /// The version on the server is updated (+1)
    version++;

    /// Now both the server and b2 are synced
    QVERIFY(b2->getVersion()==version);

    /// remove the global annotation
    b2->getPages().at(0)->deleteGlobalAnnotation("Global_IsWhitePage");
    _bookService->update(b2);

    /// Verify that the annotation does not exists on server :
    BookPtr b3 = _bookService->get (toBeCleanedBookId);
    QVERIFY(b3->getPages().at(0)->getAnnotation("Global_IsWhitePage").isNull());

}

void TestConcurencyAndAutoUpdates::testAutoUpdate()
{
    int toBeCleanedBookId = _toBeCleanedUpBooks.at(0)->getId();

    BookPtr b = _bookService->get (toBeCleanedBookId);
    b->getPages().at(0)->addGlobalAnnotation("Global_IsWhitePage", false);
    _bookService->update(b);
    QVERIFY(!b.isNull());

    bool ann = false;

    AutoUpdateBook *autoUpdate = new AutoUpdateBook (_bookService, b, ann);
    QTimer *timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), autoUpdate, SLOT(updateBook()));
    timer->start(1000);

    AutoModfifyBook *autoShow = new AutoModfifyBook (_bookService, b, ann);
    QTimer *timer2 = new QTimer(this);
    connect(timer2, SIGNAL(timeout()), autoShow, SLOT(updateBook()));
    timer2->start(2000);

    QTest::qWait(10000);
}

#if QT_VERSION >= 0x050000
  #include "testConcurencyAndAutoUpdates.moc"
#else
  #include "moc_testConcurencyAndAutoUpdates.cxx"
#endif
