
#include <iostream>
#include <QApplication>
#include <QLabel>
#include <DoQuBook/ApplicationContext.hpp>

#include <future>
#include <thread>
#include <chrono>


#include <DoQuBook/WebClient.hpp>


using namespace  DoQuBook;



template <class T>
class AsyncRESTObjectManager {

   public :
    AsyncRESTObjectManager ()
    {}
      std::future<QSharedPointer<T>> get (const QString url, std::function<void (QSharedPointer<T>)> func) {

        std::future<QSharedPointer<T>>  f = p.get_future();                     ;
        auto unMarshall = [=] (const QByteArray& response){
          QSharedPointer<T> b;
          if (!response.isEmpty() ) {
              b = m_formater.unMarshall (response);
              func(b);
              p.set_value(b);
          }

        };

        wc.get(url, unMarshall);
        return f;

      }
  private :
      std::promise<QSharedPointer<T>> p;
      MarshallableFormaterJSon<T> m_formater;
      AsyncWebClientHelper wc;
};


class AsyncRESTImageManager {
   public :
      std::future<QImage> get (const QString url, std::function<void (QImage)> func) {

        std::future<QImage>  f = p.get_future();
        auto unMarshall = [=] (const QByteArray& response){
          QImage img;
          if (!response.isEmpty() ) {
              img.loadFromData (response);
              func(img);
              p.set_value(img);
          }

        };
        wc.get(url, unMarshall);
        return f;

      }
  private :
      std::promise<QImage> p;
      AsyncWebClientHelper wc;
};

int main(int argc, char *argv[]) {

  QApplication app(argc, argv);
  app.setApplicationName("testCpp11Features");
  app.setOrganizationName("LaBRI");

  auto saveImage = [] (QImage img) {
    img.save("monImage.png");
    qDebug() << "Saved Image";
    QLabel * l = new QLabel();
    l->setPixmap(QPixmap::fromImage(img));
    l->show();
  };

  auto showName = [] (PagePtr p) {
    qDebug() << "Show name " << p->getName();
  };

  AsyncRESTImageManager imageManager;
  imageManager.get("http://localhostdev:8080/DoQuBookWeb/rest/image/63", saveImage);


  AsyncRESTObjectManager<Page> manager;
  manager.get("http://localhostdev:8080/DoQuBookWeb/rest/page/2", showName);

  AsyncRESTObjectManager<Page> manager2;
  manager2.get("http://localhostdev:8080/DoQuBookWeb/rest/page/4", showName);
  AsyncRESTObjectManager<Page> manager3;
  manager3.get("http://localhostdev:8080/DoQuBookWeb/rest/page/62", showName);
  AsyncRESTObjectManager<Page> manager4;
  manager4.get("http://localhostdev:8080/DoQuBookWeb/rest/page/63", showName);


  PageRESTResponseManager m;
  m.get("http://localhostdev:8080/DoQuBookWeb/rest/page/2");
  m.connect(&m, &PageRESTResponseManager::pageReceived, showName);

  app.exec();
}


/*

  RepositoryPtr repo = ApplicationContext::instance()->getRepository("LocalhostDev");
  PageServicePtr pageService = repo->getPageService();

  //pageService->onPageTaggedEvent("mytag", [=](const PagePtr &p){
   // qDebug() << "CallBack :";
   // qDebug() << p->getName();
  //});/

  auto func = [](const float &p) -> float {
    std::cout << p << std::endl;
    return 2*p;
  };

  float final = func(2.0);
  std::cout << final << std::endl;




  std::packaged_task<PagePtr (std::function<void (const PagePtr&)>)> task([=](std::function<void (const PagePtr&)> func)->PagePtr {
    std::this_thread::sleep_for(std::chrono::milliseconds(2000));
    PagePtr p = pageService->get(62);
    func(p);
    return p;
  });
  std::future<PagePtr> f1 = task.get_future();
  task([=](const PagePtr & p){
    qDebug() << QTime::currentTime().toString ("hh:mm:ss:zzz") << "In callback : "<< p->getName();
  });
  std::thread(std::move(task), [=](const PagePtr & p){
    qDebug() << QTime::currentTime().toString ("hh:mm:ss:zzz") << "In callback : "<< p->getName();
  }).detach();

  int i = 0;
  while(i <= 5) {
    qDebug() << "Sleeping";
    std::this_thread::sleep_for(std::chrono::milliseconds(1000));
    i++;
  }
  qDebug() << QTime::currentTime().toString ("hh:mm:ss:zzz") << f1.get()->getName();
*/
