#include <QObject>
#include <QtTest>
#include <DoQuBook/MarshallableFormater.hpp>
#include <Utils.hpp>
#include <DoQuBook/RESTObjectManager.hpp>
#include <DoQuBook/PluginLoader.hpp>

class TestServerMarshalling : public QObject
{
    Q_OBJECT
public:
    TestServerMarshalling() {
      marshaller = PluginLoader::instance()->getPlugin<MarshallableFormater>("labri.QJSONMarshallableFormater.0.0.1");
    }
    virtual ~TestServerMarshalling() {}

private slots:
    /**
      * Called before each test function is executed.
      */
    void init();

    /**
      * Called after every test function.
      */
    void cleanup();

    void testMarshallingAnnotationLocal ();
    void testMarshallingPage ();
    void testMarshallingAnnotationGlobal ();

  private :
    MarshallableFormaterPtr marshaller;

};

void TestServerMarshalling::testMarshallingAnnotationLocal()
{
    RESTObjectManager<AnnotationLocal> annotationManager (WebClient::JSON, marshaller);
    annotationManager.addRessourcePath(WebClient::POST, "http://localhost:8080/DoQuBookWeb/rest/marshalling_tests/annotation");

    AnnotationLocalPtr annLocal = createAnnotationLocal();
    AnnotationLocalPtr receivedAnnLocal = annotationManager.post(annLocal);
    QVERIFY(annLocal->equals(*receivedAnnLocal));
}

void TestServerMarshalling::testMarshallingPage()
{
    RESTObjectManager<Page> pageManager (WebClient::JSON, marshaller);
    pageManager.addRessourcePath(WebClient::POST, "http://localhost:8080/DoQuBookWeb/rest/marshalling_tests/page");

    PagePtr page = createPage();

    try {
        PagePtr receivedPage = pageManager.post(page);
        QVERIFY(page->getName() == receivedPage->getName());
        QVERIFY(page->equals(*receivedPage));
    } catch (CouldNotParseJSONException e) {
        qCritical() << "WRONG response from server";
        qCritical() << e.getFailedJSONString();
        QVERIFY(false);
    }

}

void TestServerMarshalling::testMarshallingAnnotationGlobal()
{
    RESTObjectManager<Annotation> annotationManager (WebClient::JSON, marshaller);
    annotationManager.addRessourcePath(WebClient::POST, "http://localhost:8080/DoQuBookWeb/rest/marshalling_tests/annotation");

    AnnotationGlobalPtr annGlobal = createAnnotationGlobal();
    AnnotationGlobalPtr receivedAnnGlobal = annotationManager.post(annGlobal).dynamicCast<AnnotationGlobal>();
    QVERIFY(annGlobal->equals(*receivedAnnGlobal));

    QVERIFY (receivedAnnGlobal->metaObject()->className()==QString("DoQuBook::AnnotationGlobal"));
}

void TestServerMarshalling::cleanup()
{
}

void TestServerMarshalling::init()
{
}

QTEST_MAIN(TestServerMarshalling)

#if QT_VERSION >= 0x050000
  #include "testServerMarshalling.moc"
#else
  #include "moc_testServerMarshalling.cxx"
#endif
