#include "BinarizationNiblack.hpp"
#include <QDebug>
#include <QtPlugin>

#include <cv.h>
#include <highgui.h>
#include <iostream>
#include <DoQuBook/DoQuBookOpenCVConverter.hpp>

#include <ise/iseSegmentationContext.hpp>
#include <ise/iseSegmentationClustering.hpp>
#include <ise/iseSplitAlgorithm.hpp>
#include <ise/iseSplitDriverDefault.hpp>
#include <ise/iseSplitAlgorithmDefault.hpp>

#include <ise/iseHistogram1D.hpp>
#include <ise/iseSegmentationClusteringLocalThresholds.hpp>

#include <ise/iseSegmentationClusteringLocalThresholderNiblack.hpp>

#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsRegionInfos.hpp>

using namespace DoQuBook;

using namespace Binarizers;

class RegionColorizer : public girl::toger::pointIterator
{
public:
  RegionColorizer(QImage &img, QRgb v)
    : image(img)
    , value(v)
  {}

  virtual void processPoint(const girl::point &pt)
  {
      image.setPixel(pt.x(), pt.y(), value);
  }

protected:

  QImage &image;
  QRgb value;

};


cv::RNG rng(12345);

girl::region
getBiggerRegion(const girl::toger &tog, const girl::regionDeque &regions)
{
    girl::region max = regions.at(0);
    size_t airmax = girl::tools::computeArea(tog,max);

    foreach (girl::region r, regions) {
        size_t regionArea = girl::tools::computeArea (tog, r);
        if (regionArea > airmax) {
            max = r;
            airmax = regionArea;
        }
    }

    return max;
}

#include <QTime>

QImage BinarizationNiblack::binarize(const QImage &qimage, const QVariantMap &/*params */)
{
    DoQuBookOpenCVConverter converteur;
    cv::Mat src = converteur.getCvMat(qimage);

    girl::tools::Image image (src);

    girl::toger tog(image.width(), image.height());


    ise::SegmentationContext segContext;
    segContext.setToger(&tog);
    segContext.setImage(&image);

    girl::regionDeque toBeSplittedRegions;
    /// On choisis la premiere region à spliter
    toBeSplittedRegions.push_back(1);

    ise::SplitAlgorithmDefault splitAlgorithm(&segContext);
    ise::SplitDriverDefault defaultDriver(&segContext);

    girl::regionDeque createdRegions;
    ise::SegmentationClusteringLocalThresholds segCluster(&segContext);
    ise::SegmentationClusteringLocalThresholderNiblack *localThresholder = new ise::SegmentationClusteringLocalThresholderNiblack;
    
    girl::shared_ptr<ise::SegmentationClusteringLocalThresholder> thresholder (localThresholder);
    segCluster.setParams(thresholder);

    splitAlgorithm.apply(toBeSplittedRegions, defaultDriver, segCluster , 0, createdRegions);

    QImage img (image.width(), image.height(), QImage::Format_ARGB32);

    RegionColorizer blackColorizer (img, qRgb(0, 0, 0));
    RegionColorizer whiteColorizer (img, qRgb(255, 255, 255));

    girl::region back = getBiggerRegion(tog, createdRegions);

    foreach (const girl::region r, createdRegions) {
        girl::regionDeque neighbours;
        tog.regionNeighbourhood(r, girl::toger::NEIGHBOUR_ANY, girl::toger::MARK_ANY, neighbours);
        bool hasBackGround = false;
        unsigned int i = 0;
        while(!hasBackGround && i < neighbours.size())
        {
            hasBackGround = (neighbours[i] == back);
            i++;
        }
        if(r != back && hasBackGround) {
            tog.iterateOnRegionDomain(r, whiteColorizer);
        } else {
            tog.iterateOnRegionDomain(r, blackColorizer);
        }
    }

    return img;
}


DQB_EXPORT_PLUGIN(BinarizationNiblack)

