#include "BinarizationOtsu2T.hpp"
#include <QtPlugin>
#include <cv.h>
#include <highgui.h>
#include <iostream>
#include <DoQuBook/DoQuBookOpenCVConverter.hpp>

#include <ise/iseSegmentationContext.hpp>
#include <ise/iseSegmentationClustering.hpp>
#include <ise/iseSplitAlgorithm.hpp>
#include <ise/iseSplitDriverDefault.hpp>
#include <ise/iseSplitAlgorithmDefault.hpp>
#include <ise/iseQuantificationThresholderOtsu2T.hpp>
#include <ise/iseHistogram1D.hpp>
#include <ise/iseSegmentationClusteringLocalThresholds.hpp>
#include <ise/iseSegmentationClusteringLocalThresholderSauvola.hpp>

#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsRegionInfos.hpp>

using namespace DoQuBook;

using namespace Binarizers;

QImage BinarizationOtsu2T::binarize(const QImage & qimage, const QVariantMap &/*params*/)
{
    DoQuBookOpenCVConverter converteur;
    cv::Mat src = converteur.getCvMat(qimage);

    girl::tools::Image image (src);

    ise::QuantificationThresholderOtsu2T quantif;
    ise::Histogram1D h(image, 0, 0, 255);
    std::vector<double> levels;
    quantif.getThresholds(h, levels);


    cv::Mat g_gray = cv::Mat::zeros(src.rows, src.cols, CV_8UC1);;
    cv::cvtColor( src, g_gray, CV_RGB2GRAY );

    cv::Mat out(g_gray.rows, g_gray.cols, cv::DataType<uchar>::type);

    int nl= out.rows; // number of lines
    int nc= out.cols ; // number of columns

    // is it a continous image?
    if (out.isContinuous())  {
        nc= nc*nl;
        nl= 1;  // it is now a 1D array
    }

    for (int j=0; j<nl; j++) {
        // pointer to first column of line j
        uchar* data= g_gray.ptr<uchar>(j);
        uchar* outData= out.ptr<uchar>(j);
        for (int i=0; i<nc; i++) {


            if( *(data) <= levels[0])
                *outData++= 255; // only one channel
            else
                *outData++= 0;
            data++;
        }
    }

    cv::imwrite("OCVBinOtsu2T.png", out);


    return converteur.getQImage(out);
}


DQB_EXPORT_PLUGIN(BinarizationOtsu2T)


/* Usefull code to keep :
    cv::Mat g_gray = cv::Mat::zeros(src.rows, src.cols, CV_8UC1);;
    cv::cvtColor( src, g_gray, CV_RGB2GRAY );
    cv::imwrite("g_gray.png", g_gray);

    cv::Mat dst = cv::Mat::zeros(src.rows, src.cols, CV_8UC3);

    cv::Mat_<cv::Vec3b>::const_iterator it=  g_gray.begin<cv::Vec3b>();
    cv::Mat_<cv::Vec3b>::const_iterator itend= g_gray.end<cv::Vec3b>();
    cv::Mat_<cv::Vec3b>::iterator itout= dst.begin<cv::Vec3b>();
    // for each pixel
    for ( ; it!= itend; ++it, ++itout) {
        // process each pixel ---------------------
        const cv::Vec3b &pixel = *it;
        cv::Vec3b &out = *itout;
        if(pixel[0] <= levels[0] ) {
            out[0]= 255;
            out[1]= 255;
            out[2]= 255;
        } else {
            out[0]= 0;
            out[1]= 0;
            out[2]= 0;
        }
        // end of pixel processing ----------------
    }
    cv::imwrite("dst.png", dst);

    DoQuBookOpenCVConverter converteur;

    return  converteur.getQImage(dst);
    */

