#include "BinarizationShanbag.hpp"
#include <QDebug>
#include <QtPlugin>

#include <cv.h>
#include <highgui.h>
#include <iostream>
#include <DoQuBook/DoQuBookOpenCVConverter.hpp>

#include <ise/iseSegmentationContext.hpp>
#include <ise/iseSegmentationClustering.hpp>
#include <ise/iseSplitAlgorithm.hpp>
#include <ise/iseSplitDriverDefault.hpp>
#include <ise/iseSplitAlgorithmDefault.hpp>

#include <ise/iseQuantificationThresholderShanbag.hpp>


#include <ise/iseHistogram1D.hpp>

#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsRegionInfos.hpp>

using namespace DoQuBook;

using namespace Binarizers;

QImage BinarizationShanbag::binarize(const QImage & qimage, const QVariantMap &/*params*/)
{
    DoQuBookOpenCVConverter converteur;
    cv::Mat src = converteur.getCvMat(qimage);

    girl::tools::Image image (src);

    ise::QuantificationThresholderShanbag quantif;
    ise::Histogram1D h(image, 0, 0, 255);
    std::vector<double> levels;
    quantif.getThresholds(h, levels);

    cv::Mat g_gray = cv::Mat::zeros(src.rows, src.cols, CV_8UC1);;
    cv::cvtColor( src, g_gray, CV_RGB2GRAY );

    cv::Mat out(g_gray.rows, g_gray.cols, cv::DataType<uchar>::type);

    int nl= out.rows; // number of lines
    int nc= out.cols ; // number of columns

    // is it a continous image?
    if (out.isContinuous())  {
        nc= nc*nl;
        nl= 1;  // it is now a 1D array
    }

    for (int j=0; j<nl; j++) {
        // pointer to first column of line j
        uchar* data= g_gray.ptr<uchar>(j);
        uchar* outData= out.ptr<uchar>(j);
        for (int i=0; i<nc; i++) {
            if( *(data) <= levels[0])
                *outData++= 255; // only one channel
            else
                *outData++= 0;
            data++;
        }
    }

    return converteur.getQImage(out);
}


DQB_EXPORT_PLUGIN(BinarizationShanbag)

