#include "GlobalBinarization.hpp"

#include <cv.h>
#include <highgui.h>
#include <iostream>

#include <DoQuBook/DoQuBookOpenCVConverter.hpp>

#include <QDebug>
#include <QtPlugin>

using namespace DoQuBook;

QImage GlobalBinarization::binarize(const QImage & qimage, const QVariantMap &params)
{
    DoQuBookOpenCVConverter converter;
    cv::Mat mat = converter.getCvMat(qimage);
    int thd = 100;
    if(params.contains("thd"))
        thd = params["thd"].toInt();

    qDebug() << "Global Thd :" << thd;


    cv::Mat g_gray = cv::Mat::zeros(mat.rows, mat.cols, CV_8UC1);;
    cv::cvtColor( mat, g_gray, CV_RGB2GRAY );

    cv::Mat out(g_gray.rows, g_gray.cols, cv::DataType<uchar>::type);

    int nl= out.rows; // number of lines
    int nc= out.cols ; // number of columns

    // is it a continous image?
    if (out.isContinuous())  {
        nc= nc*nl;
        nl= 1;  // it is now a 1D array
    }

    for (int j=0; j<nl; j++) {
        // pointer to first column of line j
        uchar* data= g_gray.ptr<uchar>(j);
        uchar* outData= out.ptr<uchar>(j);
        for (int i=0; i<nc; i++) {
            if( *(data+1) <= thd)
                *outData++= 255; // only one channel
            else
                *outData++= 0;
            data++;
        }
    }

    cv::imwrite("NormalOUT.png", out);

    return converter.getQImage(out);
}


DQB_EXPORT_PLUGIN(GlobalBinarization)
