#include "BleedThroughDegradationModel.hpp"

#include <cv.h>
#include <highgui.h>
#include <iostream>

#include <DoQuBook/DoQuBookOpenCVConverter.hpp>

#include <QDebug>
#include <QtPlugin>

#include <iostream>

#include "DoQuBook/ApplicationContext.hpp"
#include "DoQuBook/Repository.hpp"
#include "DoQuBook/PageCollectionService.hpp"
#include "DoQuBook/PageService.hpp"

using namespace DoQuBook;
using namespace labri;
using namespace DegradationModels;



QImage BleedThroughDegradationModel::degradate(const QImage &recto, const QVariantMap &parameters)
{
    DoQuBookOpenCVConverter converter;
    cv::Mat rectoOriginal = converter.getCvMat(recto);
   
    QVariant v = parameters["verso"];
    QImage qverso = v.value<QImage> ();

    if(qverso.isNull())
        throw MessageException("the verso parameter is null ... ");
    
    if(qverso.size() != recto.size()) 
        throw MessageException("recto and verso must have the same size");
		
    qverso = qverso.mirrored(true, false);

    cv::Mat verso = converter.getCvMat(qverso);

    /// recto and verso should have the same size.
    //  cv::resize(verso, verso, cv::Size(rectoOriginal.cols, rectoOriginal.rows));
    
    int nbIter = 100;
    bool old  = true;
    dt = 0.166666;
    sigmaB = 51;
    sigmaInk = 100;

    if(parameters.contains("optimized_old"))
        old = parameters["optimized_old"].toBool();
    if(parameters.contains("nb_iterations"))
        nbIter = parameters["nb_iterations"].toInt();

    if(parameters.contains("dt"))
        dt = parameters["dt"].toFloat();
    if(parameters.contains("sigma_b"))
        sigmaB = parameters["sigma_b"].toFloat();
    if(parameters.contains("sigma_ink"))
        sigmaInk = parameters["sigma_ink"].toFloat();

    int w = rectoOriginal.rows;
    int h = rectoOriginal.cols;


    cv::Mat out(w, h, cv::DataType<float>::type);

    for(int x = 0 ; x < rectoOriginal.rows; x++){
        for(int y = 0 ; y < rectoOriginal.cols; y++){
            float p =  rectoOriginal.at<cv::Vec3b>(x,y)[0];
            out.at<float>(x,y) = p;
        }
    }

    for (int k = 0 ; k <= nbIter; k++) {
        cv::Mat tempRecto = out.clone();

        for(int x = 0 ; x < rectoOriginal.rows; x++){
            for(int y = 0 ; y < rectoOriginal.cols; y++){

                float current = tempRecto.at<float>(x,y);

                float px = x-1;
                float nx = x+1;
                float py = y-1;
                float ny = y+1;

                if (px<0)
                    px=0;
                if (nx>=rectoOriginal.rows)
                    nx=rectoOriginal.rows-1;
                if (py<0)
                    py=0;
                if (ny>=rectoOriginal.cols)
                    ny=rectoOriginal.cols-1;

                cv::Vec3b pixel = verso.at<cv::Vec3b>(px,y);
                float ixp = pixel[0];
                pixel = verso.at<cv::Vec3b>(nx,y);
                float ixn = pixel[0];
                pixel = verso.at<cv::Vec3b>(x,ny);
                float iyp = pixel[0];
                pixel = verso.at<cv::Vec3b>(x,py);
                float iyn = pixel[0];


               /* float ixp = 255;
                float ixn = 255;
                float iyp = 255;
                float iyn = 255;*/

                float u = rectoOriginal.at<cv::Vec3b>(x,y)[0];

                float g;
                if(old)
                    g = runForOldDocument(current, u,
                                          ixp, ixn, iyp, iyn);
                else
                    g = runForCleanDocument(current, u,
                                            ixp, ixn, iyp, iyn);



                out.at<float>(x,y) = g;

            }
        }
    }

// TODO: need a fix on the OpenCVConverter
		cv::imwrite("convertTmp.png", out);
		QImage qout ("convertTmp.png");
		return qout;
    //return converter.getQImage(out);

}

float BleedThroughDegradationModel::runForCleanDocument(float current, float u, float ixp, float ixn, float iyp, float iyn)
{
    float c =  (1.0 / (1.0 + (ixn-u)*(ixn-u)/sigmaInk*sigmaInk) )  * (1.0 / ((1.0 + ixn*ixn)/(sigmaB*sigmaB)) );
    float c1 = (1.0 / (1.0 + (ixp-u)*(ixp-u)/sigmaInk*sigmaInk) )  * (1.0 / ((1.0 + ixp*ixp)/(sigmaB*sigmaB)) );
    float c2 = (1.0 / (1.0 + (iyp-u)*(iyp-u)/sigmaInk*sigmaInk) )  * (1.0 / ((1.0 + iyp*iyp)/(sigmaB*sigmaB)) );
    float c3 = (1.0 / (1.0 + (iyn-u)*(iyn-u)/sigmaInk*sigmaInk) )  * (1.0 / ((1.0 + iyn*iyn)/(sigmaB*sigmaB)) );

    float delta =  (c * (ixn - current)
                    + c1 * (ixp - current)
                    + c2 * (iyp -  current)
                    + c3 * (iyn-  current));

    return (dt * delta + current) ;

}

int BleedThroughDegradationModel::runForOldDocument(float current, float u, float ixp, float ixn, float iyp, float iyn)
{
    float c =  (1.0 / (1.0 + (ixn-u)*(ixn-u)/sigmaInk*sigmaInk) )  ; // * (1.0 / ((1.0 + ixn*ixn)/(rateOfBleedThrough*rateOfBleedThrough)) );
    float c1 = (1.0 / (1.0 + (ixp-u)*(ixp-u)/sigmaInk*sigmaInk) )  ; //* (1.0 / ((1.0 + ixp*ixp)/(rateOfBleedThrough*rateOfBleedThrough)) );
    float c2 = (1.0 / (1.0 + (iyp-u)*(iyp-u)/sigmaInk*sigmaInk) )  ; //* (1.0 / ((1.0 + iyp*iyp)/(rateOfBleedThrough*rateOfBleedThrough)) );
    float c3 = (1.0 / (1.0 + (iyn-u)*(iyn-u)/sigmaInk*sigmaInk) )  ; //* (1.0 / ((1.0 + iyn*iyn)/(rateOfBleedThrough*rateOfBleedThrough)) );

    float delta =  (c * (ixn - current)
                    + c1 * (ixp - current)
                    + c2 * (iyp -  current)
                    + c3 * (iyn-  current));


    return (int) (dt * delta + current) ;
}

DQB_EXPORT_PLUGIN(BleedThroughDegradationModel)
