#include <DoQuBook/DegradationModel.hpp>
#include <PluginConfig.hpp>

namespace labri {
namespace DegradationModels {

class plugin_EXPORT BleedThroughDegradationModel : public DegradationModel
{
    Q_OBJECT
    DQB_PLUGIN(DegradationModel, "fr.labri.doqubook.DegradationModel")

    virtual QList<DoQuBook::ExecutorParameterPtr> parameters () {
        QList<DoQuBook::ExecutorParameterPtr> params;
        params << DoQuBook::ExecutorParameter::create("verso", "Image", true);
        params << DoQuBook::ExecutorParameter::create("nb_iterations", "Number", false);
        params << DoQuBook::ExecutorParameter::create("optimized_old", "Boolean", false);
        return params;
    }

    /**
      * Usage example :
      * \code
      *  QImage recto(...);
      *  QImage verso(...);
      *
      *  QVariantMap params;
      *  params ["verso"] = verso;
      *  params ["nb_iterations"] = 30;
      *  params ["optimized_old"] = false;
      *
      *  DoQuBook::PluginLoader * loader = PluginLoader::instance();
      *  DegradationModelPtr model = loader->getPlugin<DegradationModel>("labri::DegradationModels::BleedThroughDegradationModel");
      *
      *
      *  QImage res = model->degradate(recto, params);
      *  res.save("bleedthrough_image.png");
      *
      * \endcode
      *
      */
    virtual QImage degradate(const QImage &image,  const QVariantMap &parameters = QVariantMap());

private :
    float runForCleanDocument(float current, float u,
                              float ixp,
                              float ixn,
                              float iyp,
                              float iyn);

    int runForOldDocument(float current, float u,
                              float ixp,
                              float ixn,
                              float iyp,
                              float iyn);

    /// Plus haut que les valeur de l'encre du verso
    float sigmaB;

    float sigmaInk;

    float dt;
};

}
}
