#include <DoQuBook/DegradationModel.hpp>
#include <PluginConfig.hpp>

namespace labri {
namespace DegradationModels {

class plugin_EXPORT SampleLinkedParameters : public DegradationModel
{
    Q_OBJECT
    DQB_PLUGIN(DegradationModel, "fr.labri.doqubook.DegradationModel")
    virtual QList<DoQuBook::ExecutorParameterPtr> parameters () {
        QList<DoQuBook::ExecutorParameterPtr> params;
        params << DoQuBook::ExecutorParameter::create("p1", "Number", true);
        params << DoQuBook::ExecutorParameter::create("p2", "Number", true);
        return params;
    }

		QVariantMap onParameterChanged(const QString &name, const QVariant &var);

    /**
      * Usage example :
      * \code
      *  QImage recto(...);
      *  QImage verso(...);
      *
      *  QVariantMap params;
      *  params ["verso"] = verso;
      *  params ["nb_iterations"] = 30;
      *  params ["optimized_old"] = false;
      *
      *  DoQuBook::PluginLoader * loader = PluginLoader::instance();
      *  DegradationModelPtr model = loader->getPlugin<DegradationModel>("labri::DegradationModels::BleedThroughDegradationModel");
      *
      *
      *  QImage res = model->degradate(recto, params);
      *  res.save("bleedthrough_image.png");
      *
      * \endcode
      *
      */
    virtual QImage degradate(const QImage &image,  const QVariantMap &parameters = QVariantMap());
};

}
}
