#include "FScoreImage.hpp"

#include <cv.h>
#include <highgui.h>
#include <iostream>

#include <DoQuBook/DoQuBookOpenCVConverter.hpp>

#include <QDebug>
#include <QtPlugin>

using namespace DoQuBook;


double FScoreImage::compare(const QImage &image1, const QImage &image2)
{
    int color = 255; // 0 if bin inverse
    DoQuBookOpenCVConverter converter;
    cv::Mat mat1 = converter.getCvMat(image1);
    cv::Mat mat2 = converter.getCvMat(image2);

    cv::Mat g_grayMat1 = cv::Mat::zeros(mat1.rows, mat1.cols, CV_8UC1);;
    cv::cvtColor( mat1, g_grayMat1, CV_RGB2GRAY );

    cv::Mat g_grayMat2 = cv::Mat::zeros(mat2.rows, mat2.cols, CV_8UC1);;
    cv::cvtColor( mat2, g_grayMat2, CV_RGB2GRAY );

#if DEBUG_LOG
    cv::Mat out = cv::Mat::zeros(mat2.rows, mat2.cols, CV_8UC1);
#endif

    double fscore = -1;

    double correctBlack = 0.0;
    double foundBlack = 0.0;
    double errorsBlack = 0.0;
    double shouldHaveBeenReturned = 0.0;

    int nl= g_grayMat1.rows; // number of lines
    int nc= g_grayMat1.cols ; // number of columns

    // is it a continous image?
    if (g_grayMat1.isContinuous())  {
        nc= nc*nl;
        nl= 1;  // it is now a 1D array
    }

    double mismatch = 0.0;
    double same = 0.0;

    for (int j=0; j<nl; j++) {
        // pointer to first column of line j
        uchar* mat1Data= g_grayMat1.ptr<uchar>(j);
        uchar* mat2Data= g_grayMat2.ptr<uchar>(j);
#if DEBUG_LOG
        uchar* outData= out.ptr<uchar>(j);
#endif
        for (int i=0; i<nc; i++) {
            if(*(mat1Data) != *(mat2Data))
                mismatch++;
            else
                same++;


            if(*(mat1Data) == color) {
                shouldHaveBeenReturned++;
                if(*(mat2Data) == color) {
                    correctBlack++;
                    foundBlack++;
                } else {
                    errorsBlack++;
                    #if DEBUG_LOG
                    *outData = 255;
                    #endif
                }
            }else if(*(mat2Data) == color) {
                foundBlack++;
            }
            mat1Data++;
            mat2Data++;
            #if DEBUG_LOG
            outData++;
            #endif
        }
    }
#if DEBUG_LOG
    cv::imwrite("LOGFSCORE.png", out);
#endif

    //qDebug() << "CorrectBlack percent :" << (100 *correctBlack)/ shouldHaveBeenReturned;
    //qDebug() << "Corect black = " << correctBlack ;
    //qDebug() << "Error black = " << errorsBlack ;

    double precision = 0.0;
    double recall = 0.0;
    if(foundBlack > 0)
        precision = correctBlack / foundBlack;
    if (shouldHaveBeenReturned > 0)
        recall = correctBlack / shouldHaveBeenReturned;

    //qDebug() << "Precision : " << precision;
    //qDebug() << "Recall : " << recall;

    if(precision == 0 && recall == 0)
        return 0.0;
    else if (precision == recall) {
        fscore = 1.0;
    } else {
        fscore = 2.0 * ((precision * recall) / (precision + recall));
    }

    //return (errorsBlack * 100) / shouldHaveBeenReturned;
   // return (mismatch) / same;
    return fscore;
    //return precision;
   // return recall;
}

DQB_EXPORT_PLUGIN(FScoreImage)
