#include "SampleLayoutExtractor.hpp"


#include <QtPlugin>


#include <tesseract/capi.h>

#include <ise/iseSegmentationContext.hpp>
#include <ise/iseSegmentationClustering.hpp>
#include <ise/iseSplitAlgorithm.hpp>
#include <ise/iseSplitDriverDefault.hpp>
#include <ise/iseSplitAlgorithmDefault.hpp>
#include <ise/iseQuantificationThresholderOtsu2T.hpp>
#include <ise/iseHistogram1D.hpp>

#include <ise/iseSegmentationClusteringDocumentPreseg4.hpp>


#include <ise/iseParagraph.hpp>


#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsRegionInfos.hpp>

#include <DoQuBook/DoQuBookOpenCVConverter.hpp>

using namespace DoQuBook;


class RegionColorizer : public girl::toger::pointIterator
{
public:
  RegionColorizer(QImage &img, QRgb v)
    : image(img)
    , value(v)
  {}

  virtual void processPoint(const girl::point &pt)
  {
      image.setPixel(pt.x(), pt.y(), value);
  }

protected:

  QImage &image;
  QRgb value;

};


cv::RNG rng(12345);

girl::region
getBiggerRegion(const girl::toger &tog, const girl::regionDeque &regions)
{
    girl::region max = regions.at(0);
    size_t airmax = girl::tools::computeArea(tog,max);

    foreach (girl::region r, regions) {
        size_t regionArea = girl::tools::computeArea (tog, r);
        if (regionArea > airmax) {
            max = r;
            airmax = regionArea;
        }
    }

    return max;
}


DoQuBook::LayoutPtr SampleLayoutExtractor::extract(const QImage &qimage, const QVariantMap &/*params*/)
{

  DoQuBookOpenCVConverter converteur;
  cv::Mat src = converteur.getCvMat(qimage);

  girl::tools::Image image (src);

  girl::toger tog(image.width(), image.height());


  ise::SegmentationContext segContext;
  segContext.setToger(&tog);
  segContext.setImage(&image);

  girl::regionDeque toBeSplittedRegions;
  /// On choisis la premiere region à spliter
  toBeSplittedRegions.push_back(1);

  ise::SplitAlgorithmDefault splitAlgorithm(&segContext);
  ise::SplitDriverDefault defaultDriver(&segContext);

  girl::regionDeque createdRegions;
  ise::SegmentationClusteringDocumentPreseg4 segCluster(&segContext);
  //ise::SegmentationClusteringLocalThresholds segCluster(&segContext);
  //ise::SegmentationClusteringLocalThresholderSauvola *sauvola = new ise::SegmentationClusteringLocalThresholderSauvola;

  //sauvola->setParameters(5, 98 , 0.1 );
  //sauvola->setParameters(201, 98 , 0.1 ); Change rien
  //sauvola->setParameters(21, 128 , 0.1 );

  //girl::shared_ptr<ise::SegmentationClusteringLocalThresholder> thresholder (sauvola);
  //segCluster.setParams(thresholder);

  splitAlgorithm.apply(toBeSplittedRegions, defaultDriver, segCluster , 0, createdRegions);

  QImage img (image.width(), image.height(), QImage::Format_ARGB32);

  RegionColorizer blackColorizer (img, qRgb(0, 0, 0));
  RegionColorizer whiteColorizer (img, qRgb(255, 255, 255));

  girl::region back = getBiggerRegion(tog, createdRegions);

  foreach (const girl::region r, createdRegions) {
      girl::regionDeque neighbours;
      tog.regionNeighbourhood(r, girl::toger::NEIGHBOUR_ANY, girl::toger::MARK_ANY, neighbours);
      bool hasBackGround = false;

      unsigned int i = 0;
      while(!hasBackGround && i < neighbours.size())
      {
          hasBackGround = (neighbours[i] == back);
          i++;
      }
      if(r != back && hasBackGround) {
          tog.iterateOnRegionDomain(r, whiteColorizer);
      } else {
          tog.iterateOnRegionDomain(r, blackColorizer);
      }
  }

  img.save("layoutfirst.png");


    LayoutPtr l(new Layout);
    l->setNb(25);
    return l;
    /*Extraction Layout*/
}


DQB_EXPORT_PLUGIN(SampleLayoutExtractor)
