#include "TestSampleLayoutExtractor.hpp"
#include"SampleLayoutExtractor.hpp"

#include "Config.hpp"

#include <leptonica/environ.h>

#include <leptonica/pix.h>

#include <tesseract/baseapi.h>
#include <leptonica/allheaders.h>

#include <QPainter>

using namespace tesseract;

void TestSampleLayoutExtractor::testPlugin()
{
  SampleLayoutExtractor sle;
  QImage im (DATA_DIR+"/img0.png");
  DoQuBook::LayoutPtr l=sle.extract(im);
  QVERIFY(l->getNb()==27);
}

void TestSampleLayoutExtractor::testTesseract()
{

  //QString fileName (DATA_DIR+"/testTesseract0.png");
  QString fileName (DATA_DIR+"/img0.png");
  QImage img(fileName);
  img.save("./temp.jpg");
  QString jpegFileName ("./temp.jpg");

  setlocale (LC_ALL, "C"); // To fix problem with lib Tesseract upto 3.0

  QByteArray dataDir ("/usr/local/share/tessdata/");
  QByteArray language = "eng";
  //QByteArray language = "fra";


  PIX  *pix = pixRead(jpegFileName.toUtf8());

  QVERIFY(pix != NULL);

  TessBaseAPI tess;
  int initialized = tess.Init(dataDir.constData(), language.constData());
  tess.SetPageSegMode(PSM_AUTO);
  tess.SetImage(pix);
  tess.Recognize(0L);



  tesseract::ResultIterator* pageIt = tess.GetIterator();

  /*
   *enum PolyBlockType {
  PT_UNKNOWN,        // Type is not yet known. Keep as the first element.
  PT_FLOWING_TEXT,   // Text that lives inside a column.
  PT_HEADING_TEXT,   // Text that spans more than one column.
  PT_PULLOUT_TEXT,   // Text that is in a cross-column pull-out region.
  PT_EQUATION,       // Partition belonging to an equation region.
  PT_INLINE_EQUATION,  // Partition has inline equation.
  PT_TABLE,          // Partition belonging to a table region.
  PT_VERTICAL_TEXT,  // Text-line runs vertically.
  PT_CAPTION_TEXT,   // Text that belongs to an image.
  PT_FLOWING_IMAGE,  // Image that lives inside a column.
  PT_HEADING_IMAGE,  // Image that spans more than one column.
  PT_PULLOUT_IMAGE,  // Image that is in a cross-column pull-out region.
  PT_HORZ_LINE,      // Horizontal Line.
  PT_VERT_LINE,      // Vertical Line.
  PT_NOISE,          // Lies outside of any column.
  PT_COUNT
};
   */

  QImage blocks = img.convertToFormat(QImage::Format_ARGB32_Premultiplied);
   do {
    int left, right, bottom, top;

        pageIt->BoundingBox(RIL_BLOCK, 0, &left, &top, &right, &bottom);

        QRgb color;
        qDebug() << pageIt->BlockType();
        switch (pageIt->BlockType()) {
          case PT_HEADING_TEXT:
              color = qRgb(0,255,0);
            break;
          case PT_FLOWING_TEXT:
              color = qRgb(0,0,255);
            break;
          case PT_HORZ_LINE:
              color = qRgb(0,255,255);
            break;
          case PT_UNKNOWN:
              color = qRgb(0,0,0);
            break;
          case PT_PULLOUT_TEXT:
              color = qRgb(255,255,0);
            break;
          case PT_VERTICAL_TEXT:
              color = qRgb(0,100,0);
            break;
          case PT_FLOWING_IMAGE:
              color = qRgb(100,0,0);
            break;
          default:
              color = qRgb(255,0,0);
            break;
        }
        for(int i = left; i <= right; i++) {
          for(int j = top; j <= bottom; j++) {
            blocks.setPixel(i, j, color);
          }
        }

    } while (pageIt->Next(RIL_BLOCK));

  blocks .save("blocks.png");



  pageIt = tess.GetIterator();
  QImage words = img.convertToFormat(QImage::Format_ARGB32_Premultiplied);
   do {
    int left, right, bottom, top;

        pageIt->BoundingBox(RIL_WORD, 0, &left, &top, &right, &bottom);

        for(int i = left; i <= right; i++) {
          for(int j = top; j <= bottom; j++) {
            words .setPixel(i, j, qRgb(255, 0, 0));
          }
        }
        words .setPixel(left, top, qRgb(255, 0, 0));
        words .setPixel(right, bottom, qRgb(255, 0, 0));

        qDebug() << pageIt->GetUTF8Text(RIL_WORD);

    } while (pageIt->Next(RIL_WORD));

  words.save("words.png");




  QImage paras = img.convertToFormat(QImage::Format_ARGB32_Premultiplied);
  pageIt = tess.GetIterator();
   do {
    int left, right, bottom, top;

        pageIt->BoundingBox(RIL_PARA, 0, &left, &top, &right, &bottom);

        for(int i = left; i <= right; i++) {
          for(int j = top; j <= bottom; j++) {
            paras .setPixel(i, j, qRgb(255, 0, 0));
          }
        }
        paras .setPixel(left, top, qRgb(255, 0, 0));
        paras .setPixel(right, bottom, qRgb(255, 0, 0));

        qDebug() << pageIt->GetUTF8Text(RIL_PARA);


    } while (pageIt->Next(RIL_PARA));

    paras.save("para.png");



    QImage symbols = img.convertToFormat(QImage::Format_ARGB32_Premultiplied);
    pageIt = tess.GetIterator();
     do {
      int left, right, bottom, top;

          pageIt->BoundingBox(RIL_SYMBOL, 0, &left, &top, &right, &bottom);

          for(int i = left; i <= right; i++) {
            for(int j = top; j <= bottom; j++) {
              symbols .setPixel(i, j, qRgb(255, 0, 0));
            }
          }
          symbols .setPixel(left, top, qRgb(255, 0, 0));
          symbols .setPixel(right, bottom, qRgb(255, 0, 0));


      } while (pageIt->Next(RIL_SYMBOL));

      symbols.save("symbols.png");

}


void TestSampleLayoutExtractor::init()
{
}


void TestSampleLayoutExtractor::cleanup()
{
}



QTEST_MAIN(TestSampleLayoutExtractor)
