#include "RegionExtractorISEWhiteCC.hpp"

#include <QtPlugin>
#if QT_VERSION >= 0x050000
#include <QtConcurrent/QtConcurrent>
#else
#include <QtConcurrentMap>
#endif
#include <QTime>


#include <cv.h>
#include <highgui.h>
#include <iostream>
#include <DoQuBook/DoQuBookOpenCVConverter.hpp>
#include <DoQuBook/Logger.hpp>

#include <ise/iseSegmentationContext.hpp>
#include <ise/iseSegmentationClustering.hpp>
#include <ise/iseSplitAlgorithm.hpp>
#include <ise/iseSplitDriverDefault.hpp>
#include <ise/iseSplitAlgorithmDefault.hpp>

#include <ise/iseHistogram1D.hpp>

#include <girl/girlCommon.hpp>
#include <girlTools/girlToolsRegionInfos.hpp>

#include "SegmentationClusteringSameColorPixel.hpp"

using namespace DoQuBook;



RegionExtractorISEWhiteCC::RegionExtractorISEWhiteCC()
    : _toger (NULL)
    , _image (NULL)
{
}
RegionExtractorISEWhiteCC::~RegionExtractorISEWhiteCC()
{
    deletePointers();
}


/*class RegionFaceConverter : public girl::toger::directionIterator
{
public:
    RegionFaceConverter()
        : m_r(NULL)
    {}

    virtual void processFirstPoint(const girl::point &p) {
        lastPoint = p;
    }

    virtual void processDirection(girl::direction d)
    {
        lastPoint.move(d);
        addPoint(lastPoint);
    }

    virtual void processLastPoint(const girl::point &) {
    }

    virtual void addPoint(const girl::point &pt)
    {
        QPointF p (pt.x(), pt.y());
        m_r->addPointToContour(p);
    }

    DoQuBook::Region createdRegion()
    {
        return *m_r;
    }

    void setRegion (DoQuBook::Region *region)
    {
        m_r = region;
    }

protected:

    DoQuBook::Region *m_r;
    girl::point lastPoint;
};*/

class RegionDomainConverter : public girl::toger::pointIterator
{
public:
    RegionDomainConverter(DoQuBook::RegionPointProcessorPtr &processor)
        : _processor (processor)
    {}

    virtual void processPoint(const girl::point &pt)
    {
        _processor->processPoint(pt.x(), pt.y());
    }

protected:

    DoQuBook::RegionPointProcessorPtr &_processor;
};


struct processRegions
{

    struct Parameter {
        Parameter (girl::toger &t,
                   girl::tools::Image &i,
                   girl::regionDeque &regions,
                   QColor &rgb)
            : tog(t), image (i), createdRegions (regions), color (rgb)
        {}

        girl::toger &tog;
        girl::tools::Image &image;
        girl::regionDeque &createdRegions;
        QColor &color;
    };

    processRegions(Parameter &params)
        : _params (params)
    {}

    void operator()(RegionPointProcessorPtr & processor)
    {
        RegionDomainConverter converter (processor);

        foreach (const girl::region r, _params.createdRegions) {

            girl::point p = _params.tog.anyPoint(r);
            // Its ink ?
            girl::tools::PixelFromImage<girl::tools::Image::type> pixel = _params.image.getPixel(p.x(), p.y());
            if( pixel[0] == _params.color.red() && pixel[1] == _params.color.green() && pixel[2] == _params.color.blue()) {
                girl::point p1; girl::point p2;
                girl::tools::computeBoundingBox(_params.tog, r, p1, p2);

                double a = girl::tools::computeArea(_params.tog, r);
                processor->area(a);
                double p = girl::tools::computePerimeter(_params.tog, r);
                processor->perimeter(p);
                processor->boundingBoxOfRegion(p1.x(), p1.y(), p2.x(), p2.y());
                processor->newRegion();

                _params.tog.iterateOnRegionDomain(r, converter);

                processor->endRegion();

            }
        }
    }

    Parameter &_params;
};


void RegionExtractorISEWhiteCC::regions (const QImage &qimage)
{
    
    deletePointers();
    QTime t = QTime::currentTime();
    {

        DoQuBookOpenCVConverter converteur;
        t.start();
        cv::Mat src = converteur.getCvMat(qimage);

        _image = new girl::tools::Image (src);
    }
    t.restart();
    applySplit();
}




void RegionExtractorISEWhiteCC::applySplit()
{
    _toger = new girl::toger (_image->width(), _image->height());

    ise::SegmentationContext segContext;
    segContext.setToger(_toger);

    segContext.setImage(_image);

    girl::regionDeque toBeSplittedRegions;
    // On choisis la premiere region à spliter
    toBeSplittedRegions.push_back(1);

    ise::SplitAlgorithmDefault splitAlgorithm(&segContext);
    ise::SplitDriverDefault defaultDriver(&segContext);


    SegmentationClusteringSameColorPixel otsu(&segContext);

    _createdRegions.clear();
    splitAlgorithm.apply(toBeSplittedRegions, defaultDriver, otsu , 0, _createdRegions);
}

void RegionExtractorISEWhiteCC::runProcesors(QList<RegionPointProcessorPtr> &processors, const QVariantMap &parameters)
{
    if(parameters.contains("color"))
        _color = parameters["color"].value<QColor>();
    else
        _color = Qt::white;

    processRegions::Parameter parameter
            (*_toger, *_image, _createdRegions, _color);

    QTime t = QTime::currentTime();
    t.start();

    bool concurrent = true;
    if (parameters.contains("concurrent"))
        concurrent = parameters["concurrent"].toBool();

    if(concurrent) {
        QFuture<void> preps = QtConcurrent::map(processors, processRegions(parameter));
        preps.waitForFinished();
    } else {
        foreach (RegionPointProcessorPtr proc, processors) {
            processRegions process (parameter);
            process (proc);
        }
    }
}

void RegionExtractorISEWhiteCC::deletePointers()
{
    if(_image) {
        delete _image;
        _image = NULL;
    }
    if(_toger) {
        delete _toger;
        _toger = NULL;
    }
}

DQB_EXPORT_PLUGIN(RegionExtractorISEWhiteCC)



