#include <DoQuBook/RegionExtractor.hpp>
#include <PluginConfig.hpp>
#include <QColor>

#include <girlTools/girlToolsImage.hpp>
#include <girl/girlToger.hpp>

namespace DoQuBook {
    class Logger;
}

/**
  * \mainpage
  * Process a set of RegionPointProcessorPtr using GIRL library.
  * The segmentation (RegionExtractorISEWhiteCC#regions) is a bit slow but the processing of RegionPointProcessorPtr is
  * Really fast.
  *
  * The function RegionExtractorISEWhiteCC#runProcesors Takes 2 parameters :
  *   - color = Rgb color as type QColor default is white
  *   - concurrent = Bool. true = every Processor is ran on a thread, false = every Processor is ran sequentially
  *
  */
class RegionExtractorISEWhiteCC : public RegionExtractor
{
    Q_OBJECT
    DQB_PLUGIN(RegionExtractor, "fr.labri.doqubook.RegionExtractor")

    RegionExtractorISEWhiteCC();
    ~RegionExtractorISEWhiteCC();

    virtual void regions (const QImage &image);

    /**
      * Takes 2 parameters :
      *   - color = Rgb color as type QColor default is white
      *   - concurrent = Bool. true = every Processor is ran on a thread, false = every Processor is ran sequentially
      */
    void runProcesors (QList<DoQuBook::RegionPointProcessorPtr> &processors, const QVariantMap &parameters = QVariantMap());

protected:
    void applySplit ();
protected:
    DoQuBook::Logger *logger;

private :
    void deletePointers();

private :
    QColor _color;
    girl::toger *_toger;
    girl::tools::Image *_image;
    girl::regionDeque _createdRegions;
};
