#include "RegionExtractorOpenCVForeground.hpp"
#include <QDebug>
#include <QtPlugin>
#include <QtConcurrent/QtConcurrent>
#include <QTime>

#include <iostream>
#include <DoQuBook/DoQuBookOpenCVConverter.hpp>

#include <highgui.h>

using namespace DoQuBook;
using namespace cv;

void RegionExtractorOpenCVForeground::regions(const QImage &qimage)
{
  DoQuBookOpenCVConverter converteur;
  {
    // Add a black border for cv::findContours
    // Other wise wite components on the border are not considered as components ?
    QImage add1pxBorder (qimage.width()+2, qimage.height()+2, qimage.format());

    QRgb black = qRgb(0,0,0);
    for(int i = 0; i < add1pxBorder.width(); i++) {
      for(int j = 0; j < add1pxBorder.height(); j++) {
        if(i == 0 || i == add1pxBorder.width()-1
           || j == 0 || j == add1pxBorder.height()-1)
          add1pxBorder.setPixel(i,j, black);
        else
          add1pxBorder.setPixel(i,j, qimage.pixel(i-1,j-1));
      }
    }

    src = converteur.getCvMat(add1pxBorder);
  }

  bin = cv::Mat::zeros(src.rows, src.cols, CV_8UC1);
  cv::cvtColor( src, bin, CV_BGR2GRAY );
  cv::threshold( bin, bin, 0, 255, CV_THRESH_OTSU );

  cv::findContours(bin, list, hierachy,
                   CV_RETR_TREE, CV_CHAIN_APPROX_NONE);
}



struct processRegions
{

    struct Parameter {
        Parameter (RegionExtractorOpenCVForeground::ShapeList &l,
                   RegionExtractorOpenCVForeground::Hierarchy &h,
                   cv::Mat &source,
                   cv::Mat &pbin,
                   QColor &rgb)
          : list (l), hierachy (h),
            src (source), bin(pbin), color (rgb)
        {}

        RegionExtractorOpenCVForeground::ShapeList &list;
        RegionExtractorOpenCVForeground::Hierarchy &hierachy;
        cv::Mat &src;
        cv::Mat &bin;
        QColor &color;
    };

    processRegions(Parameter &params)
      : _params (params)
    {}

    void operator()(RegionPointProcessorPtr & processor)
    {

      for(int idx = 0; idx < _params.list.size(); idx++ )
      {
        // All except hoes ?
        if(_params.hierachy[idx][3] == -1) {

          processor->newRegion();
          // We use Qt's polygon for checking if a point is within the shape.
          QPolygon polygon;
          foreach (const cv::Point &p, _params.list[idx]) {
            polygon.append(QPoint(p.x, p.y));
          }
          // iterate over the all BBox and consider every point within the polygon.
          QRect bRect = polygon.boundingRect();
          for(int y=bRect.top(); y <= bRect.bottom(); y++) {
            for(int x=bRect.left(); x <= bRect.right(); x++) {
              cv::Vec3b pixel = _params.src.at<cv::Vec3b>(y, x);
              QPolygon sec;
              sec << QPoint(x,y);
              // Should be equal to params["color"] nope ?
              // contains (for the border) coz some border points not in polygon with containsPoint
              if((polygon.contains(QPoint(x,y)) || polygon.containsPoint(QPoint(x,y), Qt::OddEvenFill))
                 && (pixel[0] != 0 || pixel[1] != 0 || pixel[2] != 0 )) {
                // Shifted by -1 because of the added black border
                processor->processPoint(x-1, y-1);
              }
            }
          }
          processor->endRegion();
        }
      }
    }
    Parameter &_params;
};

void RegionExtractorOpenCVForeground::runProcesors(QList<DoQuBook::RegionPointProcessorPtr> &processors, const QVariantMap &parameters)
{
  if(parameters.contains("color"))
    _color = parameters["color"].value<QColor>();
  else
    _color = Qt::white;

  processRegions::Parameter parameter
      (list, hierachy, src, bin, _color);

  QTime t = QTime::currentTime();
  t.start();

  bool concurrent = true;
  if (parameters.contains("concurrent"))
    concurrent = parameters["concurrent"].toBool();

  if(concurrent) {
    QFuture<void> preps = QtConcurrent::map(processors, processRegions(parameter));
    preps.waitForFinished();
  } else {
    foreach (RegionPointProcessorPtr proc, processors) {
      processRegions process (parameter);
      process (proc);
    }
  }

}

DQB_EXPORT_PLUGIN(RegionExtractorOpenCVForeground)
