#include <DoQuBook/PluginLoader.hpp>
#include <DoQuBook/DoQuBookApp.hpp>
#include <DoQuBook/ApplicationContext.hpp>
#include <DoQuBook/RegionExtractor.hpp>

#include <DoQuBook/Region.hpp>

#include <QColor>

using namespace DoQuBook;

QString orgName ("your_organization");
/// Must be the name of the target in CMake
QString appName ("SampleRegionExtractor");

class SampleRegionExtractor : public DoQuBook::DoQuBookApp
{
public :
    SampleRegionExtractor(QCoreApplication &app)
        : DoQuBook::DoQuBookApp(app, orgName, appName)

    {}

    QList<ProgramParameter> paramsAndDescriptions()
    {
        QList<ProgramParameter> params;
        //ProgramParameter param1 ("param1Name", false, "Param1 Description");
        //params.append(param1);
        params << DoQuBook::DoQuBookApp::paramsAndDescriptions();
        return params;
    }

protected :
    QImage damier() {
      QImage img(100, 100, QImage::Format_RGB888);
      bool lastWhite = true;
      for(int i = 0; i< img.width(); i++) {
         for(int j = 0; j< img.height(); j++) {
          if(lastWhite)
            img.setPixel(i,j, qRgb(0, 0, 0));
          else
           img.setPixel(i,j, qRgb(255, 255, 255));

          lastWhite = !lastWhite;
          //qDebug() << i << "  " << j << "  " << lastWhite;
         }
         lastWhite = !lastWhite;
      }
      return img;
    }

    virtual void runApp (QMap<QString, QString> /*args*/)
    {
        // Get the needed services
        DoQuBook::RepositoryPtr repo = DoQuBook::ApplicationContext::instance()->getDefaultRepository();
        PageCollectionServicePtr pageCollectionSrv = repo->getPageCollectionService();
        PageServicePtr pageSrv = repo->getPageService();
        
        // Get the plugin
        PluginLoader *loader = PluginLoader::instance();
        qDebug() << loader->listPlugins<Binarizer>();
        qDebug() << loader->listPlugins<RegionExtractor>();
        //BinarizerPtr binarizer = loader->getPlugin<Binarizer>("labri.OtsuBinarization.0.1.1");
        BinarizerPtr binarizer = loader->getPlugin<Binarizer>("labri.BinarizationOtsu2T.0.1.1");

        RegionExtractorPtr regionExtractor = loader->getPlugin<RegionExtractor>("labri.RegionExtractorOpenCVForegroundPlugin.0.1.1");
        //RegionExtractorPtr regionExtractor = loader->getPlugin<RegionExtractor>("labri.RegionExtractorISEWhiteCCPlugin.0.1.1");

        PagePtr p = pageSrv->get(1);
        QImage img = p->getImage();
        //QImage img = damier();
        QImage bin = binarizer->binarize(img);
        regionExtractor->regions(bin);


        img.save("LOLDAMIER.png");
        qDebug() << "Downloaded image and binarized";
        sleep(5);


        QSharedPointer<RegionPainter> painter (new RegionPainter(img));
        QList<RegionPointProcessorPtr> processors1;
        processors1 << painter;
        QVariantMap params;
        params["color"] = QColor(255, 255, 255);
        //params["concurrent"] = false;
        QTime t1; t1.start();
        regionExtractor->runProcesors(processors1, params);
        qDebug() << t1.elapsed();
        img.save("redBinarizedWithProcessors.png");

        qDebug() << "Raw region painter";
        sleep(5);

        // extract regions
        QSharedPointer<RegionsExtractor> scan (new RegionsExtractor);
        QList<RegionPointProcessorPtr> processors2;
        processors2 << scan;
        regionExtractor->runProcesors(processors2, params);


        QList<RegionPtr> scanLines = scan->getRegions();
        painter->setColor(qRgb(255,255,0));

        QTime t2; t2.start();
        foreach (const RegionPtr &r, scanLines) {
            r->iterateOnDomain(painter);
        }
        qDebug() << t2.elapsed();
        img.save("redBinarizedWithScanLines.png");

        qDebug() << "Unparallel scan lin region painter";
        sleep(5);


        painter->setColor(qRgb(0,0,255));

        MappedRegionsProcessor regionsProcessor (scanLines);
        QTime t3; t3.start();
        regionsProcessor.runProcessors(painter);
        qDebug() << t3.elapsed();
        img.save("redBinarizedMapped.png");

        qDebug() << "Parrallel scan lin region painter";
        sleep(5);



        MappedRegionsProcessors regionsProcessors (scanLines);
        QTime t4; t4.start();
        QSharedPointer<RegionPainter> painter1 (new RegionPainter(img));
        QSharedPointer<RegionPainter> painter2 (new RegionPainter(img));
        QSharedPointer<RegionPainter> painter3 (new RegionPainter(img));
        QSharedPointer<RegionPainter> painter4 (new RegionPainter(img));
        QList<RegionPointProcessorPtr> processors;
        processors << painter1 << painter2 << painter3 << painter4;
        regionsProcessors.runProcessors(processors);
        qDebug() << "Time for MappedRegionsProcessors with 4 painters :" << t4.elapsed();


        QTime t5; t5.start();
        foreach (const RegionPointProcessorPtr &proc, processors) {
          foreach (const RegionPtr &r, scanLines) {
              r->iterateOnDomain(proc);
          }
        }
        qDebug() << "Time for norman O(2n) with 4 painters : " <<  t5.elapsed();


        QTime t6; t6.start();
        regionExtractor->runProcesors(processors, params);
        qDebug() << "Time for raw with 4 painters : " <<  t6.elapsed();
        
/*        meta_iterator<PagePtr>::IteratorPtr it = pageCollectionSrv->getPages("(H)DIBCO");
        while(it->hasNext()) {
            try{
                PagePtr page = it->next();
                QImage pageImage = page->getImage();
                QImage bin = binarizer->binarize(pageImage);
                bin.save(page->getName()+"_bin.png");
            }catch (MessageException e) {
                qCritical() << tr("Error while binarizing a page") << e.message();
            }
        }*/
    }
};


int main(int argc, char *argv[])
{
    QCoreApplication app(argc, argv);
    SampleRegionExtractor myApp (app);
    return myApp.execConsoleApp();
}
