# Find QJSON - JSON handling library for Qt
#
# This module defines
#  QJSON_FOUND - whether the qsjon library was found
#  QJSON_LIBRARIES - the qjson library
#  QJSON_INCLUDE_DIR - the include path of the qjson library
#

if (QJSON_INCLUDE_DIR AND QJSON_LIBRARIES)

  # Already in cache
  set (QJSON_FOUND TRUE)

else (QJSON_INCLUDE_DIR AND QJSON_LIBRARIES)

  if (NOT WIN32)
    # use pkg-config to get the values of QJSON_INCLUDE_DIRS
    # and QJSON_LIBRARY_DIRS to add as hints to the find commands.
    include (FindPkgConfig)
    pkg_check_modules (PC_QJSON QJson>=0.5)
  endif (NOT WIN32)
  
  if(WIN32)
    SET(qjson_POSSIBLE_ROOT_DIRS
        "$ENV{QJSON_ROOT_DIR}"
        "$ENV{SystemDrive}/qjson"                    
        "$ENV{SystemDrive}/Program Files/qjson"       
        "$ENV{SystemDrive}/Program Files (x86)/qjson"
        )
	FIND_PATH(qjson_ROOT_DIR
          NAMES include/qjson/parser.h
          PATHS ${qjson_POSSIBLE_ROOT_DIRS}
          )
	SET(qjson_LIB_SEARCH_PATH "${qjson_ROOT_DIR}/bin")
	SET(qjson_INCLUDE_SEARCH_PATH "${qjson_ROOT_DIR}/include")
  endif(WIN32)

  find_library (QJSON_LIBRARIES
    NAMES
    qjson
    PATHS
    ${PC_QJSON_LIBRARY_DIRS}
    ${LIB_INSTALL_DIR}
	${qjson_LIB_SEARCH_PATH}
    ${KDE4_LIB_DIR}
  )
	message(${QJSON_LIBRARIES})

  find_path (QJSON_INCLUDE_DIR
    NAMES
    qjson/parser.h
    PATHS
    ${PC_QJSON_INCLUDE_DIRS}
    ${INCLUDE_INSTALL_DIR}
	${qjson_INCLUDE_SEARCH_PATH}
    ${KDE4_INCLUDE_DIR}
  )
  message(${QJSON_INCLUDE_DIR})

  include(FindPackageHandleStandardArgs)
  find_package_handle_standard_args(QJSON DEFAULT_MSG QJSON_LIBRARIES QJSON_INCLUDE_DIR)

endif (QJSON_INCLUDE_DIR AND QJSON_LIBRARIES)
