#ifndef CLASSBUILDER_HPP
#define CLASSBUILDER_HPP
#include <QList>
#include <QFile>
#include <QTextStream>

#include "GenericBuilder.hpp"
#include "Entity.hpp"

/**
  * Writes a xml entity to a class in different languages :
  *  cpp \see CppClassBuilder.
  *  java \see JavaClassBuilder.
  *  csharp \see CSharpClassBuilder.
  * \code
  * <entity name="Personne">
        <field type="int" name="age" id="true" />
        <list type="Page" name="pages" marshallable="true"/>
        <list type="String" name="prenoms" marshallable="false" />
        <field type="int" name="version" />
    </entity>
   \endcode
  *
  */
class ClassBuilder : public GenericBuilder
{
public:
    ClassBuilder(Entity &e, QString, QString, QString = "");

protected :

    /// Returns the field type to String. (example list of Page => java = List<Page>, cpp QList<PagePtr>)
    virtual QString getFieldType(const Field&f);
    /// Returns the field annotations for marshalling or hibernate
    virtual QString getFieldAnnotations(const Field&f);

    /// Returns the declaration of a field (public int id;) and its annotations (@Id)
    QString fieldDeclaration(const Field &f);
    QString fieldDeclarationGettersAndSetters(const Field &f);
    QString fieldImplementationGettersAndSetters(const Field &f);

    /// Writes the class implemantation (.cpp for example)
    virtual void writeImplementationFile();
    /// Writes the class declaration (.hpp for example)
    virtual void writeDeclarationFile();

    void setInheritence(QString & templ);

    /// permit to setup inheritence schemes
    virtual QString getInheritKeyword();
    virtual QString getInheritParent();
    virtual QString getInheritDefaultKeyword();
    virtual QString getInheritDefaultParent();

    QString declarationFileSuffix();
    QString implementationFileSuffix();


    // Templates Methods
    virtual QString getDeclarationFileTemplate();
    virtual QString getImplementationFileTemplate();

    QString getterTemplate();
    QString getAnnotationsTemplate();

    QString getInitArrayListTemplate();
    QString getFieldDeclarationTemplate();
    QString getBasicFieldAnnotation();
    QString getIdFieldAnnotation();
    QString getIntIdFieldAnnotation();
    QString getListFieldAnnotations();

    QString getGetterTemplate();


    QString getGetterDeclarationTemplate();

    QString getGetterImplementationTemplate();

    QString getSetterTemplate();

    QString getSetterDeclarationTemplate();

    QString getSetterImplementationTemplate();

    QString getConstructorTemplate();
    QString getConstructorImplementationTemplate();
    QString getConstructorDeclarationTemplate();
    QString getImportTemplate(); // path th the import template

    virtual QString imports();

};

#endif // CLASSBUILDER_HPP
