#include "CppClassBuilder.hpp"

CppClassBuilder::CppClassBuilder(Entity &e)
    : ClassBuilder(e, "cpp", ".hpp", ".cpp")
{
    //_pathFileImports = ":/" + lang() + "/imports";
}

QString CppClassBuilder::imports()
{
    QString t;
    t.append("#include <DoQuBook/DoQuBook_global.hpp> \n");
    t.append("#include <DoQuBook/SimpleMarshallable.hpp> \n");
    t.append("#include <DoQuBook/Exceptions.hpp> \n");
    t.append("#include <QString> \n");
    t.append("#include <QList> \n");
    t.append("#include <QObject> \n");
    t.append("#include <QSharedPointer> \n");

    foreach(Field f, entity().fields() ) {
        if (f.isMarshallable())
        {
            QString marshallableImport = "#include <DoQuBook/${TYPE}.hpp> \n";
            t.append(marshallableImport.replace("${TYPE}", f.type()));
        }
    }

    return t;
}

QString CppClassBuilder::getFieldType(const Field &f)
{
    if(f.isList() && f.isMarshallable())
        return "QList<"+f.type()+"Ptr>";
    else if(f.isList())
        return "QList<"+f.type()+">";

    else if (f.isMap() && f.isMarshallable())
        return "QMap<"+f.key()+","+f.type()+"Ptr>";
    else if (f.isMap() )
        return "QMap<"+f.key()+","+f.type()+">";

    else
        return f.type();
}

QString CppClassBuilder::getFieldAnnotations(const Field &f)
{
    QString s = ClassBuilder::getFieldAnnotations(f);
    if(f.isMarshallable())
        s.replace("${LIST_MACRO}", "MLIST");
    else
        s.replace("${LIST_MACRO}", "QTLIST");

    return s;
}

void CppClassBuilder::moreTemplatesOnDeclarationFile(QString &templ)
{
    QString activateList = "MARSHALLABLE_ACTIV_${N}LIST(";
    int nb = 0;
    bool firstList = true;
    QList<Field> _fields = entity().fields();
    for(int i = 0; i < _fields.size(); i++) {
        if(_fields[i].isList()) {
            if(firstList) {
                activateList.append(_fields[i].name()+", _"+_fields[i].name());
                firstList = false;
            } else
                activateList.append(","+_fields[i].name()+", _"+_fields[i].name());
            nb++;
        }
    }

    activateList.replace("${N}", QString::number(nb));
    activateList.append(");");
    templ.replace("${ACTIVATE_LIST}", activateList);
}

void CppClassBuilder::moreTemplatesOnImplementationFile(QString &templ)
{
}

QString CppClassBuilder::getInheritKeyword()
{
    return ": public";
}

QString CppClassBuilder::getInheritDefaultKeyword()
{
    return ": public";
}

QString CppClassBuilder::getInheritDefaultParent()
{
    return "SimpleMarshallable";
}
