#ifndef ENTITY_HPP
#define ENTITY_HPP

#include <QString>
#include <QList>

class Field {
public:
    Field (const QString &type, QString &name);
    Field (const QString &key, const QString &type, QString &name);

    QString name() const;
    QString type() const;
    QString key() const;

    bool isId() const;
    void setId(bool id);

    void setIsList(bool isList);
    bool isList() const;

    void setIsMap(bool isMap);
    bool isMap() const;

    void setIsMarshallable (bool isMarshallable);
    bool isMarshallable () const;

private :
    QString _type;
    QString _name;
    QString _key;
    bool _id;
    bool _isList;
    bool _isMap;
    bool _isMarshallable;
};

class CRUDMethods
{
public:
    CRUDMethods (bool create, bool read, bool update, bool del);


    bool create();
    bool read();
    bool update();
    bool del();

private :
    bool _create;
    bool _read;
    bool _update;
    bool _del;
};

class Method {
public :
    Method (const QString &name);

    void addParam (const Field &f);

    QList<Field> getParams();

private :
    QString type;
    QString _name;
    QList<Field> _params;
};


class Entity
{
public:
    Entity();

    void setName (const QString &name);
    QString name();

    void extends(const QString &parentName);
    QString parent();

    void addField(const Field &f);
    void setFields(const QList<Field> &fields);
    QList<Field> fields();

private :
    QString _name;
    QString _parent;
    QList<Field> _fields;
};

#endif // ENTITY_HPP
