#include "GenericBuilder.hpp"

#include <QDebug>
#include <QDir>

GenericBuilder::GenericBuilder(Entity &e, QString lang, QString declarationExtention, QString implementationExtention)
    : _entity (e), _bHasImplementationFile(true)

{
    setLangInfo(lang, declarationExtention, implementationExtention);
}

Entity GenericBuilder::entity()
{
    return _entity;
}

QString GenericBuilder::getNamespace()
{
    // @TODO <entity name="Personne" namespace="NamespaceName">
    return "DoQuBook";
}

QString GenericBuilder::classNameUpper()
{
    return _entity.name().toUpper();
}

QString GenericBuilder::classNameLower()
{
    return _entity.name().toLower();
}

void GenericBuilder::setLangInfo(QString lang, QString declarationExtention, QString implementationExtention)
{
    _lgg_name       = lang;
    _lgg_ext_decl   = declarationExtention;
    _lgg_ext_impl   = implementationExtention;

    if( implementationExtention.isEmpty() )
        _bHasImplementationFile = false;
}

QString GenericBuilder::lang()
{
        return _lgg_name;
}

QString GenericBuilder::langExtDecl()
{
        return _lgg_ext_decl;
}

QString GenericBuilder::langExtImpl()
{
        return _lgg_ext_impl;
}

bool GenericBuilder::hasDeclarationAndImplementationFiles()
{
    return _bHasImplementationFile;
}

QString GenericBuilder::openTemplate(const QString &templateName)
{
    QString templ;
    QFile file (templateName);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qCritical() << lang() << ": Could not open file "+ templateName;
        return templ;
    }

    QTextStream in(&file);
    templ.append(in.readAll());
    return templ;
}

QString GenericBuilder::BuildOutDir()
{
    // ../outFiles/entity/...
    QString outDirName("outFiles");
    return QString( outDirName + QDir::separator() + _entity.name() + QDir::separator() );
}

void GenericBuilder::CheckForOutDir()
{
    QString myPath = QDir::currentPath();
    QString myOutDir = BuildOutDir();
    QDir myDir(myPath) ;

    if( !myDir.exists(myOutDir) )
        myDir.mkpath(myOutDir);

    if( !myDir.exists(myOutDir) )
        throw new std::exception();
}

QFile* GenericBuilder::declarationFile()
{
    CheckForOutDir();
    QFile *f = new QFile( BuildOutDir() + declarationFileSuffix() );
    return f;
}

QFile * GenericBuilder::implementationFile()
{
    CheckForOutDir();
    QFile *f = new QFile( BuildOutDir() + implementationFileSuffix() );
    return f;
}


void GenericBuilder::moreTemplatesOnDeclarationFile(QString & /*templ*/)
{
}

void GenericBuilder::moreTemplatesOnImplementationFile(QString & /*templ*/)
{
}

void GenericBuilder::toFile()
{
    // create and write the .hpp or .whatever declaraton file
    writeDeclarationFile();

    // create and write the implementation file if needed (indeed :java and c# only have declaration files)
    if(hasDeclarationAndImplementationFiles()) {
        writeImplementationFile();
    }
}

/**
  * The Template file : stocked in ./{lgg_Name}/{buildertype}/class
  * lgg_name is the name of the currently used langage : given by lang()
  * buildertype is the type of generator to create ( = entity if classBuilder || = service if serviceBuilder )
  * get the class template according to the langage and read it
  */
QString GenericBuilder::getTemplate()
{
    // the out string
    QString templ;

    // open the template file
    QFile file (getDeclarationFileTemplate());
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qCritical() << lang() << ": Could not open file " << getDeclarationFileTemplate();
        return templ;
    }

    // get a stream to the template file
    QTextStream in(&file);

    // push it to the out string
    templ.append(in.readAll());
    return templ.trimmed();
}
