#ifndef GENERICBUILDER_HPP
#define GENERICBUILDER_HPP
#include <QList>
#include <QFile>
#include <QTextStream>

#include "Entity.hpp"

class GenericBuilder
{
public:
    GenericBuilder(Entity &e, QString lang, QString declarationExtention, QString implementationExtention = "" );

    //virtuales
    /// Writes the class to files
    virtual void toFile ();

protected :
    Entity entity();

    /// set the langage name, and extentions
    void setLangInfo(QString, QString, QString = "");
    /// return the langage name
    QString lang();
    /// return the extention used for declaration files
    QString langExtDecl();
    /// return the extention used for implementation files
    QString langExtImpl();

    /// retrieve the (default) namespace ("DoQuBook")
    QString getNamespace();

    /// permit to know if the langage needs an implementation file
    bool hasDeclarationAndImplementationFiles();

    /// open and read a template file
    QString openTemplate(const QString &templateName);

    // Utility functions
    QString classNameUpper();
    QString classNameLower();

    // vpures

    /// Writes the class implemantation (.cpp for example)
    virtual void writeImplementationFile() = 0;
    /// Writes the class declaration (.hpp for example)
    virtual void writeDeclarationFile() = 0;

    /// build the output path
    QString BuildOutDir();
    /// ensure the Out Dir exists
    void CheckForOutDir();
    /// return the declaration file for output
    QFile * declarationFile();
    /// return the implementation file for output
    QFile * implementationFile();


    /// get the declaration file suffix specific to each subclass
    virtual QString declarationFileSuffix() = 0;
    /// getthe implementation file suffix specific to each subclass
    virtual QString implementationFileSuffix() = 0;

    /// return a path that point to the template declaration
    /// only this template have to be declared (the implementation one is facultative)
    virtual QString getDeclarationFileTemplate() = 0;


    //virtuals
    /// Re-implement this function for language specific stuff
    virtual void moreTemplatesOnDeclarationFile(QString & templ);
    /// Re-implement this function for language specific stuff
    virtual void moreTemplatesOnImplementationFile(QString & templ);

    /// read the template file
    virtual QString getTemplate();

private :
    // langage name
    QString _lgg_name;
    //langage extention for declaration files
    QString _lgg_ext_decl;
    //langage extention for inmplementation files
    QString _lgg_ext_impl;

    // the entity
    Entity &_entity;

    // true if the used langage uses an implementation file and a declaration one, else false (ex :java and c#)
    bool _bHasImplementationFile;
};

#endif // GENERICBUILDER_HPP
