#include "ServiceBuilder.hpp"
#include <QFile>
#include <QTextStream>
#include <QDebug>

ServiceBuilder::ServiceBuilder(Entity &e, QString lang, QString declarationExtention, QString implementationExtention)
    : GenericBuilder(e, lang, declarationExtention, implementationExtention)
    , _crudmethods (false, false, false, false)
{
}


void ServiceBuilder::writeDeclarationFile()
{
    QString templ = getDeclarationFileTemplate();

    QFile * file = declarationFile();

    // error checkin'
    if (!file->open(QIODevice::WriteOnly | QIODevice::Text))
        return;

    QTextStream out(file);

    QString getMethodsTempl = replaceMethod(crudMethods().read(), ":/"+lang()+"/service/getmethods");
    templ.replace("${GET_METHODS}", getMethodsTempl);

    QString delMethodsTempl = replaceMethod(crudMethods().del(), ":/"+lang()+"/service/delmethods");
    templ.replace("${DEL_METHODS}", delMethodsTempl);

    QString updateMethodsTempl = replaceMethod(crudMethods().update(), ":/"+lang()+"/service/updatemethods");
    templ.replace("${UPDATE_METHODS}", updateMethodsTempl);

    QString addMethodsTempl = replaceMethod(crudMethods().create(), ":/"+lang()+"/service/addmethods");
    templ.replace("${ADD_METHODS}", addMethodsTempl);


    QListIterator<Field> it (entity().fields());
    bool trouve = false;
    QString none("none");
    Field foundF (none, none);
    while(it.hasNext() && !trouve) {
        foundF = it.next();
        trouve = foundF.isId();
    }
    if(trouve)
        templ.replace("${ID_TYPE}", foundF.type());

    templ.replace("${SERVICE_NAME}", serviceName());
    templ.replace("${ENTITY_NAME}", entity().name());
    templ.replace("${SERVICE_NAME_UPPER}", serviceName().toUpper());
    templ.replace("${ENTITY_NAME_LOWER}", entity().name().toLower());

    // more replacement if specificly needed
    moreTemplatesOnDeclarationFile(templ);

    out << templ;

    if(file) delete file; file = NULL;
}

void ServiceBuilder::writeImplementationFile()
{
    // Open the implementation file
    QFile * file = implementationFile();
    if (!file->open(QIODevice::WriteOnly | QIODevice::Text))
        return;

    // get a stream to the file
    QTextStream out(file);

    // retrieve the classe implementation template
    QString templ = getImplementationFileTemplate();


    QString getMethodsTempl = replaceMethod(crudMethods().read(), ":/"+lang()+"/service/getmethods_impl");
    templ.replace("${GET_METHODS}", getMethodsTempl);

    QString delMethodsTempl = replaceMethod(crudMethods().del(), ":/"+lang()+"/service/delmethods_impl");
    templ.replace("${DEL_METHODS}", delMethodsTempl);

    QString updateMethodsTempl = replaceMethod(crudMethods().update(), ":/"+lang()+"/service/updatemethods_impl");
    templ.replace("${UPDATE_METHODS}", updateMethodsTempl);

    QString addMethodsTempl = replaceMethod(crudMethods().create(), ":/"+lang()+"/service/addmethods_impl");
    templ.replace("${ADD_METHODS}", addMethodsTempl);

    QListIterator<Field> it (entity().fields());
    bool trouve = false;
    QString none("none");
    Field foundF (none, none);
    while(it.hasNext() && !trouve) {
        foundF = it.next();
        trouve = foundF.isId();
    }
    if(trouve)
        templ.replace("${ID_TYPE}", foundF.type());

    templ.replace("${SERVICE_NAME}", serviceName());
    templ.replace("${ENTITY_NAME}", entity().name());
    templ.replace("${SERVICE_NAME_UPPER}", serviceName().toUpper());
    templ.replace("${ENTITY_NAME_LOWER}", entity().name().toLower());


    // Do more replacements if needed
    moreTemplatesOnImplementationFile(templ);

    // push the stream content to the file
    out << templ;

    if(file) delete file; file = NULL;
}

QString ServiceBuilder::replaceMethod(bool b, QString pathToTemplate)
{
    if(b)
        return openTemplate(pathToTemplate);
    else
        return "";
}

QString ServiceBuilder::getDeclarationFileTemplate()
{
    QString templ = openTemplate(":/"+ lang() +"/service/declaration");
    return templ;
}

QString ServiceBuilder::getImplementationFileTemplate()
{
    QString templ = openTemplate(":/"+ lang() +"/service/implementation");
    return templ;
}

void ServiceBuilder::setNamesAndCrudMethods(QString serviceName, QString entityName, CRUDMethods crudmethods)
{
    setName(serviceName);
    setEntityName(entityName);
    setCrudMethods(crudmethods);
}

void ServiceBuilder::setName(const QString &name)
{
    _name = name;
}

void ServiceBuilder::setCrudMethods(const CRUDMethods &crudmethods)
{
    _crudmethods = crudmethods;
}

void ServiceBuilder::setEntityName(const QString &name)
{
    entity().setName(name);
}

QString ServiceBuilder::serviceName()
{
    return _name;
}


CRUDMethods ServiceBuilder::crudMethods()
{
    return _crudmethods;
}


QString ServiceBuilder::declarationFileSuffix()
{
    QString f ( serviceName() + langExtDecl() );
    return f;
}

QString ServiceBuilder::implementationFileSuffix()
{
    QString f ( serviceName() + langExtImpl() );
    return f;
}


