#ifndef SERVICEBUILDER_HPP
#define SERVICEBUILDER_HPP

#include <QFile>

#include "GenericBuilder.hpp"
#include "Entity.hpp"


class ServiceBuilder : public GenericBuilder
{
public:
    ServiceBuilder(Entity &e, QString lang, QString declarationExtention, QString implementationExtention = "" );

    void setNamesAndCrudMethods(QString serviceName, QString entityName, CRUDMethods crudmethods);

protected :
    void setName (const QString &serviceName);
    void setCrudMethods (const CRUDMethods  &crudmethods);
    void setEntityName (const QString &serviceName);

    /// return name
    QString serviceName();

    /// return CRUDMethods object
    CRUDMethods crudMethods();

    /// open the content of the template file for the creation of the declaration file
    QString openTemplateDeclaration();

    /// build the implementation file
    virtual void writeImplementationFile();

    /// Writes the class declaration (.hpp for example)
    virtual void writeDeclarationFile();

    virtual QString declarationFileSuffix();
    virtual QString implementationFileSuffix();

    /// template of the declaration file
    virtual QString getDeclarationFileTemplate();

    /// template of the implementation file
    virtual QString getImplementationFileTemplate();

    // return the text to use to replace a method
    QString replaceMethod(bool b, QString pathToTemplate);
private :
    QString _name;
    CRUDMethods _crudmethods;
};

#endif // SERVICEBUILDER_HPP
