
#voir http://www.cmake.org/cmake/help/syntax.html
macro(hello ARG1)
    message(${ARG1})
endmacro(hello)

############################################################################
# Documentation avec Doxygen
############################################################################
macro(doxygen PROJECT_NAME)
FIND_PACKAGE(Doxygen)

IF (DOXYGEN_FOUND AND EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/Doxyfile.in)


SET(OUTPUT_DIRECTORY   ${CMAKE_CURRENT_SOURCE_DIR}/doc)
SET(INPUT ${CMAKE_CURRENT_SOURCE_DIR})
SET(OUTPUT_LANGUAGE English)
SET(BUILD_DIRS "${CMAKE_CURRENT_BINARY_DIR} tests")

CONFIGURE_FILE(${CMAKE_CURRENT_SOURCE_DIR}/Doxyfile.in
               ${CMAKE_CURRENT_BINARY_DIR}/Doxyfile
               @ONLY)

ADD_CUSTOM_TARGET(doc
 COMMAND ${CMAKE_COMMAND} -E echo_append "Building API Documentation..."
 COMMAND ${DOXYGEN_EXECUTABLE} ${CMAKE_CURRENT_BINARY_DIR}/Doxyfile
 COMMAND ${CMAKE_COMMAND} -E echo "Done."
 WORKING_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
)

ENDIF (DOXYGEN_FOUND AND EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/Doxyfile.in)
endmacro(doxygen)

###### Creates a DoQuBook_Plugin (with Qt and DoQuBook)
macro(doqubook_plugin NAME HEADERS MOC_HEADERS SOURCES)

# some cmake conf stuff
SET(CMAKE_ALLOW_LOOSE_LOOP_CONSTRUCTS TRUE)


# name of the project and target.
PROJECT(${NAME})

############################################
### lets find the required libs.

#Qt
find_package(Qt4 REQUIRED)
INCLUDE(${QT_USE_FILE})

#DoQubook
find_package(DoQuBook REQUIRED)

INCLUDE_DIRECTORIES(${DoQuBook_INCLUDE_DIRS} ${QT_QTCORE_INCLUDE_DIR} ${QT_QTGUI_INCLUDE_DIR} ${QT_QTOPENGL_INCLUDE_DIR} ${QT_QTDESIGNER_INCLUDE_DIR})
INCLUDE_DIRECTORIES(${DoQuBook_INCLUDE_DIRS})

############################################
# Add some definitions
add_definitions(${QT_DEFINITIONS})
add_definitions(-DQT_PLUGIN)
add_definitions(-DQT_SHARED)
add_definitions(-DQT_NO_DEBUG)

############################################
# prepare Qt sources
QT4_WRAP_CPP(MOC_SOURCES ${MOC_HEADERS})


############################################
SET(ALL_SOURCES ${SOURCES} ${MOC_SOURCES} ${MOC_HEADERS} ${HEADERS})

############################################
# the target
ADD_LIBRARY(${NAME} SHARED ${ALL_SOURCES})

SET_TARGET_PROPERTIES(${NAME} PROPERTIES INSTALL_NAME_DIR "@loader_path/../Frameworks")
SET_TARGET_PROPERTIES( ${NAME} PROPERTIES DEFINE_SYMBOL "DoQuBookSHARED_EXPORT" )


TARGET_LINK_LIBRARIES(${NAME} ${DoQuBook_LIBRARIES} ${QT_LIBRARIES})

############################################
# installation rules
set(DoQuBook_PLUGIN_DEST /Library/Frameworks/DoQuBook.framework/PlugIns)
INSTALL(TARGETS ${NAME}
#  FRAMEWORK DESTINATION Library/Frameworks
  RUNTIME DESTINATION ${DoQuBook_PLUGIN_DEST}
  LIBRARY DESTINATION ${DoQuBook_PLUGIN_DEST}
  ARCHIVE DESTINATION ${DoQuBook_PLUGIN_DEST}
)

############################################
doxygen(${NAME})

endmacro(doqubook_plugin)


## ICONE filename but not the path [path must be CMAKE_CURRENT_SOURCE_DIR]
macro(doqubook_app NAME ICONE RESOURCES SOURCES)

SET(CMAKE_CXX_FLAGS "-g -Wuninitialized -O")

############################################
# find the libs

#QJSon
find_package(QJSON REQUIRED)
SET(LIBS ${LIBS} ${QJSON_LIBRARIES})
INCLUDE_DIRECTORIES(${QJSON_INCLUDE_DIR})

#DoQuBook
FIND_PACKAGE(DoQuBook REQUIRED)

INCLUDE_DIRECTORIES(${DoQuBook_INCLUDE_DIRS})
include_directories(${QT_QTNETWORK_INCLUDE_DIR})

#FIND_LIBRARY(GROWLCPP GrowlCpp)

SET(LIBS ${LIBS} ${QT_LIBRARIES} ${DoQuBook_LIBRARIES} ${QT_QTXML_LIBRARY} ${QT_QTNETWORK_LIBRARY} ${GROWLCPP})


###########################################
#Preparation des sources
QT4_ADD_RESOURCES( APP_RESOURCES ${RESOURCES} )



SET(ALL_SOURCES ${SOURCES} ${APP_RESOURCES})

############################################
# The icon
# Pour le bundle osx on ajoute l'icone
IF(APPLE)
# set how it shows up in the Info.plist file
message (ICONE)
message(${CMAKE_CURRENT_SOURCE_DIR}/${ICONE})
SET(MACOSX_BUNDLE_ICON_FILE ${ICONE})
# set where in the bundle to put the icns file
SET_SOURCE_FILES_PROPERTIES(${CMAKE_CURRENT_SOURCE_DIR}/${ICONE} PROPERTIES MACOSX_PACKAGE_LOCATION Resources)
#INSTALL(FILES ${CMAKE_CURRENT_SOURCE_DIR}/${ICONE} DESTINATION  MACOSX_PACKAGE_LOCATION COMPONENT Resources)
# include the icns file in the target
SET(ALL_SOURCES ${ALL_SOURCES} ${CMAKE_CURRENT_SOURCE_DIR}/${ICONE})
ENDIF(APPLE)

############################################
# the exe
message(LES SOURCES)
message(${ALL_SOURCES})
ADD_EXECUTABLE(${NAME} MACOSX_BUNDLE
        ${ALL_SOURCES}
)


TARGET_LINK_LIBRARIES(${NAME} ${LIBS})


#--------------------------------------------------------------------------------
# def des variable necesaires :
# APPS : est utilisÃ© pour CPack
SET(plugin_dest_dir bin)
SET(qtconf_dest_dir bin)
SET(APPS "\${CMAKE_INSTALL_PREFIX}/bin/${NAME}")
IF(APPLE)
SET(plugin_dest_dir ${NAME}.app/Contents)
SET(qtconf_dest_dir ${NAME}.app/Contents/Resources)
SET(APPS "\${CMAKE_INSTALL_PREFIX}/${NAME}.app")
ENDIF(APPLE)
IF(WIN32)
SET(APPS "\${CMAKE_INSTALL_PREFIX}/bin/${NAME}.exe")
ENDIF(WIN32)


#--------------------------------------------------------------------------------
# Installation de QtTest, sure apple, le bundle se trouve Ã  la racine,
# sur les autres plate-forme dans bin
INSTALL(TARGETS ${NAME}
BUNDLE DESTINATION . COMPONENT Runtime
RUNTIME DESTINATION bin COMPONENT Runtime
)

#--------------------------------------------------------------------------------
# On copie le dossier de plugins QT dans le bundle courant

INSTALL(DIRECTORY "${QT_PLUGINS_DIR}/imageformats" DESTINATION ${plugin_dest_dir}/plugins COMPONENT Runtime)


if(APPLE)
set(DoQuBook_plugins /Library/Frameworks/DoQuBook.framework/PlugIns/)
INSTALL(DIRECTORY ${DoQuBook_plugins} DESTINATION ${plugin_dest_dir}/DoQuBook_plugins COMPONENT Runtime)
endif(APPLE)
message(DoQuBookPlugins_DIR = ${DoQuBook_plugins})

#--------------------------------------------------------------------------------
# installation du qt.conf
# on insert du code cmake pour Ã©crire le fichier dans le script d'installation

INSTALL(CODE "
file(WRITE \"\${CMAKE_INSTALL_PREFIX}/${qtconf_dest_dir}/qt.conf\" \"\")
" COMPONENT Runtime)

#--------------------------------------------------------------------------------
# On copie le qt_menu.nib de QT dans le bundle courant
IF(APPLE)
install (DIRECTORY ${QT_LIBRARY_DIR}/QtGui.framework/Resources/qt_menu.nib DESTINATION ${NAME}.app/Contents/Resources)
#--------------------------------------------------------------------------------
# Grace a BundleUtilities on peux copier les libs nÃ©cessaire dans la partie
# Frameworks du bundle

# Répèrtoire à regarder pour les dépendance
SET(DIRS ${QT_LIBRARY_DIRS} ${LIBS} /usr/local/lib/)

# on insert du code cmake pour utiliser BundleUtilities dans le script d'installation
INSTALL(CODE "
file(GLOB_RECURSE QTPLUGINS
\"\${CMAKE_INSTALL_PREFIX}/${plugin_dest_dir}/plugins/*${CMAKE_SHARED_LIBRARY_SUFFIX}\")
file(GLOB_RECURSE DOQUBOOKPLUGINS
\"\${CMAKE_INSTALL_PREFIX}/${plugin_dest_dir}/DoQuBook_plugins/*${CMAKE_SHARED_LIBRARY_SUFFIX}\")
include(BundleUtilities)
SET(PLUGINS \${QTPLUGINS} \${DOQUBOOKPLUGINS})
fixup_bundle(\"${APPS}\" \"\${PLUGINS}\" \"${DIRS}\")
" COMPONENT Runtime)

endif(APPLE)

# on utilise cpack : "cpack -G DragNDrop CPackConfig.cmake" sur mac os x
# le CPackConfig.cmake est crÃ©er par un premier cmake
# Command osx /Applications/CMake\ 2.8-1.app/Contents/bin/cpack -G DragNDrop CPackConfig.cmake
SET(CPACK_GENERATOR "DragNDrop")
#set(CPACK_BINARY_DRAGNDROP ON)
include(CPack)

endmacro(doqubook_app)
